<?php

namespace App\Http\Controllers;

use App\Models\ActivityType;
use App\Models\OptStudentCriteriaGrade;
use App\Models\Period;
use App\Models\SchoolYear;
use App\Models\Subperiod;
use Illuminate\Http\Request;
use App\Repositories\SubjectPeriodActivityTypeRepository;
use App\Repositories\StudentCriteriaGradeRepository;
use App\Repositories\SubjectCourseRepository;
use App\Repositories\GlobalRepository;
use App\Repositories\QualitativeEquivalenceRepository;
use App\Repositories\OptSubjectPeriodActivityTypeRepository;
use App\Repositories\OptStudentCriteriaGradeRepository;
use App\Repositories\OptionalSubjectRepository;
use App\Repositories\PeriodRepository;
use App\Repositories\SchoolYearRepository;
use App\Repositories\ActivityRepository;
use App\Repositories\OptActivityRepository;
use Laravel\Lumen\Routing\Controller as BaseController;


class ActivityController extends BaseController {

    protected $subjectPeriodActivityType;
    protected $studentCriteriaGrade;
    protected $subjectCourse;
    protected $global;
    protected $qualitativeEquivalence;
    protected $optSubjectPeriodActivityType;
    protected $optStudentCriteriaGrade;
    protected $period;
    protected $schoolYear;
    protected $activity;
    protected $optionalSubject;

    public function __construct(SubjectPeriodActivityTypeRepository $subjectPeriodActivityType,StudentCriteriaGradeRepository $studentCriteriaGrade,SubjectCourseRepository $subjectCourse, GlobalRepository $global, QualitativeEquivalenceRepository $qualitativeEquivalence, OptSubjectPeriodActivityTypeRepository $optSubjectPeriodActivityType,
                                OptStudentCriteriaGradeRepository $optStudentCriteriaGrade, PeriodRepository $period, SchoolYearRepository $schoolYear, ActivityRepository $activity,OptionalSubjectRepository $optionalSubject, OptActivityRepository $optActivity)
    {
        $this->subjectPeriodActivityType = $subjectPeriodActivityType;
        $this->studentCriteriaGrade = $studentCriteriaGrade;
        $this->subjectCourse = $subjectCourse;
        $this->global = $global;
        $this->qualitativeEquivalence = $qualitativeEquivalence;
        $this->optSubjectPeriodActivityType = $optSubjectPeriodActivityType;
        $this->optStudentCriteriaGrade = $optStudentCriteriaGrade;
        $this->period = $period;
        $this->schoolYear = $schoolYear;
        $this->activity = $activity;
        $this->optionalSubject = $optionalSubject;
        $this->optActivity = $optActivity;
    }

    /**
     * Método que devuelve las actividades por subperiodo
     * @since   2018-06-08
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   integer $serial serial de la materia (osb o serial_sbc)
     * @param   integer $serial_sbp serial del subperiodo
     * @param   integer $serial_std serial del estudiante
     * @param   integer $serial_att serial del tipo de actividad
     * @param   string $type tipo de materia obligatoria u optativa
     * @return  object $subject
     *
     * @edit    2018-06-08<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getActivitiesBySubperiodByStudent($serial, $serial_sbp, $serial_std, $serial_att,$type)
    {
        $subperiod = Subperiod::find($serial_sbp);
        $period = Period::find($subperiod->serial_prd);
        $periodGroupYear = $period->periodGroupYear;
        $schoolYear = SchoolYear::find($periodGroupYear->serial_scy);
        $activityType = ActivityType::find($serial_att);
        if ($type == "MANDATORY") {
            $subject = $this->subjectCourse->getSubjectCourse($serial);
            $subject->activity_type = $activityType->name_att;
            $activities = $this->studentCriteriaGrade->getActivitiesByActivityTypeBySubperiod($serial, $subperiod->serial_prd, $serial_sbp, $periodGroupYear->serial_scy, $serial_std, $serial_att);
            foreach ($activities as $activity) {
                if ($activity != null && is_numeric($activity->act_grade)) {
                    $activity->act_grade = number_format($this->global->truncate($activity->act_grade, $schoolYear->grade_precision_scy), $schoolYear->grade_precision_scy);
                    if ($subject->grade_type == 'QUALITATIVE') {
                        $activity->act_grade = $this->qualitativeEquivalence->convertQuantitativeToQualitative($activity->act_grade, $schoolYear->serial_scy);
                    }
                }
            }

            $subject->activities = $activities;
            return response()->json($subject);
        } elseif ($type == "OPTATIVE") {
            $subject = $this->optionalSubject->getOptionalSubject($serial);
            $subject->activity_type = $activityType->name_att;
            $activities = $this->optStudentCriteriaGrade->getActivitiesByActivityTypeBySubperiod($serial, $subperiod->serial_prd, $serial_sbp, $schoolYear->serial_scy, $serial_std, $serial_att);
            foreach ($activities as $activity) {
                if ($activity != null && is_numeric($activity->act_grade)) {
                    $activity->act_grade = number_format($this->global->truncate($activity->act_grade, $schoolYear->grade_precision_scy), $schoolYear->grade_precision_scy);
                    if ($subject->grade_type == "QUALITATIVE") {
                        $activity->act_grade = $this->qualitativeEquivalence->convertQuantitativeToQualitative($activity->act_grade, $schoolYear->serial_scy);
                    }
                }
            }
            $subject->activities = $activities;
            return response()->json($subject);
        }
    }

    /**
     * Método que devuelve los promedios de las actividades por tipo de actividad por subperiodo
     * @since   2018-06-08
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   integer $serial serial de la materia (osb o serial_sbc)
     * @param   string $type tipo de materia obligatoria u optativa
     * @param   integer $serial_prd serial del periodo
     * @param   integer $serial_sbp serial del subperiodo
     * @param   integer $serial_scy serial del año lectivo
     * @param   integer $serial_std serial del estudiante
     * @param   integer $serial_att serial del tipo de actividad
     * @return  Listobject $activities actividades por tipo de actividad por subperiodo
     *
     * @edit    2018-06-08<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getAveragesByActivityTypeBySubperiod($serial, $type, $serial_prd, $serial_sbp, $serial_scy, $serial_std, $serial_att)
    {
        if ($type == "MANDATORY") {
            $activities = $this->studentCriteriaGrade->getActivitiesByActivityTypeBySubperiod($serial, $serial_prd, $serial_sbp, $serial_scy, $serial_std, $serial_att);

        } else {
            $activities = $this->optStudentCriteriaGrade->getActivitiesByActivityTypeBySubperiod($serial, $serial_prd, $serial_sbp, $serial_scy, $serial_std, $serial_att);
        }
    }


    public function getActivitiesByPeriodByStudent($serial_std, $serial_act = null)
    {
        $activities = $this->activity->getActivitiesByStudent($serial_std, $serial_act);
        return response()->json($activities);
    }



    /**
     * Método para obtener el detalle de las actividades
     * @since   2018-07-27
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $request
     *
     * @return  Listobject $activities
     *
     */
    public function getActivityDetails(Request $request)
    {

        $this->validate($request, [
            'serial_act' => 'required',                
            'optative' => 'required',               
        ]);


        $optative = $request['optative'];


        if ($optative == 'NO') {
            $message = $this->activity->getActivityDetails($request);
        } else if ($optative == 'YES') {
            $message = $this->optActivity->getOptActivityDetails($request);
        }            

        if (is_array($message)){
            $old_format_date_send = explode('-', $message['send_date']);
            $old_format_date_due = explode('-', $message['due_date']);
            $array_days = array('Monday' => 'Lunes', 'Tuesday' => 'Martes', 'Wednesday' => 'Miércoles', 'Thursday' => 'Jueves', 'Friday' => 'Viernes', 'Saturday' => 'Sabado', 'Sunday' => 'Domingo');
            $array_month = array('January' => 'Enero', 'February' => 'Febrero', 'March' => 'Marzo', 'April' => 'Abril', 'May' => 'Mayo', 'June' => 'Junio', 'July' => 'Julio', 'August' => 'Agostro', 'September' => 'Septiembre', 'October' => 'Octubre', 'November' => 'Noviembre', 'December' => 'Diciembre');
            $number_month = array('January' => '01', 'February' => '02', 'March' => '03', 'April' => '04', 'May' => '05', 'June' => '06', 'July' => '07', 'August' => '08', 'September' => '09', 'October' => '10', 'November' => '11', 'December' => '12');
            $new_format_date_send = $array_days[$old_format_date_send[0]] . ', ' . $old_format_date_send[1] . ' de ' . $array_month[$old_format_date_send[2]] . ', ' . $old_format_date_due[3];
            $new_format_date_due = $array_days[$old_format_date_due[0]] . ', ' . $old_format_date_due[1] . ' de ' . $array_month[$old_format_date_due[2]] . ', ' . $old_format_date_due[3];
            $message['send_date'] = $new_format_date_send;
            $message['due_date'] = $new_format_date_due;
            $message['due_day'] = $old_format_date_due[1];
            $message['due_month'] = $number_month[$old_format_date_due[2]];
            $message['due_year'] = $old_format_date_due[3];

            return response()->json($message);
        }else{
            return response()->json(false);
        }
    }

}