<?php

namespace App\Http\Controllers;


use App\Models\SubjectPlanningUnit;
use App\Repositories\OptionalSubjectRepository;
use App\Repositories\SubjectPlanningUnitRepository;
use Illuminate\Http\Request;
use App\Models\SchoolSetup;
use App\Repositories\CourseRepository;
use App\Repositories\SchoolYearRepository;
use App\Repositories\UserRepository;
use Laravel\Lumen\Routing\Controller as BaseController;


class CourseController extends BaseController {

    protected $course;
    protected $schoolYear;
    protected $user;
    protected $optionalSubject;
    protected $subjectPlanningUnit;

    public function __construct(CourseRepository $course, SchoolYearRepository $schoolYear, UserRepository $user, OptionalSubjectRepository $optionalSubject, SubjectPlanningUnitRepository $subjectPlanningUnit) {
        $this->course = $course;
        $this->schoolYear = $schoolYear;
        $this->user = $user;
        $this->optionalSubject = $optionalSubject;
        $this->subjectPlanningUnit = $subjectPlanningUnit;


    }

    /**
     * Método que devuelve el la lista de estudiantes del curso
     * @since   2017-08-03
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   array $request serial del curso
     * @return  ListObject $students
     * 
     * @edit    2017-08-03<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function postStudentsByCourse(Request $request) {
        $this->validate($request, [
            'serial_crs' => 'required'
        ]);
        $students = $this->course->getStudents($request->input('serial_crs'));

        return response()->json($students);
    }


    /**
     * Método que devuelve curso de estudiante por serial_std y anio
     * @since   2018-04-02
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $serial_std serial del estudiante
     * @param   array $serial_scy serial del anio
     * @return  ListObject $course
     * 
     * @edit    2018-04-02<br />
     *          Erick Vaca <epvaca@rutatec.com> <br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getCourseByStudentYear($serial_std, $serial_scy) {

        $course = $this->course->getCourseByStudentYear($serial_std, $serial_scy);

        return response()->json($course);
    }
    /**
     * Método que devuelve los compañeros de curso de un estudiante con la foto
     * por serial del estudiante.
     * @since   2018-06-20
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @param   integer $serial_std
     * @return  json $students
     * @edit    2018-06-25
     *          Henry Vivanco <hvvivanco@rutatec.com>
     *          Agregar el serial del estudiante.
     *
     */
    public function getStudentCompanionBySerialStd($serial_std)
    {
        $schoolYear = $this->schoolYear->getCurrentSchoolYear();
        $course = $this->course->getCourseByStudentYear($serial_std, $schoolYear->serial_scy);
        $students = $this->course->getStudents($course->serial_crs);
        $schoolSetup = SchoolSetup::first();
        $stds = array();
        foreach ($students as $student) {
            $photo_usr = env('SERVER_LEARNBOX') . 'img/users/';
            if ($student->photo_usr != null) {
                $student->photo_usr = $photo_usr . $student->photo_usr;
            } else {
                $student->photo_usr = $photo_usr . 'noavatar.png';
            }
            $student = [
                'serial_std' => $student->serial_std,
                'first_name_std' => $student->first_name_std,
                'last_name_std' => $student->last_name_std,
                'photo_usr' => $student->photo_usr
            ];
            array_push($stds, $student);
        }
        return response()->json($stds);
    }

    public function getAcademicArea($serial_scy,$serial_set,$serial_lvl){
        $academicAreas = $this->course->getAcademicArea($serial_scy,$serial_set,$serial_lvl);
        $optionalAcademicAreas = $this->optionalSubject->getAcademicArea($serial_scy,$serial_set,$serial_lvl);
        return response()->json($academicAreas);
    }
    public function getSubject($serial_scy,$serial_set,$serial_lvl,$serial_aca = false) {
        $subjects = $this->course->getCourseSubject($serial_scy,$serial_set,$serial_lvl,$serial_aca,'false');
        return $subjects;
    }

    public function getSubjectConfig($serial_scy,$serial_set,$serial_lvl,$serial_aca = false) {
        $subjects = $this->course->getCourseSubject($serial_scy,$serial_set,$serial_lvl,$serial_aca,'true');
        return $subjects;
    }

    public function getSubjectConfigInfo($serial_scy,$serial_set,$serial_lvl,$serial_aca) {
        $subjects = $this->course->getCourseSubjectInfo($serial_scy,$serial_set,$serial_lvl,$serial_aca);
        return $subjects;
    }

    public function getLevelsAcademicArea($serial_scy,$serial_aca){
        $levelsAcademicAreas = $this->course->getLevelsAcademicArea($serial_scy,$serial_aca);
        return $levelsAcademicAreas;
    }

    public function postUnitsPlanning(Request $request){
        $data = $request->all();
        $unitsPlanning = $this->course->getUnitsPlanning($data);
        foreach ($unitsPlanning as $unitPlanning) {
            if($unitPlanning->serial_syp != null){
            $subjectUnitPlanning = $this->subjectPlanningUnit->getUnitPlanningBySerialSyp($unitPlanning->serial_syp);
            $unitPlanning->block_planning = $subjectUnitPlanning;
            }
        }
        if(count($unitsPlanning)){
            return response()->json($unitsPlanning,200);
        } else {
            return response()->json('No se ha creado la unidad de planificación para este nivel',404);
        }
    }

    public function postUnitsPlanningWeek(Request $request){
        $data = $request->all();
        $unitsPlanning = $this->course->getUnitsPlanning($data);
        foreach ($unitsPlanning as $unitPlanning) {
            if($unitPlanning->serial_syp != null){
                $subjectUnitPlanning = $this->subjectPlanningUnit->getUnitPlanningWeekBySerialSyp($unitPlanning->serial_syp);
                $unitPlanning->block_planning = $subjectUnitPlanning;
            }
        }
        return $unitsPlanning;
    }
}
