<?php

namespace App\Http\Controllers;

use App\Models\SchoolSetup;
use App\Repositories\UserRepository;
use Illuminate\Http\Request;
use App\Repositories\EmployeeRepository;
use App\Repositories\ScheduleRepository;
use App\Repositories\OptionalSubjectRepository;
use App\Repositories\DateDayConfigurationRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

class EmployeeController extends BaseController {

    protected $employee;
    protected $optionalSubject;
    protected $schedule;
    protected $dateDayConfiguration;
    protected $user;

    public function __construct(EmployeeRepository $employee, OptionalSubjectRepository $optionalSubject, DateDayConfigurationRepository $dateDayConfiguration = null, UserRepository $user, ScheduleRepository $schedule) {
        $this->employee = $employee;
        $this->schedule = $schedule;
        $this->optionalSubject = $optionalSubject;
        $this->dateDayConfiguration = $dateDayConfiguration;
        $this->user = $user;
    }

    /**
     * Método para devolver los horarios y materias del docente
     * @since   2017-08-03
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   array $request serial del empleado y la fecha seleccionada
     * @return  boolean true
     * 
     * @edit    2017-08-03<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function postSubjectsSchedulesEmployee(Request $request) {
        $this->validate($request, [
            'serial_emp' => 'required',
            'date' => 'required'
        ]);
        //se buscan las materias obligatorias y optativas del docente
        $optativeSubjects = $this->employee->getOptativeSubjectsAndSchedulesByEmployee($request->input('serial_emp'), $request->input('date'))->toArray();

        $optativeSubjectsDay = $this->employee->getOptativeSubjectsAndSchedulesByEmployeeDay($request->input('serial_emp'), $request->input('date'))->toArray();
        $mandatorySubjects = $this->employee->getMandatorySubjectsAndSchedulesByEmployee($request->input('serial_emp'), $request->input('date'))->toArray();
        $mandatorySubjectsDay = $this->employee->getMandatorySubjectsAndSchedulesByEmployeeDay($request->input('serial_emp'), $request->input('date'))->toArray();
        if (count($mandatorySubjectsDay) || count($optativeSubjectsDay)) {
            if (count($mandatorySubjectsDay)) {
                $schedule = $this->schedule->getScheduleByCourseDateDay($mandatorySubjectsDay[0]['serial_crs'], $request->input('date'), $mandatorySubjectsDay[0]['day_scd']);
                if (isset($schedule [0])) {
                    if ($mandatorySubjectsDay[0]['serial_scd'] == $schedule[0]['serial_scd']) {
                        $subjects = array_merge($mandatorySubjectsDay, $mandatorySubjects, $optativeSubjects);
                    } else {
                        $subjects = array_merge($mandatorySubjects, $optativeSubjects);
                    }
                } else {
                    $subjects = array_merge($mandatorySubjects, $optativeSubjects);
                }
            } elseif (count($optativeSubjectsDay)) {
                $schedule = $this->schedule->getScheduleByCourseDateDay(false , $request->input('date'), $optativeSubjectsDay[0]['day_scd'], $optativeSubjectsDay[0]['serial_lop']);
                if (isset($schedule [0])) {
                    if ($optativeSubjectsDay[0]['serial_scd'] == $schedule[0]['serial_scd']) {
                        $subjects = array_merge($optativeSubjectsDay, $mandatorySubjects, $optativeSubjects);
                    } else {
                        $subjects = array_merge($mandatorySubjects, $optativeSubjects);
                    }
                } else {
                    $subjects = array_merge($mandatorySubjects, $optativeSubjects);
                }
            }
        } else {
            $subjects = array_merge($mandatorySubjects, $optativeSubjects);
        }
        return response()->json($subjects);
    }

    /**
     * Método para devolver los horarios y materias del docente con configuracion de incidencia
     * @since   2018-03-22
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   array $request serial del empleado y la fecha seleccionada
     * @return  boolean true
     * 
     * @edit    2018-04-26<br />
     *          Erick Vaca <epvaca@rutatec.com><br />
     *          Se agrego consulta de dia de fecha fecha seleccionada en base a configuracion de dias<br/>
     *          #edit1
     */
    public function postIncidenceSubjectsSchedulesEmployee(Request $request) {
        $this->validate($request, [
            'serial_emp' => 'required',
            'date' => 'required'
        ]);


        //Obtener dia a partir de la fecha de configuracion
        $dateDayConfig = $this->dateDayConfiguration->getDateDayConfiguration($request->input('date'));
        $day = $dateDayConfig['day_ddc'];

        //se buscan las materias obligatorias y optativas del docente
        $optativeSubjects = $this->employee->getIncidenceOptativeSubjectsAndSchedulesByEmployee($request->input('serial_emp'), $request->input('date'), $day)->toArray();
        //Filtrado de horarios repetidos de optativa
        if (!empty($optativeSubjects)) {
            $serialsCdt = array();
            foreach ($optativeSubjects as $k => $sbClass) {
                if (!in_array($sbClass['serial_cdt'], $serialsCdt)) {
                    $serialsCdt[] = $sbClass['serial_cdt'];
                } else {
                    unset($optativeSubjects[$k]);
                }
            }
        }
        $mandatorySubjects = $this->employee->getIncidenceMandatorySubjectsAndSchedulesByEmployee($request->input('serial_emp'), $request->input('date'), $day)->toArray();
        $subjects = array_merge($mandatorySubjects, $optativeSubjects);

        //Solo reordenar cuando se haya hecho merge con optativas si existian. Esto porq si se ordena a obligatorias desordena por curso
        if (!empty($optativeSubjects)) {
            //Ordenamiento de arreglo con merge por start_at_cdt
            usort($subjects, function($a, $b) {
                $a = strtotime($a['start_at_cdt']);
                $b = strtotime($b['start_at_cdt']);
                return $a - $b;
            });
        }
        return response()->json($subjects);
    }

    /**
     * Método que devuelve el empleado en base al usuario
     * @since   2018-03-22
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   array $serial_usr serial del usuario
     * @return  ListObject 
     * 
     */
    public function getEmployeeBySerialUsr($serial_usr) {
        $employee = $this->employee->getEmployeeBySerialUsr($serial_usr);
        return response()->json($employee);
    }

    /**
     * Método que retorna docentes por seccion, nivel , especialidad, anio lectivo
     * @since   2018-04-07
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   void
     * @return  ListObject
     * 
     */
    public function getTeachersBySchoolYearSectionLevelSpecialty(Request $request) {
        $this->validate($request, [
            'serial_scy' => 'required',
            'serial_set' => 'required',
            'serial_lvl' => 'required',
            'serial_spt' => 'required',
        ]);

        $employee = $this->employee->getTeachersBySchoolYearSectionLevelSpecialty($request->input('serial_scy'), $request->input('serial_set'), $request->input('serial_lvl'), $request->input('serial_spt'));

        if ($request->input('serial_incidence_emp')) {
            $permissions = $this->getIncidencePermissionByEmployeeSchoolYearArray($request->input('serial_incidence_emp'), $request->input('serial_scy'));
            if (!empty($permissions)) {
                foreach ($employee as $key => &$emp) {
                    $remove = true;

                    foreach ($permissions as $p) {


                        if ((!isset($p['serial_crs']) && $p['serial_set'] == $emp['serial_set'] && $p['serial_lvl'] == $emp['serial_lvl']) ||
                                (isset($p['serial_crs']) && $p['serial_crs'] == $emp['serial_crs'] ) ||
                                (!($emp['serial_crs']) && $p['serial_set'] == $emp['serial_set'] && $p['serial_lvl'] == $emp['serial_lvl'])) {
                            $remove = false;
                            break;
                        }
                    }
                    if ($remove) {
                        unset($employee[$key]);
                    }
                }
            }
        }

        //Filtrado de docentes duplicados
        $employee = array_values($employee); //Reordeno keys
        $employeeKeyAux = array_unique(array_column($employee, 'serial_emp'));
        $employee = array_intersect_key($employee, $employeeKeyAux);

        return response()->json($employee);
    }

    /**
     * Método para devolver los permisos de seccion y cursos de un empleado para incidencias para perfil INSPECTOR, MENTOR
     * @since   2018-04-08
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   $serial_emp serial del empleado
     * @param   $serial_scy serial del anio
     * @return  boolean true
     * 
     * @edit    2018-04-08<br />
     *          Erick Vaca <epvaca@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getIncidencePermissionByEmployeeSchoolYear($serial_emp, $serial_scy) {
        $permission = $this->getIncidencePermissionByEmployeeSchoolYearArray($serial_emp, $serial_scy);
        return response()->json($permission);
    }

    /**
     * Método para devolver los permisos de seccion y cursos de un empleado para incidencias para perfil INSPECTOR, MENTOR como array
     * @since   2018-04-08
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   $serial_emp serial del empleado
     * @param   $serial_scy serial del anio
     * @return  boolean true
     * 
     * @edit    2018-04-08<br />
     *          Erick Vaca <epvaca@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    private function getIncidencePermissionByEmployeeSchoolYearArray($serial_emp, $serial_scy) {
        //se buscan las materias obligatorias y optativas del docente
        $sectionEmployee = $this->employee->employeeSectionBySerialEmpSerialScyTypes($serial_emp, $serial_scy, ['DIRECTOR'])->toArray();
        $courseEmployee = $this->employee->courseEmployeeBySerialEmpSerialScyEmployeeTypes($serial_emp, $serial_scy, ['INSPECTOR', 'MENTOR'])->toArray();
        $permission = array_merge($sectionEmployee, $courseEmployee);
        return $permission;
    }

    /**
     * Método que devuelve informacion del empleado en base a serial_emp
     * @since   2018-07-19
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   array $serial_emp serial del empleado
     * @return  ListObject 
     * 
     */
    public function getEmployeeBySerialEmp($serial_emp) {
        $employee = $this->employee->getEmployeeBySerialEmp($serial_emp);
        $schoolSetup = SchoolSetup::first();
        $photo_usr = env('SERVER_LEARNBOX') . 'img/users/';
        if ($employee) {
            $emp = [
                'serial_emp' => $employee->serial_emp,
                'last_name_emp' => $employee->last_name_emp,
                'first_name_emp' => $employee->first_name_emp,
                'id_emp' => $employee->id_emp,
                'email_emp' => $employee->email_emp,
                'photo_usr' => $photo_usr . $employee->photo_usr
            ];
        } else {
            $emp = [];
        }



        return response()->json($emp);
    }

    /**
     * Método para obtener los docentes asignados a un nivel
     * por sección, año lectivo y usuario
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-29
     * @return json $employeeSection
     */
    public function getEmployeeLevel(Request $request) {

        try {
            $data = $request->all();
            $employeeSection = $this->user->getEmployeeLevel($data);
            return response()->json($employeeSection);
        } catch (Exception $ex) {
            return response()->json($ex);
        }
    }

}
