<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Repositories\EventRepository;
use App\Repositories\SchoolYearRepository;
use App\Repositories\SectionRepository;
use App\Repositories\LevelSectionYearRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

class EventController extends BaseController {

    protected $event;
    protected $schoolYear;
    protected $section;
    protected $levelSectionYear;

    public function __construct(EventRepository $event,SchoolYearRepository $schoolYear,SectionRepository $section,LevelSectionYearRepository $levelSectionYear) {
        $this->event = $event;
        $this->schoolYear=$schoolYear;
        $this->section=$section;
        $this->levelSectionYear=$levelSectionYear;
    }

     /**
     * Devuelve los datos de un evento
     *
     * @param   int $serial_event serial del evento
     * 
     * @return  json $event
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function getEvent($serial_event) {
        $event = $this->event->findById($serial_event);
        return response()->json($event);
    }
    
     /**
     * Devuelve los perfiles asignados de un evento
     *
     * @param   int $serial_event serial del evento
     * 
     * @return  json $profiles
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function getEventProfiles($serial_event){
        $profiles = $this->event->getEventProfiles($serial_event);
        return response()->json($profiles);
    }

     /**
     * Devuelve los datos de un evento
     *
     * @param   array $request
     * 
     * @return  json $event
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function postCreateEvent(Request $request) {
        $event = $this->event->saveEvent($request);
        return response()->json($event);
    }

    /**
     * Actualiza los datos de un evento
     *
     * @param   array $request
     * 
     * @return  json $event
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function postEditEvent(Request $request) {

        $event = $this->event->editEvent($request);
        return response()->json($event);
    }
    
    /**
     * Devuelve las secciones de un evento
     *
     * @param   int $serial_event serial del evento
     * 
     * @return  json $sections
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function getEventSections($serial_event) {
        $sections = $this->event->getEventSections($serial_event);
        return response()->json($sections);
    }

    public function getListsSectionsByDate($startDate, $endDate)
    {
        $schoolYear = $this->schoolYear->getSchoolYearByDate($startDate, $endDate);
        $serial_scy = null;
        if ($schoolYear != null) {
            $serial_scy = $schoolYear->serial_scy;
            $sections = $this->levelSectionYear->listsSectionsBySchoolYear($serial_scy);

            return response()->json($sections);
        }
        return response()->json(false);


    }

    public function getSectionsByDate($date)
    {
        $schoolYear = $this->schoolYear->getSchoolYearByDate($date, $date);
        $serial_scy = null;
        if ($schoolYear != null) {
            $serial_scy = $schoolYear->serial_scy;
            $sections = $this->section->getSectionBySchoolYear($serial_scy);
            return response()->json($sections);
        }
        return response()->json(false);


    }

}
