<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Repositories\IncidenceSetupRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

/**
 * Controlador para Incidencia Configuracion
 * @since   2018-03-17
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-17<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del controlador<br/>
 *          #edit1
 */
class IncidenceSetupController extends BaseController {

    protected $incidenceSetup;

   /**
     * Constructor
     * @since   2018-03-17
     * @param   IncidenceSetupRepository  $incidenceSetup repositorio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     */
    public function __construct(IncidenceSetupRepository $incidenceSetup){
        $this->incidenceSetup = $incidenceSetup;
    }



  /**
     * Método que guarda configuracion
     * @since   2018-03-17
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $request
     * @return  
     * 
     */
    public function postSaveIncidenceSetup(Request $request) {
        try {
            //Procesamiento
            $this->validate($request, [                
                'frequency_ise' => 'required'             
            ]);
            $result = $this->incidenceSetup->saveIncidenceSetup($request);
            
            return response()->json($result);    
        } catch (Exception $ex) {
            return false;
        }
    }


    /**
     * Método para obtener configuracion por anio
     * @since   2018-03-17
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @return  IncidenceSetup array
     * 
     */
    public function getIncidenceSetupByYear($serial_scy) {
        $categories = $this->incidenceSetup->incidenceSetupByYear($serial_scy);
        return response()->json($categories);
    }

}