<?php

namespace App\Http\Controllers;

use Exception;
use Illuminate\Http\Request;
use App\Repositories\IncidenceSetupRepository;
use App\Repositories\IncidenceStudentRepository;
use App\Repositories\CourseRepository;
use App\Repositories\OptionalSubjectRepository;
use App\Repositories\SubperiodDisciplineRepository;
use App\Repositories\DisciplineEquivalenceRepository;
use App\Repositories\SchoolYearRepository;
use App\Repositories\IncidenceStudentSummaryRepository;
use App\Repositories\IncidenceStudentDetailRepository;
use App\Repositories\DateDayConfigurationRepository;
use App\Repositories\SubperiodRepository;
use App\Repositories\ScheduleRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

/**
 * Controlador para Incidencia Estudiante
 * @since   2018-03-23
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-23<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del controlador<br/>
 *          #edit1
 */
class IncidenceStudentController extends BaseController {

    protected $incidenceSetup;
    protected $course;
    protected $incidenceStudent;
    protected $optionalSubject;
    protected $subperiodDiscipline;
    protected $disciplineEquivalence;
    protected $schoolYear;
    protected $incidenceStudentSummary;
    protected $incidenceStudentDetail;
    protected $subperiod;
    protected $dateDayConfiguration;
    protected $schedule;

   /**
     * Constructor
     * @since   2018-03-23
     * @param   IncidenceSetupRepository  $incidenceSetup repositorio
     * @param   IncidenceStudentRepository  $incidenceStudent repositorio
     * @param   CourseRepository  $course repositorio
     * @param   OptionalSubjectRepository  $optionalSubject repositorio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     */
   public function __construct(IncidenceSetupRepository $incidenceSetup, IncidenceStudentRepository $incidenceStudent, CourseRepository $course, OptionalSubjectRepository $optionalSubject, SubperiodDisciplineRepository $subperiodDiscipline, DisciplineEquivalenceRepository $disciplineEquivalence, SchoolYearRepository $schoolYear, IncidenceStudentSummaryRepository $incidenceStudentSummary, IncidenceStudentDetailRepository $incidenceStudentDetail, SubperiodRepository $subperiod, DateDayConfigurationRepository $dateDayConfiguration, ScheduleRepository $schedule){
    $this->incidenceSetup = $incidenceSetup;
    $this->incidenceStudent = $incidenceStudent;
    $this->course = $course;
    $this->optionalSubject = $optionalSubject;
    $this->subperiodDiscipline = $subperiodDiscipline;
    $this->disciplineEquivalence = $disciplineEquivalence;
    $this->schoolYear = $schoolYear;
    $this->incidenceStudentSummary = $incidenceStudentSummary;
    $this->incidenceStudentDetail = $incidenceStudentDetail;
    $this->subperiod = $subperiod;
    $this->dateDayConfiguration = $dateDayConfiguration;
    $this->schedule = $schedule;
}



  /**
     * Método que guarda incidencia de estudiante
     * @since   2018-03-23
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $request
     * @return  
     * 
     */
  public function postSaveIncidenceStudent(Request $request) {
    try {
            //Procesamiento
        $this->validate($request, [
            'serial_ica' => 'required',
            'serial_usr' => 'required',
            'serial_std' => 'required',
            'serial_sbp' => 'required',
            'date_ist' => 'required',
        ]);
        $result = $this->incidenceStudent->saveIncidenceStudent($request);

        return response()->json($result);    
    } catch (Exception $ex) {
        return false;
    }
}

  
       /**
     * Método que guarda incidencias de estudiante
     * @since   2018-03-27
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $request
     * @edit 1
     * 2018-04-27
     * Se agrega soporte detalle y general con subperiodo
     * @return  
     * 
     */
    public function postSaveStudentIncidences(Request $request) {
        try {
            //Procesamiento
            $this->validate($request, [
                'serial_usr' => 'required',
                'serial_std' => 'required',
                'serial_sbp' => 'required',
                'date_ist' => 'required',
                'serial_subject' => 'required',
                'type_subject' => 'required',
                'serial_scd' => 'required',                
                'serial_scy' => 'required',
                'serial_set' => 'required',
                'serial_lvl' => 'required',
                'serial_spt' => 'required',                
                'serial_ica' => 'required',
            ]);

            //Definicion de valores de materia
            if ($request['type_subject'] == 'Obligatoria') {
                $serial_sbc = $request['serial_subject'];
                $serial_osb = null;
            } else {
                $serial_osb = $request['serial_subject'];
                $serial_sbc = null;
            }


            //Procesamiento de incidencias                
            //Si ya existe incidencia estudiante: Update o Delete
            $serial_ist = null;
            $flagAfectarSumaryDetail = true;
            $flagDelete = false;
            $flagAfectarSoloDetalle = false;
            if (($request['serial_ist'] && !isset($request['fix_ist']))|| $request['delete_ist']) {
                $serial_ist = $request['serial_ist'];

                if ($request['delete_ist']) {//DELETE (Inactivate)
                    $flagDelete = true;

                    //Consulta resumen por serial ica, serial_std, seria_sbp para obtener totales para posterior actualizacion
                    $iss = $this->incidenceStudentSummary->getIncidenceStudentSummaryBySerialIcaSerialStd($request['serial_ica'], $request['serial_std'], $request['serial_sbp']);
                    $serial_iss = $iss->serial_iss;
                    $total = $iss->total_iss;
                    $ciclo = $iss->cycle_iss;


                    //Obtencion de valor de incidencia
                    $ist = $this->incidenceStudent->getIncidenceStudentBySerialIst($serial_ist);
                    $value = $ist->value_ist * -1;

                    //Validacion si afectacion fue solo a detalle
                    if($ist->by_subject_ist == 1){
                        $flagAfectarSoloDetalle = true;
                    }

                    //Se ejecuta primero en caso que al afectar el valor exceda (inferior) y solo se afecte cierta parte
                    $value = $this->afectarDisciplinaConValorIncidencia($request, $value);

                    $dataIst = [
                        'serial_ist' => $serial_ist,
                        //Se setea el serial_usr porque aqui se pone el usuario activo                                   
                        'serial_usr_delete_ist' => $request['serial_usr'],
                        'delete_ist' => $request['delete_ist'],
                        'comentary_delete_ist' => $request['comentary_delete_ist'],
                        'value_ist' => $value
                    ];
                } else {//UPDATE
                    $flagAfectarSumaryDetail = false;
                    $dataIst = [
                        'serial_ist' => $serial_ist,
                        'comentary_ist' => $request['comentary_ist'],
                    ];
                }
            } else {
                //Consultar configuraciones generales de categoria
                $setupg = $this->incidenceSetup->incidenceGeneralSetupBySerialIca($request['serial_ica']);
                //Consultar configuraciones especificas (detalle) repeticion materia de categoria
                $setupd = $this->incidenceSetup->incidenceSpecificSubjectSetupBySerialIca($request['serial_ica']);
                //consutar configuracion general de incidencia
                if(sizeof($setupd)==0){
                    $setupd = $this->incidenceSetup->incidenceGeneralSetupBySerialIca($request['serial_ica']);
                }
                $serial_iss = null;
                //Total de registros generales                 
                //Consulta resumen por serial ica y std (En caso que ya exista un registro de otra materia)
                $iss = $this->incidenceStudentSummary->getIncidenceStudentSummaryBySerialIcaSerialStd($request['serial_ica'], $request['serial_std'], $request['serial_sbp']);
                if (empty($iss)) {
                    $total = 0; //Quemado en 0 primera vez
                    $ciclo = 0;
                } else {
                    $serial_iss = $iss->serial_iss;
                    $total = $iss->total_iss;
                    $ciclo = $iss->cycle_iss;
                }



                $serial_isd = null;
                //Total de registros especificos por materia                 
                //Consulta detalle por serial iss y materia
                $isd = $this->incidenceStudentDetail->getIncidenceStudentDetailBySerialIssSubject($serial_iss, $request['type_subject'], $request['serial_subject']);
                if (empty($isd)) {
                    $totalDet = 0; //Quemado en 0 primera vez
                    $cicloDet = 0;
                } else {
                    $serial_isd = $isd->serial_isd;
                    $totalDet = $isd->total_isd;
                    $cicloDet = $isd->cycle_isd;
                }


                //Busqueda valor especifico
                $noEncontroEnDetalle = true;
                //Si existe configuracion especfica buscar valor
                if (!empty($setupd)) {
                    //Valor a afectar
                    $value = 0;
                    foreach ($setupd as $setup) {                       
                        if ($setup['frequency_ise'] == $totalDet + 1) {
                            $noEncontroEnDetalle = false;
                            $flagAfectarSoloDetalle = true;
                            if ($setup['lower_discipline_level_ise'] == 1) {
                                $value = $this->calcularValorAfectarParaInmediatoInferior($request);
                            } else {
                                $value = $setup['value_ise'];
                            }
                            break;
                        }
                    }

                }


                //Busqueda valor general
                if ($noEncontroEnDetalle) {
                    //Valor a afectar
                    $encontroGeneral = false;
                    $value = 0;
                    foreach ($setupg as $setup) {
                        if ($setup['frequency_ise'] == $total + 1) {
                            $encontroGeneral = true;
                            if ($setup['lower_discipline_level_ise'] == 1) {
                                $value = $this->calcularValorAfectarParaInmediatoInferior($request);
                            } else {
                                $value = $setup['value_ise'];
                            }
                            break;
                        }
                    }

                    //Si no encuentra en general aplicar la primera configuracion existente de general o detalle si aplica
                    if (!$encontroGeneral) {                        
                        if(sizeof($setupg)>0){                            
                            $firstSetup = $setupg[0];
                        }else{
                            $firstSetup = $setupd[0];
                            $flagAfectarSoloDetalle = true;
                            $noEncontroEnDetalle = false;
                        }                        
                        if ($firstSetup['lower_discipline_level_ise'] == 1) {
                            $value = $this->calcularValorAfectarParaInmediatoInferior($request);
                        } else {
                            $value = $firstSetup['value_ise'];
                        }
                    }
                }


                //Si es insert afectar a disciplina subperiodo
                //Se ejecuta primero en caso que al afectar el valor exceda y solo se afecte cierta parte
                $value = $this->afectarDisciplinaConValorIncidencia($request, $value);

                if($noEncontroEnDetalle){
                    $by_subject_ist =0;
                }else{
                    $by_subject_ist =1;
                }

                $fix_ist = false;
                //En caso de ser fix de datos se trata como que fuera insercion en reglas pero al insertar se hace solo update
                if (isset($request['fix_ist']) && $request['fix_ist']) {
                    $serial_ist = $request['serial_ist'];
                    $fix_ist = true;
                }
                
                $dataIst = [
                    'serial_ist' => $serial_ist,
                    'serial_ica' => $request['serial_ica'],
                    'serial_usr' => $request['serial_usr'],
                    'serial_std' => $request['serial_std'],
                    'serial_sbp' => $request['serial_sbp'],
                    'date_ist' => $request['date_ist'],
                    'serial_sbc' => $serial_sbc,
                    'serial_osb' => $serial_osb,
                    'serial_scd' => $request['serial_scd'],
                    'comentary_ist' => $request['comentary_ist'],
                    'value_ist' => $value,
                    'by_subject_ist' => $by_subject_ist,
                    'fix_ist' => $fix_ist
                ];
            }

            //Insertar registro con ese valor en incidence_student
            $resultIs = $this->incidenceStudent->saveIncidenceStudent($dataIst);


            //Afectar si incidence_student fue insert o delete.
            if ($flagAfectarSumaryDetail) {

                
                    //Insert o update en summary
                    //Obtener frecuencia general maxima configurada (maximo total=>fin de ciclo)
                    $maxIsGeneral = $this->incidenceSetup->maxIncidenceGeneralSetupBySerialIca($request['serial_ica']);
                    if ($maxIsGeneral) {                    
                        $maximoTotal = $maxIsGeneral->frequency_ise;
                        if(!$flagAfectarSoloDetalle){
                            if ($flagDelete) {//Reducir total/ciclo
                                //Calculo total y ciclo
                                $total = $total - 1;
                                if ($ciclo > 1 && $total == 0) {
                                    $total = $maximoTotal;
                                    $ciclo = $ciclo - 1;
                                }
                                if($total < 0){
                                    $total = 0;
                                }
                            } else {//Aumentar ciclos
                                //Calculo total y ciclo
                                $total = $total + 1;
                                if ($total > $maximoTotal) {
                                    $total = 1;
                                    $ciclo = $ciclo + 1;
                                }
                                if($total>0 && $ciclo == 0){
                                    $ciclo = $ciclo + 1;
                                }
                            }
                        }
                    }else{
                        //Si no existe configuracion general y si registro general no existe total = 0 y ciclo 1.
                        if(!$serial_iss){
                            $total = 0;
                            $ciclo = 1;
                        }
                    }

                $dataIss = [
                    'serial_iss' => $serial_iss,
                    'serial_ica' => $request['serial_ica'],
                    'serial_std' => $request['serial_std'],
                    'serial_sbp' => $request['serial_sbp'],
                    'total_iss' => $total,
                    'cycle_iss' => $ciclo,
                ];
                $resultIss = $this->incidenceStudentSummary->saveIncidenceStudentSummary($dataIss);


                //Insert en detail y Totales detalle  
                $serial_isd = null;
                $serial_iss = $resultIss['serial_iss'];
                //Consulta detalle por serial iss y materia
                $isd = $this->incidenceStudentDetail->getIncidenceStudentDetailBySerialIssSubject($serial_iss, $request['type_subject'], $request['serial_subject']);
                //Cambiar maximo general por especifico si aplica
                if ($flagAfectarSoloDetalle) {
                    $maxIsSpecific = $this->incidenceSetup->maxIncidenceSpecificSubjectSetupBySerialIca($request['serial_ica']);
                    if (sizeof($maxIsSpecific) < 1) {
                        $maxIsSpecific = $this->incidenceSetup->maxIncidenceGeneralSetupBySerialIca($request['serial_ica']);
                    }
                    $maximoTotal = $maxIsSpecific->frequency_ise;
                }

                if (empty($isd)) {
                    $totalDet = 0; //Quemado en 0 primera vez
                    $cicloDet = 0;
                } else {
                    $serial_isd = $isd->serial_isd;
                    $totalDet = $isd->total_isd;
                    $cicloDet = $isd->cycle_isd;
                }


                if ($flagDelete) {//Reducir totalDet/cicloDet
                    //Calculo totalDet y cicloDet
                    $totalDet = $totalDet - 1;
                    if ($cicloDet > 1 && $totalDet == 0) {
                        $totalDet = $maximoTotal;
                        $cicloDet = $cicloDet - 1;
                    }
                    if($totalDet < 0){
                        $totalDet = 0;
                    }
                } else {
                    //Calculo totalDet y cicloDet
                    $totalDet = $totalDet + 1;
                    if ($totalDet > $maximoTotal) {
                        $totalDet = 1;
                        $cicloDet = $cicloDet + 1;
                    }
                }

                $dataIsd = [
                    'serial_isd' => $serial_isd,
                    'serial_iss' => $serial_iss,
                    'serial_sbc' => $serial_sbc,
                    'serial_osb' => $serial_osb,
                    'total_isd' => $totalDet,
                    'cycle_isd' => $cicloDet,
                ];
                $this->incidenceStudentDetail->saveIncidenceStudentDetail($dataIsd);
            }

            return response()->json($resultIs);
        } catch (Exception $ex) {            
            return response()->json($ex->getMessage());
        }
    }


    /**
     * Método para obtener calcular valor a afectar para reducir nota disciplina a inmediato inferior (limite superior)
     * @since   2018-03-27
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array data
     * @return  IncidenceSetup array
     * 
     */
    private function calcularValorAfectarParaInmediatoInferior($data){
        $value = 0;
        //Obtener nota actual de disciplina de estudiante        
        $subDisc = $this->subperiodDiscipline->getStudentSubperiodDiscipline($data);
        $notaDiscActual = $subDisc->general_sbp_grade_dsc;

        //Listado de equivalencias por anio
        $equivalences = $this->disciplineEquivalence->getDisciplineEquivalencesBySchoolYear($data['serial_scy']);

        $subperiod = $this->subperiod->getSubperiodDataBySerialSbp($data['serial_sbp']);
        $notaDiscActualPorcentaje = ($notaDiscActual * 100) / $subperiod->max_grade_prd;
        //Obtener equivalencia de nota actual
        $equivalence = $this->disciplineEquivalence->getGradeDisciplineEquivalenceBySchoolYear($data['serial_scy'], $notaDiscActualPorcentaje);

        if(!$equivalence){          
            throw new Exception ('No existe equivalencia válida de disciplina');            
        }

        $flag = false;
        
        //Determinar escala inmediata inferior
        foreach ($equivalences as $equival) {
            if ($flag) {
                $inmediateLowerMaxValue = $equival;
                break;
            }

            if($equival->serial_deq == $equivalence->serial_deq){
                $inmediateLowerMaxValue = $equival;
                $flag = true;
            }
        }

        if($inmediateLowerMaxValue->serial_deq != $equivalence->serial_deq){        
            $value = $notaDiscActualPorcentaje - $inmediateLowerMaxValue->higher_percentage_deq;
            //Cambiar value que es porcentual a puntaje de nota real
            $value = ($value * $subperiod->max_grade_prd )/100;
        }        

        return $value * -1;
    }



    /**
     * Método para cambiar la nota de disciplina con el valor de la incidencia. No se afecta a la disciplina aun, solo 
     * se evalua si el valor excede o no para recortarlo.
     * @since   2018-03-28
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array data
     * @return  
     * 
     */
    private function afectarDisciplinaConValorIncidencia($data, $valor){

        //Carga de informacion de subperiodo/periodo para tener maximos y minimos permitidos
        $subperiod = $this->subperiod->getSubperiodDataBySerialSbp($data['serial_sbp']);

        //Obtener nota actual de disciplina de estudiante        
        $subDisc = $this->subperiodDiscipline->getStudentSubperiodDiscipline($data);


        $generalSubGradeDscNuevo = $subDisc->general_sbp_grade_dsc + $valor;

        //Validar que no supere limite
        if ($generalSubGradeDscNuevo >  $subperiod->max_grade_prd) {
            $valor = $subperiod->max_grade_prd - $subDisc->general_sbp_grade_dsc;
            $generalSubGradeDscNuevo = $subperiod->max_grade_prd;            
        }

        //Validar que limite no sea menor a lo permitido
        if ($generalSubGradeDscNuevo <  $subperiod->min_grade_prd) {
            $valor = $subperiod->min_grade_prd - $subDisc->general_sbp_grade_dsc;
            $generalSubGradeDscNuevo = $subperiod->min_grade_prd;            
        }

        return $valor;        
    }


    /**
     * Método para obtener listado de estudiantes indicando si tiene alguna incidencia en ese dia
     * @since   2018-03-23
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @return  IncidenceSetup array
     * 
     */
    public function postListStudents(Request $request) {
        $this->validate($request, [
            'type_subject' => 'required',
            'date' => 'required',            
            'serial_subject' => 'required',            
        ]);

        if ($request->input('type_subject') == 'Obligatoria') {
            //obtengo los estudiantes del curso
            $students = $this->course->getStudents($request->input('serial_crs_lop'));            
        } else {
            //obtengo los estudiantes de la materia optativa
            $students = $this->optionalSubject->getStudents($request->input('serial_subject'), $request->input('date'));           
        }

        //Valida si tiene incidencias en esa fecha y/u horario
        foreach ($students as $student) {
            $serial_scd = $request->input('serial_scd');

            //Obtencion de serial_scd real del estudiante en caso que sea optativa
            if ($request->input('type_subject') == 'Optativa') {
                //Obtener curso estudiante
                $serial_scdAux = $this->getStudentRealOptativeSerialScd($student->serial_std, $request->input('serial_scy'), $request->input('serial_cdt'), $request->input('date'), $request->input('serial_crs_lop'));
                if ($serial_scdAux) {
                    $serial_scd = $serial_scdAux;
                }
            }

            //Obtener total de incidencias                
            $student->incidence = $this->incidenceStudent->getIncidenceTotalsByStudentDate($student->serial_std, $request->input('date'), $serial_scd);
        }
        return response()->json($students);
    }

    /**
     * Obtencion de serialScd real de materias optativa en horario del curso de estudiante
     * @since   2018-04-20
     * @author  Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @param   $serial_std serial estudiante
     * @param   $serial_cdt serial credito/hora
     * @param   $serial_scy serial anio
     * @param   $serial_lop serial lop
     * @param   $date fecha
     * @return  boolean
     */
    public function getStudentRealOptativeSerialScd($serial_std, $serial_scy, $serial_cdt, $date, $serial_lop)
    {
        $real_scd = null;
        //Obtener curso de estudiante
        $courseData = $this->course->getCourseByStudentYear($serial_std, $serial_scy);        
        $serial_crs = $courseData['serial_crs'];

        //Obtener dia a partir de la fecha de configuracion
        $dateDayConfig = $this->dateDayConfiguration->getDateDayConfiguration($date);
        $day = $dateDayConfig['day_ddc'];

        //Carga de horario por curso        
        $courseSchedule = $this->schedule->getScheduleByCourseDateDay($serial_crs,$date,$day);

        
        //Valida si existe ese horario en el curso
        if ($courseSchedule) {
            foreach ($courseSchedule as $cs) {
                if ($cs['serial_lop'] == $serial_lop) {
                    $sch = $this->schedule->getScheduleBySerialScd($cs['serial_scd']);     

                    if ($sch['serial_cdt'] == $serial_cdt) {
                        $real_scd = $cs['serial_scd'];                        
                        break;
                    }                    
                }
            }
        }
        return $real_scd;
    }



    /**
     * Método para obtener listado de incidencias de un estudiant
     * @since   2018-03-24
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_std serial del estudiante
     * @param   int $date fecha
     * @return  IncidenceSetup array
     * 
     */
    public function postListStudentIncidences(Request $request) {
        $this->validate($request, [
            'serial_std' => 'required',
            'date' => 'required',           
        ]);

        $incidences = $this->incidenceStudent->getIncidencesByStudentDate($request->input('serial_std'), $request->input('date'), $request->input('serial_scd'));

        return response()->json($incidences);
    }


    /**
     * Método que obtiene listado de incidencias de estudiante en un anio y periodo
     * @since   2018-04-05
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   $serial_std serial de estudiante
     * @param   $serial_scy serial de anio
     * @param   $serial_prd serial de periodo
     * @return  
     * 
     */
    public function getIncidencesByStudentYearPeriod($serial_std, $serial_scy, $serial_prd) {
         $incidences = $this->incidenceStudent->getIncidencesByStudentYearPeriod($serial_std, $serial_scy, $serial_prd);
         return response()->json($incidences);
    }


    /**
     * Método que obtiene listado de incidencias de estudiante por serial
     * @since   2018-04-05
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   $serial_std serial de estudiante
     * @param   $serial_ist serial de incidencia
     * @return
     *
     */
    public function getIncidencesByIncidenceStudent($serial_ist) {
         $incidence = $this->incidenceStudent->getIncidenceStudentBySerialIst($serial_ist);
         return response()->json($incidence);
    }


 /**
     * Método que obtiene listado de incidencias de estudiante en un anio, periodo y subperiodo usado
     * para correccion y recalculo de incidencias por orden de registro
     * @since   2018-10-16
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   $serial_std serial de estudiante
     * @param   $serial_scy serial de anio
     * @param   $serial_prd serial de periodo
     * @param   $serial_sbp serial de subperiodo
     * @return  
     * 
     */
    public function getIncidencesByStudentYearPeriodSubperiod($serial_std, $serial_scy, $serial_prd, $serial_sbp) {
         $incidences = $this->incidenceStudent->getIncidencesByStudentYearPeriodSubperiod($serial_std, $serial_scy, $serial_prd, $serial_sbp);
         return response()->json($incidences);
    }


     /**
     * Método para borrar registros relacionados a incidencias como summary y detail por estudiante y sbperiodo, usado
     * para correccion y recalculo de incidencias
     * @since   2018-10-16
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   $serial_std serial de estudiante     
     * @param   $serial_sbp serial de subperiodo
     * @return  
     * 
     */
    public function deleteIncidenceRelatedRecordsBySerialStdSubperiod(Request $request) {
         
        $iss = $this->incidenceStudentSummary->getIncidenceStudentSummaryBySerialStdSubperiod( $request['serial_std'], $request['serial_sbp']);

            if($iss){
                //Borrado de cada detalle
                foreach ($iss as $val) {
                    $this->incidenceStudentDetail->deleteIncidenceStudentDetailBySerialIss($val['serial_iss']);
                }
                //Borrado de todos los summary 
                $this->incidenceStudentSummary->deleteIncidenceStudentSummaryBySerialStdSubperiod($request['serial_std'], $request['serial_sbp']);
            }


         return response()->json(true);
    }

}