<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Repositories\IncidenceYearSectionLevelRepository;
use App\Repositories\SubjectCourseRepository;
use App\Repositories\OptionalSubjectRepository;
use App\Repositories\CourseRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

/**
 * Controlador para IncidenceYearSectionLevel
 * @since   2018-03-15
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-15<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del controlador<br/>
 *          #edit1
 */
class IncidenceYearSectionLevelController extends BaseController {

    protected $incidenceYearSectionLevel;
    protected $subjectCourse;
    protected $optionalSubject;
    protected $course;

   /**
     * Constructor
     * @since   2018-03-15
     * @param   IncidenceYearSectionLevelRepository  $incidenceYearSectionLevel repositorio
     * @param   SubjectCourseRepository  $subjectCourse repositorio
     * @param   OptionalSubjectRepository  $optionalSubject repositorio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     */
    public function __construct(IncidenceYearSectionLevelRepository $incidenceYearSectionLevel, SubjectCourseRepository $subjectCourse, OptionalSubjectRepository $optionalSubject, CourseRepository $course){
        $this->incidenceYearSectionLevel = $incidenceYearSectionLevel;
        $this->subjectCourse = $subjectCourse;
        $this->optionalSubject = $optionalSubject;
        $this->course = $course;
    }



  /**
     * Método que guarda categoria
     * @since   2018-03-15
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $request
     * @return  
     * 
     */
    public function postSaveIncidenceYearSectionLevel(Request $request) {
        try {
            //Procesamiento categoria Padre
            $this->validate($request, [
                'serial_scy' => 'required',
                'serial_lvl' => 'required',
                'serial_set' => 'required',       
                'status_iyl' => 'required'
            ]);
            $this->incidenceYearSectionLevel->saveIncidenceYearSectionLevel($request);          
            
            return response()->json(true);    
        } catch (Exception $ex) {
            return false;
        }
    }


    /**
     * Método para obtener incidencia nivel por anio y seccion
     * @since   2018-03-15
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @param   int $serial_set serial de la seccion     
     * @return  Categories array
     * 
     */
    public function getIncidenceLevelsBySchoolYearSection($serial_scy,$serial_set){
        $iyl = $this->incidenceYearSectionLevel->getIncidenceLevelBySchoolYearSection($serial_scy,$serial_set);
        return response()->json($iyl);
    }


    /**
     * Método para obtener configuracion de incidencia por materia y tipo de materia
     * @since   2018-03-19
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_subject serial de materia
     * @param   int $type_subject tipo de materia
     * @return  true or false
     * 
     */
    public function getIncidenceSetupBySubjectType($serial_subject,$type_subject){
        //Obtener serial anio, section y nivel en base a materia
        if($type_subject == 'Obligatoria'){
            $subject = $this->subjectCourse->getSubjectCourseInformation($serial_subject);
        }else{
            $subject = $this->optionalSubject->getOptionalSubjectInformationBySerialOsb($serial_subject);
        }


        $iyl = $this->incidenceYearSectionLevel->getIncidenceLevelBySchoolYearSectionLevel($subject[0]['serial_scy'],$subject[0]['serial_set'],$subject[0]['serial_lvl']);


        return response()->json($iyl);
    }


    /**
     * Método para obtener configuracion de incidencia por curso
     * @since   2018-05-01
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_crs serial de curso     
     * @return  true or false
     * 
     */
    public function getIncidenceSetupByCourse($serial_crs){
        //Obtener serial anio, section y nivel en base a curso        
        $course = $this->course->getCourse($serial_crs);        

        $iyl = $this->incidenceYearSectionLevel->getIncidenceLevelBySchoolYearSectionLevel($course['serial_scy'],$course['serial_set'],$course['serial_lvl']);


        return response()->json($iyl);
    }



}