<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Repositories\OptionalSubjectRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

/**
 * Controlador para materias optativas
 * @since   2017-07-09
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2017-07-09<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creacin del controlador<br/>
 *          #edit1
 */
class OptionalSubjectController extends BaseController {

    protected $optionalSubject;

    /**
     * Constructor
     * @since   2016-07-09
     * @param   OptionalSubjectRepository  $optionalSubject repositorio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     */
    public function __construct(OptionalSubjectRepository $optionalSubject){
        $this->optionalSubject = $optionalSubject;
    }

    /**
     * Mtodo para obtener tipos de materias
     * @since   2016-07-09
     * @param   integer serial_scy ao lectivo
     * @param   integer $serial_sec seccion
     * @param   integer $serial_spt especialidad
     * @param   integer $serial_lvl nivel
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getSubjectsBySchoolYearSectionLevel($serial_scy, $serial_set, $serial_lvl, $serial_spt){

        $subjectList = $this->optionalSubject->getSubjectsBySchoolYearSectionLevel($serial_scy, $serial_set, $serial_lvl, $serial_spt);

        return response()->json($subjectList);
    }

    public function postStudents(Request $request) {
        $this->validate($request, [
            'serial_osb' => 'required',
            'date' => 'required'
        ]);
        $students = $this->optionalSubject->getStudents($request->input('serial_osb'), $request->input('date'));
        return response()->json($students);
    }

}
