<?php

namespace App\Http\Controllers;

use App\Repositories\CountryRepository;
use Illuminate\Http\Request;
use App\Repositories\SectionRepository;
use App\Repositories\SchoolYearRepository;
use App\Repositories\LevelSectionYearRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

/**
 * Controlador para secciones
 * @since   2017-07-09
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2017-07-09<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del controlador<br/>
 *          #edit1
 */
class SectionController extends BaseController {

    protected $section;
    protected $schoolYear;
    protected $levelSectionYear;
    protected $country;

    /**
     * Constructor
     * @since   2016-07-09
     * @param   SectionRepository  $section repositorio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     */
    public function __construct(SectionRepository $section, SchoolYearRepository $schoolYear, LevelSectionYearRepository $levelSectionYear, CountryRepository $country) {
        $this->section = $section;
        $this->schoolYear = $schoolYear;
        $this->levelSectionYear = $levelSectionYear;
        $this->country = $country;
    }

    /**
     * Método para obtener secciones/especialidad/nivel
     * @since   2016-07-09
     * @param   integer serial_scy año lectivo
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getSectionSpecialtyLevelBySchoolYear($serial_scy){

        $sectionLvlList = $this->section->getSectionSpecialtyLevelBySchoolYear($serial_scy);

        return response()->json($sectionLvlList);
        
    }
    
    public function getListsSectionsByDate($startDate, $endDate) {
        $schoolYear = $this->schoolYear->getSchoolYearByDate($startDate, $endDate);
        $sections = $this->levelSectionYear->listsSectionsBySchoolYear($schoolYear->serial_scy);
        return response()->json($sections);
    }

    public function getSectionsByDate($date) {
        $schoolYear = $this->schoolYear->getSchoolYearByDate($date, $date);
        $sections = $this->section->getSectionBySchoolYear($schoolYear->serial_scy);
        return response()->json($sections);
    }

    /**
     * Método para obtener secciones por serial de anio lectivo
     * @since   2018-03-15
     * @param   integer serial_scy año lectivo
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getSectionsBySerialScy($serial_scy) {
        $sections = $this->section->getSectionBySchoolYear($serial_scy);
        return response()->json($sections);
    }

    /**
     * Método para obtener sección por año lectivo
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-21
     * @return array $countries
     */
    public function getSectionBySchoolYear(Request $request){
        $this->validate($request, [
            'serial_scy' => 'required',
        ]);
        $serialSchoolYear = $request->get('serial_scy');
        $sections = $this->section->getSectionsBySchoolYear($serialSchoolYear);
        $countries = $this->country->getCountries();
        return response()->json(array('sections'=>$sections,
            'countries'=>$countries));
    }
}