<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Repositories\SkillsRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

/**
 * Controlador para destrezas
 * @since   2017-07-09
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2017-07-09<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del controlador<br/>
 *          #edit1
 */
class SkillsController extends BaseController {

    protected $skills;

   /**
     * Constructor
     * @since   2016-07-09
     * @param   SkillsRepository  $skills repositorio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     */
    public function __construct(SkillsRepository $skills){
        $this->skills = $skills;
    }

    /**
     * Método para obtener destrezas disponibles (no asignadas)
     * @since   2016-07-09
     * @param   integer serial_subject materia
     * @param   string subject_type tipo materia
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getSkillsAvailableBySubjectType($serial_subject,$subject_type){

        $skillsList = $this->skills->getSkillsAvailableBySubjectType($serial_subject, $subject_type);

        return response()->json($skillsList);        
    }

    /**
     * Método para obtener destrezas asignadas
     * @since   2016-07-09
     * @param   integer serial_subject materia
     * @param   string subject_type tipo materia
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getSkillsAssignedBySubjectType($serial_subject,$subject_type){

        $skillsList = $this->skills->getSkillsAssignedBySubjectType($serial_subject, $subject_type);

        return response()->json($skillsList);
    }


}