<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Repositories\SubjectCourseRepository;
use App\Repositories\OptionalSubjectRepository;
use App\Repositories\SchoolYearRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

/**
 * Controlador para materias
 * @since   2018-03-22
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-22<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del controlador<br/>
 *          #edit1
 */
class SubjectController extends BaseController {

    protected $subjectCourse;
    protected $optionalSubject;
    protected $schoolYear;

    /**
     * Constructor
     * @since   2018-03-22
     * @param   SubjectCourseRepository  $subjectCourse repositorio
     * @param   OptionalSubjectRepository  $optionalSubject repositorio
     * @param   SchoolYearRepository  $schoolYear repositorio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     *
     */
    public function __construct(SubjectCourseRepository $subjectCourse, OptionalSubjectRepository $optionalSubject, SchoolYearRepository $schoolYear){
        $this->subjectCourse = $subjectCourse;
        $this->optionalSubject = $optionalSubject;
        $this->schoolYear = $schoolYear;
    }

    /**
     * Método para obtener informacion de la materia del año lectivo en curso
     * @since   2018-03-22
     * @param   integer serial_scy serial anio
     * @param   integer serial_subject serial materia
     * @param   integer $type_subject tipo de materia
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getSubjectDataBySerialSubjectType($serial_scy, $serial_subject, $type_subject){

        if ($type_subject == 'Obligatoria') {            
            $subject = $this->subjectCourse->getSubjectCourseBySerialSbcSchoolYear($serial_subject, $serial_scy);
        } else {            
            $subject = $this->optionalSubject->getOptionalSubjectBySerialOsbSchoolYear($serial_subject, $serial_scy);
        }

        return response()->json($subject);        
    }


}