<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Repositories\LevelSectionYearRepository;
use App\Repositories\SubperiodRepository;
use Laravel\Lumen\Routing\Controller as BaseController;

class SubperiodController extends BaseController {

    protected $levelSectionYear;
    protected $subperiod;

    public function __construct(LevelSectionYearRepository $levelSectionYear, SubperiodRepository $subperiod) {
        $this->levelSectionYear = $levelSectionYear;
        $this->subperiod = $subperiod;
    }

    /**
     * Método que devuelve el subperiodo por fecha, anio, section, nivel
     * @since   2018-03-24
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   array $request 
     * @return  ListObject $subperiod
     * 
     * @edit    2018-03-24<br />
     *          Erick Vaca <epvaca@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function postSubperiodByDateSchoolYearSectionLevel(Request $request) {
        $this->validate($request, [
            'date' => 'required',
            'serial_scy' => 'required',
            'serial_lvl' => 'required',
            'serial_set' => 'required'
        ]);
        $subperiod = $this->levelSectionYear->getSubperiodByDateSchoolYearSectionLevel($request->input('date'),$request->input('serial_scy'),$request->input('serial_set'),$request->input('serial_lvl'));

        return response()->json($subperiod);
    }


    /**
     * Método para obtener datos de subperiodo por serial
     * @since   2018-03-28
     * @param   integer serial_sbp serial subperiodo
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
   public function getSubperiodDataBySerialSbp($serial_sbp) {

        $subperiod = $this->subperiod->getSubperiodDataBySerialSbp($serial_sbp);
         
        return response()->json($subperiod);
    }

    /**
     * Método para obtener datos de subperiodo por serial  del periodo
     * @since   2018-05-03
     * @param   integer serial_prd serial del periodo
     * @author  Fernando Salas <rfsalas@rutatec.com><br /><br/>
     * @return  json $subperiods subperiodos del periodod seleccionado más el examen
     *
     */
    public function getSubperiods($serial_prd)
    {
        if ($serial_prd == 0) {
            return response()->json(false);
        }
        $subperiods = $this->subperiod->getSubperiodsByPeriod($serial_prd)->toArray();
        foreach ($subperiods as &$subperiod) {
            $nameSubperiod = explode(" ", $subperiod['name_sbp']);
            $subperiodAcronym = "";

            foreach ($nameSubperiod as $name) {
                $subperiodAcronym .= $name[0];
            }
            $subperiod['name_sbp'] = $subperiodAcronym;
        }
        $exam = array('serial_sbp' => 'EX', 'name_sbp' => 'EX');
        $average_period = array('serial_sbp' => 'PR', 'name_sbp' => 'PR');
        array_push($subperiods, $exam);
        array_push($subperiods, $average_period);

        return response()->json($subperiods);

    }
    

}
