<?php

namespace App\Repositories;

use App\Models\StudentCriteriaGrade;
use App\Models\OptStudentCriteriaGrade;
use App\Models\Period;
use App\Models\Activity;
use App\Models\ActivityAttachment;
use function array_merge;

class ActivityRepository extends BaseRepository
{

    protected $studentCriteriaGrade;
    protected $optStudentCriteriaGrade;
    protected $period;
    protected $modelClass = Activity::class;



    //Declaración del constructor
    public function __construct(StudentCriteriaGrade $studentCriteriaGrade, OptStudentCriteriaGrade $optStudentCriteriaGrade, Period $period)
    {
        $this->studentCriteriaGrade = $studentCriteriaGrade;
        $this->optStudentCriteriaGrade = $optStudentCriteriaGrade;
        $this->period = $period;
    }

    /**
     * Método que devuelve el curso de un estudiante por serial_std y actividad
     * @since   2018-04-02
     * @author Erick Vaca <epvaca@rutatec.com>
     * @param   integer $serial_std serial del estudiante
     * @param   integer $serial_act serial de la actividad
     * @return  objeto con detalle de actividades tomadas desde la fecha actual
     *
     * @edit    2018-08-09<br />
     *          Jairo Serrano <jaserrano@rutatec.com> <br />
     *          Cambio el metodo a eloquent y agrego atributos a la respuesta
     * #edit2
     */
    public function getActivitiesByStudent($serial_std, $serial_act)
    {
        $now = date('Y-m-d');
        $dataTime= new \DateTime("$now");
        //modificar la fecha a 3 dias antes
        $dataTime->modify("-3 day" );
        $past_days=$dataTime->format('Y-m-d');
        //mandatory activities
        $activities = array();
        //get student_criteria_grade_data
        $student_criteria_grade = StudentCriteriaGrade::where('serial_std',$serial_std);
        if($serial_act!= null){
            $student_criteria_grade->where('serial_act',$serial_act);
        }
        $student_criteria_grades= $student_criteria_grade->where('status_scg','ACTIVE')
            ->whereHas('activity_type', function ($query) {
                $query->where('serial_att', '!=', 1)
                    ->where('status_att', '=', 'ACTIVE');
            })->whereHas('activity', function ($query) use($past_days) {
                $query->where('status_act', '=', 'ACTIVE')
                ->where('due_date_act','>=',$past_days);
            })->whereHas('subject_course', function ($query) {
                $query->where('status_sbc', '=', 'ACTIVE');
            })->whereHas('grade_criteria', function ($query) {
                $query->where('status_gct', '=', 'ACTIVE');
            })->whereHas('course', function ($query) {
                $query->where('status_crs', '=', 'ACTIVE');
            })->get()->take(30);

        //get attributes from data
        foreach ($student_criteria_grades as $key=>&$scg){
            $activity= $scg->activity()->first();
            $activities[$key]['serial_act']=$activity->serial_act;
            $activities[$key]['name_act']=$activity->name_act;
            $activities[$key]['description_act']=$activity->description_act;
            $activities[$key]['due_date_act']=$activity->due_date_act;
            $activities[$key]['std_complete_task']=$scg->finished!=0?TRUE:FALSE;
            $activity_type= $scg->activity_type()->first();
            $activities[$key]['name_att']=$activity_type->name_att;
            $cat_subject=$scg->subject_course()->first()
                ->subject()->where('status_sbj','ACTIVE')
                ->first()
                ->cat_subject()->where('status_csb','ACTIVE')
                ->first();
            $activities[$key]['name_csb']=$cat_subject->name_csb;
            $activity_attachments=$scg->activity->activity_attachments()->get();
            $activities[$key]['num_attachments']=sizeof($activity_attachments);
            if(sizeof($activity_attachments)>0){
                foreach ($activity_attachments as $attachment){
                    $attachment_details['name_ath']=$attachment->name_ath;
                    $attachment_details['file_ath']="activity_attachment/".$activity->serial_act."/".$attachment->filename_ath;
                    $activities[$key]['file_attach'][]=$attachment_details;
                }
            }
        }
        //optional activities
        $opt_activities = array();
        //get opt_student_criteria_grade
        $opt_student_criteria_grade = OptStudentCriteriaGrade::where('serial_std',$serial_std);
        if($serial_act!= null){
            $opt_student_criteria_grade->where('serial_opa',$serial_act);
        }
        $opt_student_criteria_grades =$opt_student_criteria_grade->where('status_osg','ACTIVE')
            ->whereHas('activity_type', function ($query) {
                $query->where('serial_att', '!=', 1)
                    ->where('status_att', '=', 'ACTIVE');
            })->whereHas('opt_activity', function ($query) use($past_days) {
                $query->where('status_opa', '=', 'ACTIVE')
                    ->where('due_date_opa','>=',$past_days);
            })->whereHas('optional_subject', function ($query) {
                $query->where('status_osb', '=', 'ACTIVE');
            })->whereHas('grade_criteria', function ($query) {
                $query->where('status_gct', '=', 'ACTIVE');
            })->get()->take(30);

        //get attributes from opt_activities
        foreach ($opt_student_criteria_grades as $key=>&$osg){
            $opt_activity= $osg->opt_activity()->first();
            $opt_activities[$key]['serial_act']=$opt_activity->serial_opa;
            $opt_activities[$key]['name_act']=$opt_activity->name_opa;
            $opt_activities[$key]['description_act']=$opt_activity->description_opa;
            $opt_activities[$key]['due_date_act']=$opt_activity->due_date_opa;
            $opt_activities[$key]['std_complete_task']=$osg->finished!=0?TRUE:FALSE;
            $activity_type= $osg->activity_type()->first();
            $opt_activities[$key]['name_att']=$activity_type->name_att;
            $cat_subject=$osg->optional_subject()->first()
                ->cat_subject()->where('status_csb','ACTIVE')
                ->first();
            $opt_activities[$key]['name_csb']=$cat_subject->name_csb;
            $opt_activity_attachments=$osg->opt_activity->activity_attachments()->get();
            $opt_activities[$key]['num_attachments']=sizeof($opt_activity_attachments);
            if(sizeof($opt_activity_attachments)>0){
                foreach ($opt_activity_attachments as $opt_attachment){
                    $attachment_details['name_ath']=$opt_attachment->name_ath;
                    $attachment_details['file_ath']="activity_attachment/".$opt_activity->serial_opa."/".$opt_attachment->filename_ath;
                    $opt_activities[$key]['file_attach'][]=$attachment_details;
                }
            }
        }
        $activities_details=array_merge($activities,$opt_activities);

        //order activities by due_date
        $activities_details= $this->sortArrayByAttribute($activities_details,'due_date_act');

        return ($activities_details);
    }



    /**
     * Método para obtener el detalle de las actividades
     *  @since   2018-07-27
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   array $data
     * @return  object 
     *
     */
    public function getActivityDetails($data)
    {
        $serial_act = $data['serial_act'];
       
            $query = $this->newQuery();        
            $query->join('activity_applied_criterias AS aac','aac.serial_act', '=', 'activity.serial_act');
            $query->join('subject_course AS sbc','sbc.serial_sbc', '=', 'aac.serial_sbc');
            $query->join('course AS crs','crs.serial_crs', '=', 'sbc.serial_crs');
            $query->join('school_year AS scy','scy.serial_scy', '=', 'crs.serial_scy');
            $query->join('subject AS sbj','sbj.serial_sbj', '=', 'sbc.serial_sbj');
            $query->join('cat_subject AS csb','csb.serial_csb', '=', 'sbj.serial_csb');
            $query->join('activity_type AS att','att.serial_att', '=', 'aac.serial_att');

            $query->where('activity.serial_act', $serial_act);
            $query->where('status_act', 'ACTIVE');    
            $query->where('status_sbc', 'ACTIVE');
            $query->where('status_crs', 'ACTIVE');
            $query->where('status_scy','LIKE', '%CURRENT%');
            $query->where('status_sbj', 'ACTIVE');
            $query->where('status_csb', 'ACTIVE');
            $query->where('status_att', 'ACTIVE');

            $query->orderBy('csb.name_csb');
            $query->distinct();
            $query->selectRaw("activity.serial_act, csb.name_csb, activity.name_act, activity.description_act, DATE_FORMAT(activity.send_date_act, '%W-%e-%M-%Y') as send_date, DATE_FORMAT(activity.due_date_act, '%W-%e-%M-%Y') as due_date, att.serial_att as activity_type, att.name_att");


            $message  = $this->doQueryGet($query);           
            $attach_files =ActivityAttachment::where('serial_act', '=', $serial_act)->where('status_ath', '=', 'ACTIVE')->get();
           
            if (sizeof($message)>0) {
                $return_array = array(
                    'serial_act' => $message[0]->serial_act,
                    'activity_type' => $message[0]->activity_type,
                    'name_act' => $message[0]->name_act,
                    'name_att' => $message[0]->name_att,
                    'name_csb' => $message[0]->name_csb,
                    'description_act' => $message[0]->description_act,
                    'send_date' => $message[0]->send_date,
                    'due_date' => $message[0]->due_date,
                );
                if (sizeof($attach_files)>0) {
                    $return_array['name_ath'] = $attach_files[0]->name_ath;
                    $return_array['filename_ath'] = $attach_files[0]->filename_ath;
                    $return_array['path'] = env('ATTACH_PATH') . 'activity_attachment/' . $return_array['serial_act'] . '/';
                    return $return_array;
                } else {
                    $return_array['name_ath'] = '';
                    $return_array['filename_ath'] = '';
                    $return_array['path'] = '';
                    return $return_array;
                }
            }        

    }

//order array
    function sortArrayByAttribute($array, $attribute,$order='ASC')
    {
        usort($array, function ($a, $b) use ($attribute,$order) {
            $a = strtotime($a[$attribute]);
            $b = strtotime($b[$attribute]);
            if($order=='DESC'){
                return (($a === $b) ? 0 : (($a > $b) ? -1 : 1));
            }else{
                return (($a === $b) ? 0 : (($a < $b) ? -1 : 1));
            }

        });
        return $array;
    }


}
