<?php
/**
 * Repositorio para consultas sobre un aplicante.
 * User: Henry Vivanco <hvvivanco@rutatec.com>
 * Date: 23/1/2019
 * Time: 12:17
 */

namespace App\Repositories;


use App\Models\ApplicantStudent;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;

class ApplicantStudentRepository extends Model
{
    protected $applicantStudent;

    public function __construct(ApplicantStudent $applicantStudent)
    {
        $this->applicantStudent = $applicantStudent;
    }

    public function saveApplicantStudent($student){

        $applicantStudent = ApplicantStudent::where('id_ast','=',$student['id_ast'])->first();
        if ($applicantStudent) {
            $applicantStudent->serial_agd = !empty($student['serial_agd']) ? $student['serial_agd'] : $applicantStudent->serial_agd;
            $applicantStudent->serial_scy = !empty($student['serial_scy']) ? $student['serial_scy'] : $applicantStudent->serial_scy;
            $applicantStudent->serial_set = !empty($student['serial_set']) ? $student['serial_set'] : $applicantStudent->serial_set;
            $applicantStudent->serial_lvl = !empty($student['serial_lvl']) ? $student['serial_lvl'] : $applicantStudent->serial_lvl;
            $applicantStudent->serial_cou_last_school_ast = !empty($student['serial_cou_last_school_ast']) ? $student['serial_cou_last_school_ast'] : $applicantStudent->serial_cou_last_school_ast;
            $applicantStudent->first_name_ast = !empty($student['first_name_ast']) ? $student['first_name_ast'] : $applicantStudent->first_name_ast;
            $applicantStudent->last_name_ast = !empty($student['last_name_ast']) ? $student['last_name_ast'] : $applicantStudent->last_name_ast;
            $applicantStudent->id_ast = !empty($student['id_ast']) ? $student['id_ast'] : $applicantStudent->id_ast;
            $applicantStudent->gender_ast = !empty($student['gender_ast']) ? $student['gender_ast'] : $applicantStudent->gender_ast;
            $applicantStudent->school_city_last_school_ast = !empty($student['school_city_last_school_ast']) ? $student['school_city_last_school_ast'] : $applicantStudent->school_city_last_school_ast;
            $applicantStudent->last_school_ast = !empty($student['last_school_ast']) ? $student['last_school_ast'] : $applicantStudent->last_school_ast;
            $applicantStudent->birthdate_ast = !empty($student['birthdate_ast']) ? $student['birthdate_ast'] : $applicantStudent->birthdate_ast;
            $applicantStudent->has_siblins_in_school_ast = !empty($student['has_siblins_in_school_ast']) ? $student['has_siblins_in_school_ast'] : $applicantStudent->has_siblins_in_school_ast;
            $applicantStudent->parent_ex_student_ast = !empty($student['parent_ex_student_ast']) ? $student['parent_ex_student_ast'] : $applicantStudent->parent_ex_student_ast;
            $applicantStudent->lives_with_ast = !empty($student['lives_with_ast']) ? $student['lives_with_ast'] : $applicantStudent->lives_with_ast;
            $applicantStudent->applicant_type_ast = !empty($student['applicant_type_ast']) ? $student['applicant_type_ast'] : $applicantStudent->applicant_type_ast;
            $applicantStudent->has_visited_school_ast = !empty($student['has_visited_school_ast']) ? $student['has_visited_school_ast'] : $applicantStudent->has_visited_school_ast;
            $applicantStudent->flag1_ast = !empty($student['flag1_ast']) ? $student['flag1_ast'] : $applicantStudent->flag1_ast;
            $applicantStudent->flag2_ast = !empty($student['flag2_ast']) ? $student['flag2_ast'] : $applicantStudent->flag2_ast;
            $applicantStudent->flag3_ast = !empty($student['flag3_ast']) ? $student['flag3_ast'] : $applicantStudent->flag3_ast;
            $applicantStudent->status_applicant_ast = !empty($student['status_applicant_ast']) ? $student['status_applicant_ast'] : $applicantStudent->status_applicant_ast;
            $applicantStudent->relationship_ast = !empty($student['relationship_ast']) ? $student['relationship_ast'] : $applicantStudent->relationship_ast;
            $applicantStudent->observation_ast = !empty($student['observation_ast']) ? $student['observation_ast'] : $applicantStudent->observation_ast;
            $applicantStudent->ip_address_ast = !empty($student['ip_address_ast']) ? $student['ip_address_ast'] : $applicantStudent->ip_address_ast;
            $applicantStudent->save();
        } else {
            $applicantStudent = new ApplicantStudent($student);
            $applicantStudent->save();
        }
        return $applicantStudent;

    }
    /**
     * Método para obtener lista aplicantes en un rango de fechas
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-24
     * @return array $applicants
     */
    public function getApplicants($data){
        $startDate = Date('Y-m-d 00:00:00', strtotime($data['start_date']));
        $endDate = Date('Y-m-d 23:59:59', strtotime($data['end_date']));
        $applicants = ApplicantStudent::where('applicant_student.serial_scy','=',$data['serial_scy'])
            ->where('applicant_student.serial_set','=',$data['serial_set'])
            ->where('applicant_student.serial_lvl','=',$data['serial_lvl'])
            ->where('applicant_student.created_at','>',$startDate)
            ->where('applicant_student.created_at','<',$endDate)
            ->where('status_ast','=','ACTIVE')
            ->join('applicant_guardian as agd','agd.serial_agd','=','applicant_student.serial_agd')
            ->join('level as lvl','lvl.serial_lvl','=','applicant_student.serial_lvl')
            ->join('section as set','set.serial_set','=','applicant_student.serial_set')
            ->join('school_year as scy','scy.serial_scy','=','applicant_student.serial_scy')
            ->select('applicant_student.serial_ast','applicant_student.last_name_ast','first_name_ast','scy.name_scy','set.name_set','lvl.name_lvl',
                'applicant_student.id_ast','agd.last_name_agd','agd.first_name_agd','agd.city_residence_agd','applicant_student.has_visited_school_ast',
                'flag1_ast','flag2_ast','flag3_ast','status_applicant_ast','observation_ast')
            ->selectRaw("DATE_FORMAT(applicant_student.created_at,'%d/%m/%Y') as date_app")
            ->orderBy('applicant_student.last_name_ast')
            ->get();

        return $applicants;
    }

    /**
     * Método para guardar estado si el estudiante applicante visitó
     * el colegio.
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-24
     * @return $student
     */
    public function saveVisitedSchoolStd($data){
        $student = ApplicantStudent::where('id_ast','=',$data['id_ast'])
            ->where('serial_scy','=',$data['serial_scy'])
            ->where('status_ast','=','ACTIVE')
            ->first();
        if ($student) {
            $student->has_visited_school_ast = !empty($data['has_visited_school_ast']) ? $data['has_visited_school_ast'] : $student->has_visited_school_ast;
            $student->save();
        }
        return $student;
    }

    /**
     * Método para guardar estado si el estudiante estudiante applicante es apto
     * para el proceso de admisiones
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-24
     * @return $student
     */
    public function saveProceedStudent($data){
        $student = ApplicantStudent::where('id_ast','=',$data['id_ast'])
            ->where('serial_scy','=',$data['serial_scy'])
            ->where('status_ast','=','ACTIVE')
            ->first();
        if ($student) {
            $student->status_applicant_ast = !empty($data['status_applicant_ast']) ? $data['status_applicant_ast'] : $student->status_applicant_ast;
            $student->save();
        }
        return $student;
    }

    /**
     * Método para guardar bandera 1 para el proceso de admisiones
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-24
     * @return $student
     */
    public function saveFlagOne($data){
        $student = ApplicantStudent::where('id_ast','=',$data['id_ast'])
            ->where('serial_scy','=',$data['serial_scy'])
            ->where('status_ast','=','ACTIVE')
            ->first();
        if ($student) {
            $student->flag1_ast = !empty($data['flag1_ast']) ? $data['flag1_ast'] : $student->flag1_ast;
            $student->save();
        }
        return $student;
    }

    /**
     * Método para guardar bandera 2 para el proceso de admisiones
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-24
     * @return $student
     */
    public function saveFlagTwo($data){
        $student = ApplicantStudent::where('id_ast','=',$data['id_ast'])
            ->where('serial_scy','=',$data['serial_scy'])
            ->where('status_ast','=','ACTIVE')
            ->first();
        if ($student) {
            $student->flag2_ast = !empty($data['flag2_ast']) ? $data['flag2_ast'] : $student->flag2_ast;
            $student->save();
        }
        return $student;
    }

    /**
     * Método para guardar bandera 3 para el proceso de admisiones
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-24
     * @return $student
     */
    public function saveFlagThree($data){
        $student = ApplicantStudent::where('id_ast','=',$data['id_ast'])
            ->where('serial_scy','=',$data['serial_scy'])
            ->where('status_ast','=','ACTIVE')
            ->first();
        if ($student) {
            $student->flag3_ast = !empty($data['flag3_ast']) ? $data['flag3_ast'] : $student->flag3_ast;
            $student->save();
        }
        return $student;
    }

    /**
     * Método para guardar la observacion para el proceso de admisiones
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-30
     * @return $student
     */
    public function saveObservation($data){
        $student = ApplicantStudent::where('id_ast','=',$data['id_ast'])
            ->where('serial_scy','=',$data['serial_scy'])
            ->where('status_ast','=','ACTIVE')
            ->first();
        if ($student) {
            $student->observation_ast = !empty($data['observation_ast']) ? $data['observation_ast'] : $student->observation_ast;
            $student->save();
        }
        return $student;
    }

    /**
     * Método para obtener los docentes asignados a una sección
     * por año lectivo
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-28
     * @return $employeeSection
     */
    public function getEmployeeSection($data){
        $student = ApplicantStudent::where('id_ast','=',$data['id_ast'])
            ->where('status_ast','=','ACTIVE')
            ->first();
        if ($student) {
            $student->flag3_ast = !empty($data['flag3_ast']) ? $data['flag3_ast'] : $student->flag3_ast;
            $student->save();
        }
        return $student;
    }

    /**
     * Método para obtener datos de la solicitud de un estudiante
     * aplicante por serial de aplicante
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-24
     * @return $applicants
     */
    public function getApplicant($data){
        $applicants = ApplicantStudent::where('applicant_student.serial_ast','=',$data['serial_ast'])
            ->where('applicant_student.status_ast','=','ACTIVE')
            ->join('applicant_guardian as agd','agd.serial_agd','=','applicant_student.serial_agd')
            ->join('level as lvl','lvl.serial_lvl','=','applicant_student.serial_lvl')
            ->join('section as set','set.serial_set','=','applicant_student.serial_set')
            ->join('school_year as scy','scy.serial_scy','=','applicant_student.serial_scy')
            ->join('country as cou','cou.serial_cou','=','applicant_student.serial_cou_last_school_ast')
            ->select('applicant_student.*','scy.name_scy','set.name_set','lvl.name_lvl','cou.name_cou','agd.*')
            ->first();

        return $applicants;
    }

    /**
     * Método para obtener lista aplicantes por año lectivo y sección
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-30
     * @return array $applicants
     */
    public function getAllApplicantBySection($section,$data){
        $startDate = Date('Y-m-d 00:00:00', strtotime($data['start_date']));
        $endDate = Date('Y-m-d 23:59:59', strtotime($data['end_date']));
        $applicants = ApplicantStudent::where('applicant_student.serial_scy','=',$data['serial_scy'])
            ->whereIn('applicant_student.serial_set',$section)
            ->where('status_ast','=','ACTIVE')
            ->where('applicant_student.created_at','>',$startDate)
            ->where('applicant_student.created_at','<',$endDate)
            ->join('applicant_guardian as agd','agd.serial_agd','=','applicant_student.serial_agd')
            ->join('level as lvl','lvl.serial_lvl','=','applicant_student.serial_lvl')
            ->join('section as set','set.serial_set','=','applicant_student.serial_set')
            ->join('school_year as scy','scy.serial_scy','=','applicant_student.serial_scy')
            ->join('country as cou','cou.serial_cou','=','applicant_student.serial_cou_last_school_ast')
            ->select('applicant_student.serial_ast','applicant_student.last_name_ast','first_name_ast','scy.name_scy','scy.serial_scy',
                'set.name_set','set.serial_set','lvl.serial_lvl','lvl.name_lvl',
                'applicant_student.id_ast','cou.name_cou','agd.last_name_agd','agd.first_name_agd','applicant_student.has_visited_school_ast',
                'flag1_ast','flag2_ast','flag3_ast','status_applicant_ast','observation_ast')
            ->selectRaw("DATE_FORMAT(applicant_student.created_at,'%d/%m/%Y') as date_app")
            ->orderBy('applicant_student.last_name_ast')
            ->get();
        return $applicants;
    }

    /**
     * Método para obtener lista aplicantes por año lectivo y sección
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-30
     * @return array $applicants
     */
    public function getAllApplicantByLevel($levels,$data){
        $startDate = Date('Y-m-d 00:00:00', strtotime($data['start_date']));
        $endDate = Date('Y-m-d 23:59:59', strtotime($data['end_date']));
        $applicants = ApplicantStudent::where('applicant_student.serial_scy','=',$data['serial_scy'])
            ->where('applicant_student.serial_set','>=',$data['serial_set'])
            ->whereIn('applicant_student.serial_lvl',$levels)
            ->where('status_ast','=','ACTIVE')
            ->where('applicant_student.created_at','>',$startDate)
            ->where('applicant_student.created_at','<',$endDate)
            ->join('applicant_guardian as agd','agd.serial_agd','=','applicant_student.serial_agd')
            ->join('level as lvl','lvl.serial_lvl','=','applicant_student.serial_lvl')
            ->join('section as set','set.serial_set','=','applicant_student.serial_set')
            ->join('school_year as scy','scy.serial_scy','=','applicant_student.serial_scy')
            ->join('country as cou','cou.serial_cou','=','applicant_student.serial_cou_last_school_ast')
            ->select('applicant_student.serial_ast','applicant_student.last_name_ast','first_name_ast','scy.name_scy','scy.serial_scy',
                'set.name_set','set.serial_set','lvl.serial_lvl','lvl.name_lvl',
                'applicant_student.id_ast','cou.name_cou','agd.last_name_agd','agd.first_name_agd','applicant_student.has_visited_school_ast',
                'flag1_ast','flag2_ast','flag3_ast','status_applicant_ast','observation_ast')
            ->selectRaw("DATE_FORMAT(applicant_student.created_at,'%d/%m/%Y') as date_app")
            ->orderBy('applicant_student.last_name_ast')
            ->get();
        return $applicants;
    }

    /**
     * Método para obtener lista aplicantes por representante
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-24
     * @return array $applicants
     */
    public function getApplicantsStudent($applicantGuardian){
        $applicants = ApplicantStudent::where('applicant_student.serial_agd','=',$applicantGuardian['serial_agd'])
            ->where('status_ast','=','ACTIVE')
            ->join('applicant_guardian as agd','agd.serial_agd','=','applicant_student.serial_agd')
            ->join('level as lvl','lvl.serial_lvl','=','applicant_student.serial_lvl')
            ->join('section as set','set.serial_set','=','applicant_student.serial_set')
            ->join('school_year as scy','scy.serial_scy','=','applicant_student.serial_scy')
            ->select('applicant_student.serial_ast','applicant_student.last_name_ast','first_name_ast','scy.name_scy','set.name_set','lvl.name_lvl',
                'lvl.serial_lvl','set.serial_set','scy.serial_scy','applicant_student.id_ast','agd.last_name_agd','agd.first_name_agd','applicant_student.created_at')
            ->orderBy('applicant_student.last_name_ast')
            ->get();

        return $applicants;
    }

    public function getApplicantsStudentAndEmployee($applicantStd){
        $applicants = ApplicantStudent::where('applicant_student.serial_ast','=',$applicantStd['serial_ast'])
            ->where('status_ast','=','ACTIVE')
            ->join('level as lvl','lvl.serial_lvl','=','applicant_student.serial_lvl')
            ->join('section as set','set.serial_set','=','applicant_student.serial_set')
            ->join('school_year as scy','scy.serial_scy','=','applicant_student.serial_scy')
            ->join('employee_section as emp_set','emp_set.serial_scy','=','scy.serial_scy')
            ->where('emp_set.serial_set','=',$applicantStd['serial_set'])
            ->where('emp_set.serial_lvl','=',$applicantStd['serial_lvl'])
            ->where('emp_set.type_user','=','APPLICANT_STUDENT_APPROVER')
            ->join('employee as emp','emp.serial_emp','=','emp_set.serial_emp')
            ->select('applicant_student.serial_ast','applicant_student.last_name_ast','first_name_ast','scy.name_scy','set.name_set','lvl.name_lvl',
                'lvl.serial_lvl','set.serial_set','scy.serial_scy','applicant_student.id_ast','emp_set.*','emp.first_name_emp','emp.last_name_emp','emp.email_emp')
            ->orderBy('applicant_student.last_name_ast')
            ->distinct()
            ->get();

        return $applicants;
    }

}