<?php

namespace App\Repositories;

use App\Models\DisciplineEquivalence;
use App\Repositories\BaseRepository;

/**
 * Repostiorio para disciplina equivalencia
 * @since   2018-03-28
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-28<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del archivo<br/>
 *          #edit1
 */
class DisciplineEquivalenceRepository extends BaseRepository {

    protected $modelClass = DisciplineEquivalence::class;

    //Declaración del constructor
    public function __construct(DisciplineEquivalence $disciplineEquivalence) {
        $this->disciplineEquivalence = $disciplineEquivalence;
    }



   /**
     * Método que devuelve las equivalencias por anio
     * @since   2018-03-28
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   integer $serial_scy anio lectivo
     * @return  array
     *
     */
    public function getDisciplineEquivalencesBySchoolYear($serial_scy) {

        $query = $this->newQuery();

        $query->where('serial_scy', $serial_scy);
        $query->where('status_deq', 'ACTIVE');
        $query->orderBy('higher_percentage_deq','DESC');
 
        return $this->doQueryGet($query);
    }

  
     /**
     * Método para obtener la equivalencia de comportamiento en porcentajes
     * @since   2018-03-28
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   array data
     * @return  array
     *
     */
    public function getGradeDisciplineEquivalenceBySchoolYear($serial_scy ,$grade) {

        $query = $this->newQuery();

        $query->where('serial_scy', $serial_scy);
        $query->where('lower_percentage_deq', '<=', $grade);
        $query->where('higher_percentage_deq', '>=', $grade);
        $query->where('status_deq', 'ACTIVE');


        return $this->doQueryFirst($query);
    }

}
