<?php

namespace App\Repositories;

use App\Models\Employee;
use App\Repositories\BaseRepository;

class EmployeeRepository extends BaseRepository {

    protected $modelClass = Employee::class;
    protected $employee;

    //Declaración del constructor
    public function __construct(Employee $employee) {
        $this->employee = $employee;
    }

    /**
     * Método que devuelve las materias optativas del empleado dado el serial del empleado y la fecha seleccionada
     * @since   2017-08-03
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   array $serial_emp serial del empleado
     * @param date $date fecha seleccionada
     * @return  ListObject $students
     * 
     * @edit    2017-08-03<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Creación del método<br/>
     *          #edit1
     */
    public function getOptativeSubjectsAndSchedulesByEmployee($serial_emp, $date) {
        $days = array('SUNDAY', 'MONDAY', 'TUESDAY', 'WEDNESDAY',
            'THURSDAY', 'FRIDAY', 'SATURDAY');
        $day = $days[date('N', strtotime($date))];
        $query = $this->newQuery();
        $query->join('optional_subject AS osb', function($join) use ($serial_emp) {
            $join->on('employee.serial_emp', '=', 'osb.serial_emp');
            $join->where('osb.serial_emp', '=', $serial_emp);
            $join->where('osb.status_osb', '=', 'ACTIVE');
        });
        $query->join('group_optional_subject AS gos', function($join) {
            $join->on('osb.serial_osb', '=', 'gos.serial_osb');
            $join->where('gos.status_gos', '=', 'ACTIVE');
        });
        $query->join('level_optional_group AS lop', function($join) {
            $join->on('gos.serial_lop', '=', 'lop.serial_lop');
            $join->where('status_lop', '=', 'ACTIVE');
        });
        $query->join('student_optional_subject AS sos', function($join) {
            $join->on('osb.serial_osb', '=', 'sos.serial_osb');
            $join->on('lop.serial_lop', '=', 'lop.serial_lop');
            $join->where('sos.status_sos', '=', 'ACTIVE');
        });
        $query->join('period AS prd', function($join) {
            $join->on('sos.serial_prd', '=', 'prd.serial_prd');
            $join->where('status_prd', '=', 'ACTIVE');
        });
        $query->join('optional_subject_level AS osl', function($join) {
            $join->on('osb.serial_osb', '=', 'osl.serial_osb');
            $join->where('osl.has_attendance', '=', '1');
            $join->where('osl.status_osl', '=', 'ACTIVE');
        });
        $query->join('subject_category AS scat', function($join) {
            $join->on('osl.serial_scat', '=', 'scat.serial_scat');
            $join->where('status_scat', '=', 'ACTIVE');
        });
        $query->join('cat_subject AS csb', function($join) {
            $join->on('osb.serial_csb', '=', 'csb.serial_csb');
            $join->where('status_csb', '=', 'ACTIVE');
        });
        $query->join('level AS lvl', function($join) {
            $join->on('osl.serial_lvl', '=', 'lvl.serial_lvl');
            $join->where('status_lvl', '=', 'ACTIVE');
        });
        $query->join('section', function($join) {
            $join->on('osl.serial_set', '=', 'section.serial_set');
            $join->where('status_set', '=', 'ACTIVE');
        });
        $query->join('schedule', function($join) use($day) {
            $join->on('lop.serial_lop', '=', 'schedule.serial_lop');
            $join->where('status_scd', '=', 'ACTIVE');
            $join->where('day_scd', $day);
        });
        $query->join('course', function($join) {
            $join->on('schedule.serial_crs', '=', 'course.serial_crs');
            $join->where('course.attendance_crs', '=', 'CLASS');
            $join->where('course.status_crs', '=', 'ACTIVE');
        });
        $query->join('schedule_header', function($join) use ($date) {
            $join->on('schedule.serial_shd', '=', 'schedule_header.serial_shd');
            $join->where('status_shd', '=', 'ACTIVE');
            $join->where('date_start_shd', '<=', $date);
            $join->where('date_end_shd', '>=', $date);
        });
        $query->join('credits_detail', function($join) {
            $join->on('schedule.serial_cdt', '=', 'credits_detail.serial_cdt');
            $join->where('status_cdt', '=', 'ACTIVE');
        });
        $query->join('school_year', 'lop.serial_scy', '=', 'school_year.serial_scy');
        $query->where('school_year.status_scy', 'like', 'ACTIVE,CURRENT');
        $query->orWhere('school_year.status_scy', 'like', 'CURRENT,ACTIVE');
        $query->groupBy('serial_osb');
        $query->groupBy('credits_detail.serial_cdt');
        $query->orderBy('end_at_cdt','asc');
        $query->distinct();
        $query->selectRaw("null as serial_crs,section.name_set, lvl.name_lvl,null as name_clg ,lop.serial_lop,osb.serial_osb,csb.name_csb,null as serial_sbc,schedule.serial_scd,credits_detail.start_at_cdt,credits_detail.end_at_cdt,'optative' as type_subject");
        return $this->doQueryGet($query);
    }

    public function getOptativeSubjectsAndSchedulesByEmployeeDay($serial_emp, $date) {
        $days = array('SUNDAY', 'MONDAY', 'TUESDAY', 'WEDNESDAY',
            'THURSDAY', 'FRIDAY', 'SATURDAY');
        $day = $days[date('N', strtotime($date))];
        $query = $this->newQuery();
        $query->join('optional_subject AS osb', function($join) use ($serial_emp) {
            $join->on('employee.serial_emp', '=', 'osb.serial_emp');
            $join->where('osb.serial_emp', '=', $serial_emp);
            $join->where('osb.status_osb', '=', 'ACTIVE');
        });
        $query->join('group_optional_subject AS gos', function($join) {
            $join->on('osb.serial_osb', '=', 'gos.serial_osb');
            $join->where('gos.status_gos', '=', 'ACTIVE');
        });
        $query->join('level_optional_group AS lop', function($join) {
            $join->on('gos.serial_lop', '=', 'lop.serial_lop');
            $join->where('status_lop', '=', 'ACTIVE');
        });
        $query->join('student_optional_subject AS sos', function($join) {
            $join->on('osb.serial_osb', '=', 'sos.serial_osb');
            $join->on('lop.serial_lop', '=', 'lop.serial_lop');
            $join->where('sos.status_sos', '=', 'ACTIVE');
        });
        $query->join('period AS prd', function($join) {
            $join->on('sos.serial_prd', '=', 'prd.serial_prd');
            $join->where('status_prd', '=', 'ACTIVE');
        });
        $query->join('optional_subject_level AS osl', function($join) {
            $join->on('osb.serial_osb', '=', 'osl.serial_osb');
            $join->where('osl.has_attendance', '=', '1');
            $join->where('osl.status_osl', '=', 'ACTIVE');
        });
        $query->join('subject_category AS scat', function($join) {
            $join->on('osl.serial_scat', '=', 'scat.serial_scat');
            $join->where('status_scat', '=', 'ACTIVE');
        });
        $query->join('cat_subject AS csb', function($join) {
            $join->on('osb.serial_csb', '=', 'csb.serial_csb');
            $join->where('status_csb', '=', 'ACTIVE');
        });
        $query->join('level AS lvl', function($join) {
            $join->on('osl.serial_lvl', '=', 'lvl.serial_lvl');
            $join->where('status_lvl', '=', 'ACTIVE');
        });
        $query->join('section', function($join) {
            $join->on('osl.serial_set', '=', 'section.serial_set');
            $join->where('status_set', '=', 'ACTIVE');
        });
        $query->join('schedule', function($join) use($day) {
            $join->on('lop.serial_lop', '=', 'schedule.serial_lop');
            $join->where('status_scd', '=', 'ACTIVE');
            $join->where('day_scd', $day);
        });
        $query->join('course', function($join) {
            $join->on('schedule.serial_crs', '=', 'course.serial_crs');
            $join->where('course.attendance_crs', '=', 'DAY');
            $join->where('course.status_crs', '=', 'ACTIVE');
        });
        $query->join('schedule_header', function($join) use ($date) {
            $join->on('schedule.serial_shd', '=', 'schedule_header.serial_shd');
            $join->where('status_shd', '=', 'ACTIVE');
            $join->where('date_start_shd', '<=', $date);
            $join->where('date_end_shd', '>=', $date);
        });
        $query->join('credits_detail', function($join) {
            $join->on('schedule.serial_cdt', '=', 'credits_detail.serial_cdt');
            $join->where('status_cdt', '=', 'ACTIVE');
        });
        $query->join('school_year', 'lop.serial_scy', '=', 'school_year.serial_scy');
        $query->where('school_year.status_scy', 'like', 'ACTIVE,CURRENT');
        $query->orWhere('school_year.status_scy', 'like', 'CURRENT,ACTIVE');
        $query->distinct();
        $query->groupBy('serial_osb');
        $query->orderBy('end_at_cdt','asc');
        $query->groupBy('credits_detail.serial_cdt');
        $query->orderBy('end_at_cdt','asc');
        $query->limit(1);
        $query->selectRaw("null as serial_crs,section.name_set, lvl.name_lvl,null as name_clg ,lop.serial_lop,osb.serial_osb,csb.name_csb,null as serial_sbc,schedule.serial_scd,credits_detail.start_at_cdt,credits_detail.end_at_cdt,'optative' as type_subject,day_scd");
        return $this->doQueryGet($query);
    }

    /**
     * Método que devuelve las materias obligatorias del empleado dado el serial del empleado y la fecha seleccionada
     * @since   2017-08-03
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @param   array $serial_emp serial del empleado
     * @param date $date fecha seleccionada
     * @return  ListObject $students
     * 
     * @edit    2017-09-05<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          Ordenamiento de horarios<br/>
     *          #edit1
     */
    public function getMandatorySubjectsAndSchedulesByEmployee($serial_emp, $date) {
        $days = array('SUNDAY', 'MONDAY', 'TUESDAY', 'WEDNESDAY',
            'THURSDAY', 'FRIDAY', 'SATURDAY');
        $day = $days[date('N', strtotime($date))];

        $query = $this->newQuery();
        $query->join('subject_course AS sbc', function($join) use($serial_emp) {
            $join->on('employee.serial_emp', '=', 'sbc.serial_emp');
            $join->where('sbc.serial_emp', '=', $serial_emp);
            $join->where('sbc.has_attendance', '=', 1);
            $join->where('has_opt_subject_relation', '=', 0);
            $join->where('sbc.status_sbc', '=', 'ACTIVE');
        });
        $query->join('subject AS sbj', function($join) {
            $join->on('sbc.serial_sbj', '=', 'sbj.serial_sbj');
            $join->where('sbj.status_sbj', '=', 'ACTIVE');
        });
        $query->join('cat_subject as csb', function($join) {
            $join->on('sbj.serial_csb', '=', 'csb.serial_csb');
            $join->where('csb.status_csb', '=', 'ACTIVE');
        });
        $query->leftjoin('cat_subject as csb2', function($join) {
            $join->on('csb2.serial_csb', '=', 'csb.serial_csb');
            $join->where('csb2.status_csb', '=', 'ACTIVE');
        });
        $query->join('course AS crs', function($join) {
            $join->on('sbc.serial_crs', '=', 'crs.serial_crs');
            $join->where('crs.attendance_crs', '=', 'CLASS');
            $join->where('crs.status_crs', '=', 'ACTIVE');
        });
        $query->join('section', function($join) {
            $join->on('crs.serial_set', '=', 'section.serial_set');
            $join->where('section.status_set', '=', 'ACTIVE');
        });
        $query->join('level AS lvl', function($join) {
            $join->on('crs.serial_lvl', '=', 'lvl.serial_lvl');
            $join->where('crs.status_crs', '=', 'ACTIVE');
        });
        $query->join('class_group As clg', function($join) {
            $join->on('crs.serial_clg', '=', 'clg.serial_clg');
            $join->where('status_clg', '=', 'ACTIVE');
        });
        $query->join('schedule', function($join) use($serial_emp, $day) {
            $join->on('sbc.serial_sbc', '=', 'schedule.serial_sbc');
            $join->on('crs.serial_crs', '=', 'schedule.serial_crs');
            $join->where('status_scd', '=', 'ACTIVE');
            $join->where('day_scd', $day);
        });
        $query->join('schedule_header', function($join) use ($date) {
            $join->on('schedule.serial_shd', '=', 'schedule_header.serial_shd');
            $join->where('status_shd', '=', 'ACTIVE');
            $join->where('date_start_shd', '<=', $date);
            $join->where('date_end_shd', '>=', $date);
        });
        $query->join('credits_detail', function($join) {
            $join->on('credits_detail.serial_cdt', '=', 'schedule.serial_cdt');
            $join->where('status_cdt', '=', 'ACTIVE');
        });
        $query->join('school_year', 'crs.serial_scy', '=', 'school_year.serial_scy');
        $query->where('status_scy', 'like', 'ACTIVE,CURRENT');
        $query->orWhere('status_scy', 'like', 'CURRENT,ACTIVE');
        $query->orderBy('start_at_cdt');
        $query->distinct();
        $query->selectRaw("section.name_set,lvl.name_lvl,clg.name_clg,crs.serial_crs,sbc.serial_sbc,csb.name_csb,'null' as serial_osb,schedule.serial_scd,start_at_cdt,end_at_cdt,'mandatory' as type_subject");
        return $this->doQueryGet($query);
    }

    public function getMandatorySubjectsAndSchedulesByEmployeeDay($serial_emp, $date) {
        $days = array('SUNDAY', 'MONDAY', 'TUESDAY', 'WEDNESDAY',
            'THURSDAY', 'FRIDAY', 'SATURDAY');
        $day = $days[date('N', strtotime($date))];

        $query = $this->newQuery();
        $query->join('subject_course AS sbc', function($join) use($serial_emp) {
            $join->on('employee.serial_emp', '=', 'sbc.serial_emp');
            $join->where('sbc.serial_emp', '=', $serial_emp);
            $join->where('sbc.has_attendance', '=', 1);
            $join->where('has_opt_subject_relation', '=', 0);
            $join->where('sbc.status_sbc', '=', 'ACTIVE');
        });
        $query->join('subject AS sbj', function($join) {
            $join->on('sbc.serial_sbj', '=', 'sbj.serial_sbj');
            $join->where('sbj.status_sbj', '=', 'ACTIVE');
        });
        $query->join('cat_subject as csb', function($join) {
            $join->on('sbj.serial_csb', '=', 'csb.serial_csb');
            $join->where('csb.status_csb', '=', 'ACTIVE');
        });
        $query->leftjoin('cat_subject as csb2', function($join) {
            $join->on('csb2.serial_csb', '=', 'csb.serial_csb');
            $join->where('csb2.status_csb', '=', 'ACTIVE');
        });
        $query->join('course AS crs', function($join) {
            $join->on('sbc.serial_crs', '=', 'crs.serial_crs');
            $join->where('crs.attendance_crs', '=', 'DAY');
            $join->where('crs.status_crs', '=', 'ACTIVE');
        });
        $query->join('section', function($join) {
            $join->on('crs.serial_set', '=', 'section.serial_set');
            $join->where('section.status_set', '=', 'ACTIVE');
        });
        $query->join('level AS lvl', function($join) {
            $join->on('crs.serial_lvl', '=', 'lvl.serial_lvl');
            $join->where('crs.status_crs', '=', 'ACTIVE');
        });
        $query->join('class_group As clg', function($join) {
            $join->on('crs.serial_clg', '=', 'clg.serial_clg');
            $join->where('status_clg', '=', 'ACTIVE');
        });
        $query->join('schedule', function($join) use($serial_emp, $day) {
            $join->on('sbc.serial_sbc', '=', 'schedule.serial_sbc');
            $join->on('crs.serial_crs', '=', 'schedule.serial_crs');
            $join->where('status_scd', '=', 'ACTIVE');
            $join->where('day_scd', $day);
        });
        $query->join('schedule_header', function($join) use ($date) {
            $join->on('schedule.serial_shd', '=', 'schedule_header.serial_shd');
            $join->where('status_shd', '=', 'ACTIVE');
            $join->where('date_start_shd', '<=', $date);
            $join->where('date_end_shd', '>=', $date);
        });
        $query->join('credits_detail', function($join) {
            $join->on('credits_detail.serial_cdt', '=', 'schedule.serial_cdt');
            $join->where('status_cdt', '=', 'ACTIVE');
        });
        $query->join('school_year', 'crs.serial_scy', '=', 'school_year.serial_scy');
        $query->where('status_scy', 'like', 'ACTIVE,CURRENT');
        $query->orWhere('status_scy', 'like', 'CURRENT,ACTIVE');
        $query->orderBy('start_at_cdt');
        $query->distinct();
        $query->selectRaw("section.name_set,lvl.name_lvl,clg.name_clg,crs.serial_crs,sbc.serial_sbc,csb.name_csb,'null' as serial_osb,schedule.serial_scd,start_at_cdt,end_at_cdt,'mandatory' as type_subject,day_scd");
        $query->limit(1);
        return $this->doQueryGet($query);
    }

    public function getEmployeeInfo($serial_usr) {
        $employee = $this->newQuery();
        $employee->join('user', 'employee.serial_usr', '=', 'user.serial_usr');
        $employee->where('status_emp', '=', 'ACTIVE');
        $employee->where('user.serial_usr', '=', $serial_usr);
        $employee->selectRaw('employee.*');
        return $this->doQueryFirst($employee);
    }

    /**
     * Método que devuelve las materias optativas para incidencia del empleado dado el serial del empleado y la fecha seleccionada
     * @since   2018-03-22
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   array $serial_emp serial del empleado
     * @param date $date fecha seleccionada
     * @param string $day dia correspondiente a configuracion de dias
     * @return  ListObject $students
     * 
     * @edit    2018-04-26<br />
     *          Erick Vaca <epvaca@rutatec.com><br />
     *          Se agrego parametro dia<br/>
     *          #edit1     
     */
    public function getIncidenceOptativeSubjectsAndSchedulesByEmployee($serial_emp, $date, $day) {

        $query = $this->newQuery();
        $query->join('optional_subject AS osb', function($join) use ($serial_emp) {
            $join->on('employee.serial_emp', '=', 'osb.serial_emp');
            $join->where('osb.serial_emp', '=', $serial_emp);
            $join->where('osb.status_osb', '=', 'ACTIVE');
        });
        $query->join('group_optional_subject AS gos', function($join) {
            $join->on('osb.serial_osb', '=', 'gos.serial_osb');
            $join->where('gos.status_gos', '=', 'ACTIVE');
        });
        $query->join('level_optional_group AS lop', function($join) {
            $join->on('gos.serial_lop', '=', 'lop.serial_lop');
            $join->where('status_lop', '=', 'ACTIVE');
        });
        $query->join('student_optional_subject AS sos', function($join) {
            $join->on('osb.serial_osb', '=', 'sos.serial_osb');
            $join->on('lop.serial_lop', '=', 'lop.serial_lop');
            $join->where('sos.status_sos', '=', 'ACTIVE');
        });
        $query->join('period AS prd', function($join) {
            $join->on('sos.serial_prd', '=', 'prd.serial_prd');
            $join->where('status_prd', '=', 'ACTIVE');
        });
        $query->join('optional_subject_level AS osl', function($join) {
            $join->on('osb.serial_osb', '=', 'osl.serial_osb');
            $join->where('osl.status_osl', '=', 'ACTIVE');
        });
        $query->join('subject_category AS scat', function($join) {
            $join->on('osl.serial_scat', '=', 'scat.serial_scat');
            $join->where('status_scat', '=', 'ACTIVE');
        });
        $query->join('cat_subject AS csb', function($join) {
            $join->on('osb.serial_csb', '=', 'csb.serial_csb');
            $join->where('status_csb', '=', 'ACTIVE');
        });
        $query->join('level AS lvl', function($join) {
            $join->on('osl.serial_lvl', '=', 'lvl.serial_lvl');
            $join->where('status_lvl', '=', 'ACTIVE');
        });
        $query->join('section', function($join) {
            $join->on('osl.serial_set', '=', 'section.serial_set');
            $join->where('status_set', '=', 'ACTIVE');
        });
        $query->join('schedule', function($join) use($day) {
            $join->on('lop.serial_lop', '=', 'schedule.serial_lop');
            $join->where('status_scd', '=', 'ACTIVE');
            $join->where('day_scd', $day);
        });
        $query->join('incidence_year_section_level AS iyl', function($join) {
            $join->on('osl.serial_scy', '=', 'iyl.serial_scy');
            $join->on('osl.serial_set', '=', 'iyl.serial_set');
            $join->on('osl.serial_lvl', '=', 'iyl.serial_lvl');
            $join->where('status_iyl', '=', 'ACTIVE');
        });
        $query->join('schedule_header', function($join) use ($date) {
            $join->on('schedule.serial_shd', '=', 'schedule_header.serial_shd');
            $join->where('status_shd', '=', 'ACTIVE');
            $join->where('date_start_shd', '<=', $date);
            $join->where('date_end_shd', '>=', $date);
        });
        $query->join('credits_detail', function($join) {
            $join->on('schedule.serial_cdt', '=', 'credits_detail.serial_cdt');
            $join->where('status_cdt', '=', 'ACTIVE');
        });

        $query->join('school_year', 'lop.serial_scy', '=', 'school_year.serial_scy');
        $query->where('school_year.status_scy', 'like', 'ACTIVE,CURRENT');
        $query->orWhere('school_year.status_scy', 'like', 'CURRENT,ACTIVE');
        $query->orderBy('start_at_cdt');
        $query->distinct();
        $query->selectRaw("section.name_set, lvl.name_lvl,null as name_clg ,lop.serial_lop,osb.serial_osb,csb.name_csb as name_csb,null as serial_sbc,schedule.serial_scd,credits_detail.start_at_cdt,credits_detail.end_at_cdt,'Optativa' as type_subject, 
            lop.serial_scy, lop.serial_set, lop.serial_lvl, lop.serial_spt, schedule.serial_cdt ");
        return $this->doQueryGet($query);
    }

    /**
     * Método que devuelve las materias obligatorias para incidencias del empleado dado el serial del empleado y la fecha seleccionada
     * @since   2018-03-22
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   array $serial_emp serial del empleado
     * @param date $date fecha seleccionada
     * @param string $day dia correspondiente a configuracion de dias
     * @return  ListObject $students
     * 
     * @edit    2018-04-26<br />
     *          Erick Vaca <epvaca@rutatec.com><br />
     *          #edit1
     *          Se agrego parametro dia
     */
    public function getIncidenceMandatorySubjectsAndSchedulesByEmployee($serial_emp, $date, $day) {

        $query = $this->newQuery();
        $query->join('subject_course AS sbc', function($join) use($serial_emp) {
            $join->on('employee.serial_emp', '=', 'sbc.serial_emp');
            $join->where('sbc.serial_emp', '=', $serial_emp);
            $join->where('has_opt_subject_relation', '=', 0);
            $join->where('sbc.status_sbc', '=', 'ACTIVE');
        });
        $query->join('subject AS sbj', function($join) {
            $join->on('sbc.serial_sbj', '=', 'sbj.serial_sbj');
            $join->where('sbj.status_sbj', '=', 'ACTIVE');
        });
        $query->join('cat_subject as csb', function($join) {
            $join->on('sbj.serial_csb', '=', 'csb.serial_csb');
            $join->where('csb.status_csb', '=', 'ACTIVE');
        });
        $query->leftjoin('cat_subject as csb2', function($join) {
            $join->on('csb2.serial_csb', '=', 'csb.serial_csb');
            $join->where('csb2.status_csb', '=', 'ACTIVE');
        });
        $query->join('course AS crs', function($join) {
            $join->on('sbc.serial_crs', '=', 'crs.serial_crs');
            $join->where('crs.status_crs', '=', 'ACTIVE');
        });
        $query->join('section', function($join) {
            $join->on('crs.serial_set', '=', 'section.serial_set');
            $join->where('section.status_set', '=', 'ACTIVE');
        });
        $query->join('level AS lvl', function($join) {
            $join->on('crs.serial_lvl', '=', 'lvl.serial_lvl');
            $join->where('crs.status_crs', '=', 'ACTIVE');
        });
        $query->join('class_group As clg', function($join) {
            $join->on('crs.serial_clg', '=', 'clg.serial_clg');
            $join->where('status_clg', '=', 'ACTIVE');
        });
        $query->join('incidence_year_section_level AS iyl', function($join) {
            $join->on('crs.serial_scy', '=', 'iyl.serial_scy');
            $join->on('crs.serial_set', '=', 'iyl.serial_set');
            $join->on('crs.serial_lvl', '=', 'iyl.serial_lvl');
            $join->where('status_iyl', '=', 'ACTIVE');
        });
        $query->join('schedule', function($join) use($serial_emp, $day) {
            $join->on('sbc.serial_sbc', '=', 'schedule.serial_sbc');
            $join->on('crs.serial_crs', '=', 'schedule.serial_crs');
            $join->where('status_scd', '=', 'ACTIVE');
            $join->where('day_scd', $day);
        });
        $query->join('schedule_header', function($join) use ($date) {
            $join->on('schedule.serial_shd', '=', 'schedule_header.serial_shd');
            $join->where('status_shd', '=', 'ACTIVE');
            $join->where('date_start_shd', '<=', $date);
            $join->where('date_end_shd', '>=', $date);
        });
        $query->join('credits_detail', function($join) {
            $join->on('credits_detail.serial_cdt', '=', 'schedule.serial_cdt');
            $join->where('status_cdt', '=', 'ACTIVE');
        });
        $query->join('school_year', 'crs.serial_scy', '=', 'school_year.serial_scy');
        $query->where('status_scy', 'like', 'ACTIVE,CURRENT');
        $query->orWhere('status_scy', 'like', 'CURRENT,ACTIVE');
        $query->orderBy('start_at_cdt');
        $query->orderBy('sbc.serial_sbc');
        $query->orderBy('crs.serial_crs');
        $query->distinct();
        $query->selectRaw("section.name_set,lvl.name_lvl,clg.name_clg,crs.serial_crs,sbc.serial_sbc,csb.name_csb,'null' as serial_osb,schedule.serial_scd,start_at_cdt,end_at_cdt,'Obligatoria' as type_subject , crs.serial_scy, crs.serial_set, crs.serial_lvl, crs.serial_spt  ");
        return $this->doQueryGet($query);
    }

    /**
     * Método que devuelve el empleado en base al usuario
     * @since   2018-03-22
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   array $serial_usr serial del usuario
     * @return  ListObject 
     * 
     */
    public function getEmployeeBySerialUsr($serial_usr) {
        $query = $this->newQuery();
        $query->where('serial_usr', $serial_usr);
        return $this->doQueryFirst($query);
    }

    /**
     * Método que retorna docentes por seccion, nivel , especialidad, anio lectivo
     * @since   2018-04-07
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   void
     * @return  ListObject
     * 
     */
    public function getTeachersBySchoolYearSectionLevelSpecialty($serial_scy, $serial_set, $serial_lvl, $serial_spt) {

        $query = $this->newQuery();
        $query->join('subject_course as sbc', function($join) {
            $join->on('sbc.serial_emp', '=', 'employee.serial_emp');
            $join->where('sbc.status_sbc', '=', 'ACTIVE');
        });

        $query->join('subject as sbj', function($join) use($serial_scy, $serial_set, $serial_spt, $serial_lvl) {
            $join->on('sbj.serial_sbj', '=', 'sbc.serial_sbj');
            $join->where('sbj.status_sbj', '=', 'ACTIVE');
            $join->where('sbj.serial_scy', '=', $serial_scy);
            $join->where('sbj.serial_set', '=', $serial_set);
            $join->where('sbj.serial_spt', '=', $serial_spt);
            $join->where('sbj.serial_lvl', '=', $serial_lvl);
        });

        $query->distinct();
        $query->selectRaw('employee.serial_emp as serial_emp, employee.first_name_emp AS first_name_emp, employee.last_name_emp as last_name_emp, serial_crs, sbj.serial_scy, sbj.serial_set, sbj.serial_lvl, sbj.serial_spt');


        $query2 = $this->newQuery();
        $query2->join('optional_subject as osb', function($join) {
            $join->on('osb.serial_emp', '=', 'employee.serial_emp');
            $join->where('osb.status_osb', '=', 'ACTIVE');
        });
        $query2->join('optional_subject_level as osl', function($join) use($serial_scy, $serial_set, $serial_spt, $serial_lvl) {
            $join->on('osl.serial_osb', '=', 'osb.serial_osb');
            $join->where('osl.status_osl', '=', 'ACTIVE');
            $join->where('osl.serial_scy', '=', $serial_scy);
            $join->where('osl.serial_set', '=', $serial_set);
            $join->where('osl.serial_spt', '=', $serial_spt);
            $join->where('osl.serial_lvl', '=', $serial_lvl);
        });

        $query2->distinct();
        $query2->selectRaw('employee.serial_emp as serial_emp, employee.first_name_emp AS first_name_emp, employee.last_name_emp as last_name_emp, null as serial_crs, osl.serial_scy, osl.serial_set,osl.serial_lvl, osl.serial_spt');

        $query->orderBy('last_name_emp');
        $query->union($query2);


        return $this->doQueryGet($query)->toArray();
    }

    /**
     * Método para obtener section employee by serial_emp, serial_scy y types
     * @since   2018-04-08
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_emp serial del empleado
     * @param   int $serial_scy serial del anio
     * @param   array $type tipos de usuario
     * @return  array
     * 
     */
    public function employeeSectionBySerialEmpSerialScyTypes($serial_emp, $serial_scy, $type) {

        $query = $this->newQuery();

        $query->join('employee_section as es', 'es.serial_emp', '=', 'employee.serial_emp');
        $query->where('es.status', 'ACTIVE');
        $query->where('employee.status_emp', 'ACTIVE');
        $query->where('es.serial_scy', $serial_scy);
        $query->where('es.serial_emp', $serial_emp);
        $query->whereIn('es.type_user', $type);

        $query->selectRaw('es.serial_set,es.serial_lvl');
        $query->orderBy('es.serial_set');
        $query->orderBy('es.serial_lvl');

        return $this->doQueryGet($query);
    }

    /**
     * Método para obtener course employee by serial_emp, serial_scy y employeeTypes
     * @since   2018-04-08
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_emp serial del empleado
     * @param   int $serial_scy serial del anio
     * @param   array $employeeTypes tipos de empleado
     * @return  array
     * 
     */
    public function courseEmployeeBySerialEmpSerialScyEmployeeTypes($serial_emp, $serial_scy, $employeeTypes) {

        $query = $this->newQuery();

        $query->join('course_employee as ce', 'ce.serial_emp', '=', 'employee.serial_emp');
        $query->join('course as c', 'c.serial_crs', '=', 'ce.serial_crs');
        $query->where('ce.status_cem', 'ACTIVE');
        $query->where('c.status_crs', 'ACTIVE');
        $query->where('employee.status_emp', 'ACTIVE');
        $query->where('ce.serial_emp', $serial_emp);
        $query->whereIn('ce.employee_type', $employeeTypes);
        $query->where('c.serial_scy', $serial_scy);

        $query->selectRaw('c.*');
        $query->orderBy('c.serial_crs');

        return $this->doQueryGet($query);
    }

    /**
     * Método que devuelve informacion del empleado en base a serial_emp
     * @since   2018-07-19
     * @author  Erick Vaca <epvaca@rutatec.com>
     *
     * @param   array $serial_emp serial del empleado
     * @return  ListObject 
     * 
     */
    public function getEmployeeBySerialEmp($serial_emp) {
        $query = $this->newQuery();

        $query->join('user as u', 'u.serial_emp', '=', 'employee.serial_emp');

        $query->where('employee.serial_emp', $serial_emp);
        return $this->doQueryFirst($query);
    }

    /**
     * Método para obtener los docentes asignados a una sección
     * por año lectivo y serial de usuario, si el usuario no tiene
     * asignado una sección, se obtiene los datos del usuario y TODAS
     * las secciones del año lectivo
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-28
     * @return $employeeSection
     */
    public function getEmployeeSection($data) {
        $employeeSection = Employee::where('employee.serial_usr','=',$data['serial_usr'])
            ->where('status_emp','=','ACTIVE')
            ->join('user as usr','usr.serial_usr','=','employee.serial_usr')
            ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','employee.serial_emp')
            ->leftJoin('school_year as scy','scy.serial_scy','=','emp_set.serial_scy')
            ->leftJoin('section as set','set.serial_set','=','emp_set.serial_set')
            ->leftJoin('course_employee as cem','cem.serial_emp','=','employee.serial_emp')
            ->where('emp_set.serial_scy','=',$data['serial_scy'])
            ->where('emp_set.status','=','ACTIVE')
            ->select('set.serial_set','set.name_set')
            ->distinct()
            ->pluck('set.name_set','set.serial_set');
        if (count($employeeSection)) {
            return $employeeSection;
        } else {
            $employeeSection = Employee::where('employee.serial_usr','=',$data['serial_usr'])
                ->where('status_emp','=','ACTIVE')
                ->join('user as usr','usr.serial_usr','=','employee.serial_usr')
                ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','employee.serial_emp')
                ->join('level_section_year as pgy','pgy.serial_scy','=','pgy.serial_scy')
                ->where('pgy.serial_scy','=',$data['serial_scy'])
                ->join('school_year as scy','scy.serial_scy','=','pgy.serial_scy')
                ->join('section as set','set.serial_set','=','pgy.serial_set')
                ->leftJoin('course_employee as cem','cem.serial_emp','=','employee.serial_emp')
                ->select('set.serial_set','set.name_set')
                ->distinct()
                ->pluck('set.name_set','set.serial_set');
            return $employeeSection;
        }
    }

    /**
     * Método para obtener los docentes asignados a un nivel
     * por sección, año lectivo y usuario
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-29
     * @return json $employeeSection
     */

    public function getEmployeeLevel($data) {
        $employeeSection = Employee::where('employee.serial_usr','=',$data['serial_usr'])
            ->where('status_emp','=','ACTIVE')
            ->join('user as usr','usr.serial_usr','=','employee.serial_usr')
            ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','employee.serial_emp')
            ->leftJoin('school_year as scy','scy.serial_scy','=','emp_set.serial_scy')
            ->leftJoin('section as set','set.serial_set','=','emp_set.serial_set')
            ->leftJoin('level as lvl','lvl.serial_lvl','=','emp_set.serial_lvl')
            ->leftJoin('course_employee as cem','cem.serial_emp','=','employee.serial_emp')
            ->where('emp_set.serial_scy','=',$data['serial_scy'])
            ->where('emp_set.serial_scy','=',$data['serial_set'])
            ->where('emp_set.status','=','ACTIVE')
            ->select('lvl.serial_lvl','lvl.name_lvl')
            ->distinct()
            ->pluck('lvl.name_lvl','lvl.serial_lvl');
        if (count($employeeSection)) {
            return $employeeSection;
        } else {
            $employeeSection = Employee::where('employee.serial_usr','=',$data['serial_usr'])
                ->where('status_emp','=','ACTIVE')
                ->join('user as usr','usr.serial_usr','=','employee.serial_usr')
                ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','employee.serial_emp')
                ->join('level_section_year as pgy','pgy.serial_scy','=','pgy.serial_scy')
                ->where('pgy.serial_scy','=',$data['serial_scy'])
                ->where('pgy.serial_set','=',$data['serial_set'])
                ->join('school_year as scy','scy.serial_scy','=','pgy.serial_scy')
                ->join('section as set','set.serial_set','=','pgy.serial_set')
                ->join('level as lvl','lvl.serial_lvl','=','pgy.serial_lvl')
                ->leftJoin('course_employee as cem','cem.serial_emp','=','employee.serial_emp')
                ->select('lvl.serial_lvl','lvl.name_lvl')
                ->distinct()
                ->pluck('lvl.name_lvl','lvl.serial_lvl');
            return $employeeSection;
        }
    }


}
