<?php

namespace App\Repositories;

use App\Models\IncidenceYearSectionLevel;
use App\Repositories\BaseRepository;

/**
 * Repositorio para manejo de Incidencia Nivel Seccion
 * @since   2018-03-15
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-15<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del archivo<br/>
 *          #edit1
 */
class IncidenceYearSectionLevelRepository extends BaseRepository {

    protected $modelClass = IncidenceYearSectionLevel::class;

    //Declaración del constructor
    public function __construct(IncidenceYearSectionLevel $incidenceYearSectionLevel) {
        $this->incidenceYearSectionLevel = $incidenceYearSectionLevel;
    }
  

    /**
     * Método que guarda incidencia nivel anio
     * @since   2018-03-15
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $data
     * @return  
     * 
     */
    public function saveIncidenceYearSectionLevel($data) {

        //se realiza la búsqueda para determinar si existe ya existe el registro
        $iyl = IncidenceYearSectionLevel::where('serial_scy', '=', $data['serial_scy'])
                ->where('serial_lvl', '=', $data['serial_lvl']) 
                ->where('serial_set', '=', $data['serial_set'])
                ->first();

         //se actualiza si se encontró ya un registro
        if ($iyl != null) {
                $iyl->max_time_iyl = $data['max_time_iyl'];            
                $iyl->status_iyl = $data['status_iyl'];                       
                $iyl->save();            
        }else{
            if($data['status_iyl']=='ACTIVE'){
                //Crea nuevo registro
                $iyl = new IncidenceYearSectionLevel;
                $iyl->serial_scy = $data['serial_scy'];
                $iyl->serial_set = $data['serial_set'];
                $iyl->serial_lvl = $data['serial_lvl'];                               
                $iyl->max_time_iyl = $data['max_time_iyl'];
                $iyl->status_iyl = 'ACTIVE';
                $iyl->save();
            }
        }          
    }


    /**
     * Método para obtener incidencia nivel por anio y seccion
     * @since   2018-03-15
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @param   int $serial_set serial de la seccion     
     * @return  incidence_setup array
     * 
     */
    public function getIncidenceLevelBySchoolYearSection($serial_scy,$serial_set) {

        $iyl = $this->newQuery();        
        $iyl->where('serial_scy', '=', $serial_scy);
        $iyl->where('serial_set', '=', $serial_set);
        $iyl->where('status_iyl', '=', 'ACTIVE');                          
        return $this->doQueryGet($iyl);
    }


        /**
     * Método para obtener incidencia nivel por anio , seccion y nivel
     * @since   2018-03-19
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   int $serial_scy serial del anio
     * @param   int $serial_set serial de la seccion     
     * @param   int $serial_lvl serial del nivel
     * @return  incidence_setup array
     * 
     */
    public function getIncidenceLevelBySchoolYearSectionLevel($serial_scy,$serial_set, $serial_lvl) {

        $iyl = $this->newQuery();        
        $iyl->where('serial_scy', '=', $serial_scy);
        $iyl->where('serial_set', '=', $serial_set);
        $iyl->where('serial_lvl', '=', $serial_lvl);
        $iyl->where('status_iyl', '=', 'ACTIVE');                          
        return $this->doQueryGet($iyl);
    }

}
