<?php

namespace App\Repositories;

use App\Models\LevelSectionYear;

class LevelSectionYearRepository extends BaseRepository {

    protected $modelClass = LevelSectionYear::class;
    protected $levelSectionYear;

    //Declaración del constructor
    public function __construct(LevelSectionYear $levelSectionYear) {
        $this->levelSectionYear = $levelSectionYear;
    }

    public function listsSectionsBySchoolYear($serial_scy) {

        $sections = $this->newQuery();
        $sections->join('section', 'section.serial_set', '=', 'level_section_year.serial_set');
        $sections->where('serial_scy', '=', $serial_scy);
        $sections->where('status_lsy', '=', 'ACTIVE');
        $sections->where('status_set', '=', 'ACTIVE');
        return $sections->pluck('name_set', 'section.serial_set');
    }


    /**
     * Método para obtener niveles por seccion y serial de anio lectivo
     * @since   2018-03-15
     * @param   integer serial_scy año lectivo
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
   public function getLevelBySectionSchoolYear($serial_scy,$serial_set) {

        $levels = $this->newQuery();
        $levels->join('level', 'level.serial_lvl', '=', 'level_section_year.serial_lvl');
        $levels->where('level_section_year.serial_scy', '=', $serial_scy);
        $levels->where('level_section_year.serial_set', '=', $serial_set);
        $levels->where('level_section_year.status_lsy', '=', 'ACTIVE');
        $levels->where('level.status_lvl', '=', 'ACTIVE');    
        $levels->orderBy('level_section_year.order_lsy', 'ASC');        
        return $this->doQueryGet($levels);
    }


    /**
     * Método para obtener subperiodo por fecha, anio, section, nivel
     * @since   2018-03-24
     * @param   date date fecha
     * @param   integer serial_scy año lectivo
     * @param   integer serial_set seccion
     * @param   integer serial_lvl nivel
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
   public function getSubperiodByDateSchoolYearSectionLevel($date,$serial_scy, $serial_set, $serial_lvl) {

        $query = $this->newQuery();        
        $query->join('period as prd', 'prd.serial_pgy', '=', 'level_section_year.serial_pgy');
        $query->join('subperiod as sbp', 'sbp.serial_prd', '=', 'prd.serial_prd');

        $query->where('level_section_year.serial_scy', '=', $serial_scy);
        $query->where('level_section_year.serial_set', '=', $serial_set);
        $query->where('level_section_year.serial_lvl', '=', $serial_lvl);
        $query->where('sbp.start_date_sbp', '<=', $date);
        $query->where('sbp.end_date_sbp', '>=', $date);        
        $query->where('prd.status_prd', '=', 'ACTIVE');
        $query->where('sbp.status_sbp', '=', 'ACTIVE');
        $query->where('level_section_year.status_lsy', '=', 'ACTIVE');        
         
        return $this->doQueryFirst($query);
    }


     /**
     * Método que devuelve los datos de LevelSectionYear por anio, section, nivel
     * @since   2018-04-01
     * @param   integer serial_scy año lectivo
     * @param   integer serial_set año lectivo
     * @param   integer serial_lvl año lectivo
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
   public function getLevelSectionYearBySchoolYearSectionLevel($serial_scy,$serial_set, $serial_lvl) {
    
        $lsy = $this->newQuery();        
        $lsy->where('serial_scy', '=', $serial_scy);
        $lsy->where('serial_set', '=', $serial_set);        
        $lsy->where('serial_lvl', '=', $serial_lvl);
        
        return $this->doQueryFirst($lsy);
   }

    public function getLevelByYearAndSection($serial_scy,$serial_set) {
        $levels = $this->newQuery();
        $levels->join('level', 'level.serial_lvl', '=', 'level_section_year.serial_lvl');
        $levels->join('section', 'section.serial_set', '=', 'level_section_year.serial_set');
        $levels->where('level_section_year.serial_scy', '=', $serial_scy);
        $levels->where('level_section_year.serial_set', '=', $serial_set);
        $levels->where('level_section_year.status_lsy', '=', 'ACTIVE');
        $levels->where('level.status_lvl', '=', 'ACTIVE');
        $levels->select('level_section_year.serial_scy','level.serial_lvl','level.name_lvl','section.serial_set','section.name_set');
        $levels->orderBy('level_section_year.order_lsy', 'ASC');
        return $this->doQueryGet($levels);
    }

}
