<?php

namespace App\Repositories;
use App\Models\AppSetup;
use App\Models\Employee;
use App\Models\Guardian;
use App\Models\SchoolSetup;
use App\Models\User;
use Illuminate\Support\Facades\Mail;

/**
 * Controlador para buzon de sugerencias
 * @since   2017-06-26
 * @author  Henry Vivanco <hvvivanco@rutatec.com>
 *
 */
class mailSuggestoinRepository extends BaseRepository
{
    protected $modelClass = AppSetup::class;
    protected $appSetup;
    protected $applicantStudent;
    protected $schoolSetup;
    protected $user;
    protected $message;

    public function __construct(AppSetup $appSetup, ApplicantStudentRepository $applicantStudent, SchoolSetupRepository $schoolSetup, UserRepository $user, MessageRepository $message)
    {
        $this->appSetup = $appSetup;
        $this->applicantStudent = $applicantStudent;
        $this->schoolSetup = $schoolSetup;
        $this->user = $user;
        $this->message = $message;
    }

    /**
     * Método que envía sugerencias al correo
     * @since 2018-06-28
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @return \Illuminate\Http\Response
     */
    public function getSuggestions($request)
    {
        $data = ['msg' => $request->msg];
        $msgId=$this->appSetup->first();
        $emails = explode(',', $msgId->mail_to_aps);
        $mails = array();
        foreach ($emails as $email) {
            array_push($mails, $email);
        }
        Mail::send('emails.confirmationEmail', $data, function ($message) use ($msgId, $mails) {
            $message->from($msgId->mail_from_aps);
            $message->to($mails)->subject('Sugerencia');
        });
        return response()->json($data);
    }

    public function getSuggestion($msgFrom, $maiTo)
    {
        $data = ['msg' => 'Mensaje Quemado'];
        Mail::send('emails.confirmationEmail', $data, function ($message) use ($msgFrom, $maiTo) {
            $message->from($msgFrom);
            $message->to($maiTo)->subject('Test Email');
        });
        return true;
    }

    public function mailApplicantGuardian($applicantGuardian){
        try{
            $applicantsStd = $this->applicantStudent->getApplicantsStudent($applicantGuardian);
            foreach ($applicantsStd as $applicantStd) {
                $employeeSection = $this->applicantStudent->getApplicantsStudentAndEmployee($applicantStd);
            }
            $nameSchool = $this->schoolSetup->getNameSchool();
            $linkMagazine = $this->schoolSetup->getLinkMagazine();
            $mail = $applicantGuardian->email_agd;
            $messages = ['user' => $applicantGuardian['first_name_agd'] . ' ' . $applicantGuardian['last_name_agd'],
                'secretaryGeneral' => $employeeSection[0]->first_name_emp . ' ' . $employeeSection[0]->last_name_emp,
                'nameSchool' => $nameSchool[19],
                'linkMagazine' => $linkMagazine[19],
                'bye' => 'Cordialmente.',
            ];
            Mail::send(env('APPLICANT_MAIL') == '' ? 'emailApplicant.mailGuardianDefault':env('APPLICANT_MAIL'), $messages, function ($message) use ($mail) {
                $message->from('learnbox@tomasmoro.k12.ec');
                    $message->to(env('MAIL_TEST') == '' ? $mail:env('MAIL_TEST'))->subject('Proceso de admisiones');
            });
            return true;
        }
        catch (\Exception $e){
            return $e->getMessage();
        }

    }

    public function mailApplicantApprover($applicantGuardian){
        try{

            $messages = ['user' => '',
                'msg' => 'Se han ingresado nuevos aplicantes para el proceso de admisiones, los aplicantes son: ',
                'applicants' => array(),
                'link' => env('SERVER_LEARNBOX'),
                'success' => 'Para consultar la aplicación haga click ',
                'option' => 'También puede consultar en el siguiente menú: ',
                'menu' => 'Usuarios >> Proceso de admisiones >> Gestionar aplicaciones Públicas',
                'bye' => 'Saludos.',
            ];
            $applicantsStd = $this->applicantStudent->getApplicantsStudent($applicantGuardian);
            foreach ($applicantsStd as $applicantStd) {
                $date = date('Y-m-d',strtotime($applicantStd->created_at));
                array_push($messages['applicants'],$applicantStd->last_name_ast . ' ' . $applicantStd->first_name_ast
                    . ' - ' .$applicantStd->name_scy. ' - ' . $applicantStd->name_set . ' - ' . $applicantStd->name_lvl
                    .' - ' . $date
                );
                $employeeSection = $this->applicantStudent->getApplicantsStudentAndEmployee($applicantStd);
            }
            foreach ($employeeSection as $applicantStdEmployee) {
                $mail = $applicantStdEmployee->email_emp;
                $messages['user'] = 'Estimad@: ' . $applicantStdEmployee->first_name_emp . ' ' . $applicantStdEmployee->last_name_emp;
                Mail::send('emailApplicant.mailEmployee', $messages, function ($message) use ($mail) {
                    $message->from('learnbox@tomasmoro.k12.ec');
                    $message->to(env('MAIL_TEST') == '' ? $mail : env('MAIL_TEST'))->subject('Nuevos aplicantes');
                });
            }
            return true;
        }
        catch (\Exception $e){
            return $e->getMessage();
        }

    }

    /**
     * Método para enviar mail de respuesta a mensaje
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @params $request
     **/
    public function sendMailMessage ($message,$saveMessage) {
        try {
            $userSendMessage = $this->user->getUserBySerialusr($message['serial_usr_send']);
            if ($userSendMessage->serial_emp != null) {
                $userSend = Employee::where('serial_emp', '=', $userSendMessage->serial_emp)->first();
                $userSend->name = $userSend->first_name_emp . ' ' . $userSend->last_name_emp;
            } else {
                $userSend = Guardian::where('serial_grd', '=', $userSendMessage->serial_grd)->first();
                $userSend->name = $userSend->first_name_grd . ' ' . $userSend->last_name_grd;
            }
            $userReceiveMessage = $this->user->getUserBySerialusr($message['serial_usr_receive']);
            if ($userReceiveMessage->serial_emp != null) {
                $userReceive = Employee::where('serial_emp', '=', $userReceiveMessage->serial_emp)->first();
                $userReceive->name = $userReceive->first_name_emp . ' ' . $userReceive->last_name_emp;
                $mail = $userReceive->email_emp;
            } else {
                $userReceive = Guardian::where('serial_grd', '=', $userReceiveMessage->serial_grd)->first();
                $userReceive->name = $userReceive->first_name_grd . ' ' . $userReceive->last_name_grd;
                $mail = $userReceive->email_grd;
            }
            $threadMessageID = $this->message->getThreadMessageID($saveMessage->serial_msg);
            $threadMessages = $this->message->getThreadMessage($threadMessageID->serial_mth,$saveMessage->send_date_msg);
            $messages = ['user' => $userSend->name,
                'msg' => $message['message_msg'],
                'threadMessages' => $threadMessages
            ];
            $subject = $message['subject_msg'];
            Mail::send('messages.sendMessage', $messages, function ($msg) use ($mail, $subject) {
                $msg->from(env('MAIL_FROM_ADDRESS'));
                $msg->to($mail)->subject($subject);
            });
            return true;
        } catch (\Exception $exception) {
            return $exception;
        }
    }


}