<?php

namespace App\Repositories;

use App\Models\OptActivity;
use App\Models\OptActivityAttachment;

class OptActivityRepository extends BaseRepository
{

    protected $optActivity;
    protected $modelClass = OptActivity::class;

    /**
     * @var var 
     */
    //public $attach_path=env('ATTACH_PATH');


    //Declaración del constructor
    public function __construct(OptActivity $optActivity)
    {
        $this->optActivity = $optActivity;        
    }


    /**
     * Método para obtener el detalle de las actividades
     *  @since   2018-07-29
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   array $data
     * @return  object 
     *
     */
    public function getOptActivityDetails($data)
    {
        $serial_act = $data['serial_act'];
        
        $query = $this->newQuery();        
        $query->join('opt_activity_applied_criterias AS oac','oac.serial_opa', '=', 'opt_activity.serial_opa');
        $query->join('optional_subject AS osb','osb.serial_osb', '=', 'oac.serial_osb');
        $query->join('cat_subject AS csb','csb.serial_csb', '=', 'osb.serial_csb');
        $query->join('optional_subject_level AS osbl','osbl.serial_osb', '=', 'osb.serial_osb');
        $query->join('school_year AS scy','scy.serial_scy', '=', 'osbl.serial_scy');
        $query->join('activity_type AS att','att.serial_att', '=', 'oac.serial_att');
            //Comentar para sistemas antiguos
        $query->leftJoin('subject_category as scat', 'scat.serial_scat', '=', 'osbl.serial_scat');


        $query->where('opt_activity.serial_opa', $serial_act);
        $query->where('status_opa', 'ACTIVE');    
        $query->where('status_osb', 'ACTIVE'); 
        $query->where('status_csb', 'ACTIVE');
        $query->where('status_osl', 'ACTIVE');
        $query->where('status_scy','LIKE', '%CURRENT%');
        $query->where('status_att', 'ACTIVE');
        $query->orderBy('csb.name_csb');
        $query->distinct();


            //Cometar para sistemas antiguos
            //. "scat.name_scat as name_csb,"
        $query->selectRaw("opt_activity.serial_opa as serial_act, scat.name_scat as name_csb, opt_activity.name_opa as name_act, opt_activity.description_opa as description_act, DATE_FORMAT(opt_activity.send_date_opa, '%W-%e-%M-%Y') AS send_date, DATE_FORMAT(opt_activity.due_date_opa, '%W-%e-%M-%Y') AS due_date, att.serial_att as activity_type, att.name_att ");

        $message  = $this->doQueryGet($query);


        $attach_files =OptActivityAttachment::where('serial_opa', '=', $serial_act)->where('status_oat', '=', 'ACTIVE')->get();

        if (sizeof($message)>0) {
            $return_array = array(
                'serial_act' => $message[0]->serial_act,
                'activity_type' => $message[0]->activity_type,
                'name_act' => $message[0]->name_act,
                'name_att' => $message[0]->name_att,
                'name_csb' => $message[0]->name_csb,
                'description_act' => $message[0]->description_act,
                'send_date' => $message[0]->send_date,
                'due_date' => $message[0]->due_date,
            );
            if (sizeof($attach_files)>0) {
                $return_array['name_ath'] = $attach_files[0]->name_oat;
                $return_array['filename_ath'] = $attach_files[0]->filename_oat;
                $return_array['path'] = env('ATTACH_PATH') . 'opt_activity_attachment/' . $return_array['serial_act'] . '/';
                return $return_array;
            } else {
                $return_array['name_ath'] = '';
                $return_array['filename_ath'] = '';
                $return_array['path'] = '';
                return $return_array;
            }
        }
    }
    
}
