<?php

namespace App\Repositories;

use App\Models\Period;

class PeriodRepository extends BaseRepository {

    protected $modelClass = Period::class;
    protected $period;

    //Declaración del constructor
    public function __construct(Period $period) {
        $this->period = $period;
    }

    /**
     * Método para obtener datos de subperiodo por serial
     * @since   2018-04-05
     * @param   integer serial_std serial estudiante
     * @param   integer serial_scy serial anio
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getPeriodsByYearByStudent($serial_std, $serial_scy) {

        $query = $this->newQuery();
        $query->join('period_group_year as pgy', 'pgy.serial_pgy', '=', 'period.serial_pgy');
        $query->join('student_by_course as sbc', function ($join) use ($serial_std) {
            $join->where('sbc.serial_std', '=', $serial_std);
        });
        $query->join('course as c', function ($join) use ($serial_scy) {
            $join->on('c.serial_crs', '=', 'sbc.serial_crs');
            $join->where('c.serial_scy', '=', $serial_scy);
            $join->where('c.status_crs', '=', 'ACTIVE');
        });
        $query->join('level_section_year as lsy', function ($join) use ($serial_scy) {
            $join->on('lsy.serial_pgy', '=', 'pgy.serial_pgy');
            $join->on('lsy.serial_lvl', '=', 'c.serial_lvl');
            $join->on('lsy.serial_set', '=', 'c.serial_set');
            $join->where('lsy.serial_scy', '=', $serial_scy);
            $join->where('lsy.status_lsy', '=', 'ACTIVE');
        });


        $query->where('pgy.serial_scy', '=', $serial_scy);
        $query->where('sbc.status_stc', '=', 'ACTIVE');
        $query->where('pgy.status_pgy', '=', 'ACTIVE');
        $query->where('period.status_prd', '=', 'ACTIVE');
        $query->orderBy('period.serial_prd');
        $query->selectRaw('DISTINCT period.serial_prd, period.name_prd, period.start_date_prd,end_date_prd,options_prd,options_pgy');

        return $this->doQueryGet($query);
    }

}
