<?php

namespace App\Repositories;

use App\Models\OptionalSubject;
use App\Models\Schedule;

class ScheduleRepository extends BaseRepository {

    protected $modelClass = Schedule::class;
    protected $schedule;

    //Declaración del constructor
    public function __construct(Schedule $schedule) {
        $this->schedule = $schedule;
    }

    /**
     * Método para obtener horario por curso por dia
     * @since   2018-04-14
     * @param   integer serial_crs serial curso
     * @param   integer date fecha del horario
     * @param   integer dat dia
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getScheduleByCourseDateDay($serial_crs, $date, $day, $serial_lop = false) {
        $query = $this->newQuery();
        $query->join('schedule_header as shd', 'shd.serial_shd', '=', 'schedule.serial_shd');
        $query->join('credits_detail', function($join) {
            $join->on('schedule.serial_cdt', '=', 'credits_detail.serial_cdt');
            $join->where('status_cdt', '=', 'ACTIVE');
        });
        $query->leftJoin('subject_course as sbc', function($join) {
            $join->on('sbc.serial_sbc', '=', 'schedule.serial_sbc');
            $join->where('sbc.status_sbc', '=', 'ACTIVE');
        });
        $query->leftJoin('subject as sbj', function($join) {
            $join->on('sbj.serial_sbj', '=', 'sbc.serial_sbj');
            $join->where('sbj.status_sbj', '=', 'ACTIVE');
        });
        $query->leftJoin('cat_subject as csb', function($join) {
            $join->on('csb.serial_csb', '=', 'sbj.serial_csb');
            $join->where('csb.status_csb', '=', 'ACTIVE');
        });
        $query->leftJoin('cat_subject as csb2', function($join) {
            $join->on('csb2.serial_csb', '=', 'csb.cat_serial_csb');
            $join->where('csb2.status_csb', '=', 'ACTIVE');
        });
        $query->leftJoin('level_optional_group as lop', function($join) {
            $join->on('lop.serial_lop', '=', 'schedule.serial_lop');
            $join->where('lop.status_lop', '=', 'ACTIVE');
        });
        $query->leftJoin('special_subject_course as sps', function($join) {
            $join->on('sps.serial_sps', '=', 'schedule.serial_sps');
        });
        $query->leftJoin('cat_subject as csb3', function($join) {
            $join->on('csb3.serial_csb', '=', 'sps.serial_csb');
            $join->where('csb3.status_csb', '=', 'ACTIVE');
        });

        $query->where('schedule.status_scd', '=', 'ACTIVE');
        if (!$serial_lop) {
            $query->where('schedule.serial_crs', '=', $serial_crs);
        } else {
            $query->where('schedule.serial_lop', '=', $serial_lop);
        }
        $query->where('schedule.day_scd', '=', $day);
        $query->where('shd.date_start_shd', '<=', $date);
        $query->where('shd.date_end_shd', '>=', $date);
        $query->orderBy('end_at_cdt', 'asc');
        return $this->doQueryGet($query);
    }

    /**
     * Método para obtener horario por curso por dia
     * @since   2018-04-14
     * @param   integer serial_crs serial curso
     * @param   integer date fecha del horario
     * @param   integer dat dia
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getScheduleByCourseCreditDetails($serial_crs, $serial_cdt, $date, $day) {
        $query = $this->newQuery();
        $query->join('schedule_header as shd', function($join) use($date) {
            $join->on('shd.serial_shd', '=', 'schedule.serial_shd');
            $join->where('shd.date_start_shd', '<=', $date);
            $join->where('shd.date_end_shd', '>=', $date);
        });
        $query->where('schedule.status_scd', '=', 'ACTIVE');
        $query->where('schedule.day_scd', '=', $day);
        $query->where('schedule.serial_crs', '=', $serial_crs);
        $query->where('schedule.serial_cdt', '=', $serial_cdt);
        return $this->doQueryGet($query);
    }

    public function getScheduleByCourseCreditDetailsMandatories($serial_crs, $serial_cdt, $date, $day) {
        $query = $this->newQuery();
        $query->join('schedule_header as shd', function($join) use($date) {
            $join->on('shd.serial_shd', '=', 'schedule.serial_shd');
            $join->where('shd.date_start_shd', '<=', $date);
            $join->where('shd.date_end_shd', '>=', $date);
        });
        $query->where('schedule.status_scd', '=', 'ACTIVE');
        $query->where('schedule.day_scd', '=', $day);
        $query->where('schedule.serial_crs', '=', $serial_crs);
        $query->where('schedule.serial_cdt', '=', $serial_cdt);

        $query->whereNotNull('schedule.serial_sbc');
        return $this->doQueryGet($query);
    }

    public function getScheduleByCourseCreditDetailsOptionlSubjects($serial_scd, $serial_std, $date) {
        $query = $this->newQuery();
        $query->join('student_optional_subject AS sos', 'schedule.serial_lop', '=', 'sos.serial_lop');
        $query->join('period AS prd', function($join) use($date) {
            $join->on('sos.serial_prd', '=', 'prd.serial_prd');
            $join->where('prd.start_date_prd', '<=', $date);
            $join->where('prd.end_date_prd', '>=', $date);
        });
        $query->where('schedule.status_scd', '=', 'ACTIVE');
        $query->where('schedule.serial_scd', '=', $serial_scd);
        $query->where('sos.serial_std', '=', $serial_std);
        return $this->doQueryGet($query);
    }

    /**
     * Método que obtiene horario por serial_scd
     * @since   2018-04-20
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   $serial_scd id
     * @return  
     * 
     */
    public function getScheduleBySerialScd($serial_scd) {
        $query = $this->newQuery();

        $query->where('serial_scd', '=', $serial_scd);

        return $this->doQueryFirst($query);
    }

    /**
     * Método que obtiene horario por serial_sbc o serial_osb
     * @since   2019-01-30
     * @author  Jairo Serrano <jaserrano@rutatec.com>
     *
     * @param   $serial_sbc id
     * @param   $serial_osb id
     * @return
     *
     */
    public function getScheduleBySerialSubject($serial_sbc=null,$serial_osb=null) {
        $result =null;
        $query = $this->newQuery();
        if($serial_sbc){
            $query->where('serial_sbc', '=', $serial_sbc)
                ->where('status_scd','=','ACTIVE');
            $result= $this->doQueryGet($query);
        }
        elseif ($serial_osb){
            $optionalSubject= OptionalSubject::find($serial_osb);
            $levelOptionalGroup=$optionalSubject->levelOptionalGroup()->where('status_lop','=','ACTIVE')->first();
            $query->where('serial_lop', '=', $levelOptionalGroup->serial_lop)
                ->where('status_scd','=','ACTIVE')
                ->groupBy('day_scd','serial_cdt');
            $result= $this->doQueryGet($query);
        }
        return $result;
    }



    /**
     * Método para obtener horario por curso y fecha
     * @since   2018-07-22
     * @param   integer serial_crs serial curso
     * @param   integer date fecha del horario
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getScheduleByCourseDate($serial_crs, $date) {

        $query = $this->newQuery();
        $query->join('schedule_header as shd', 'shd.serial_shd', '=', 'schedule.serial_shd');

        $query->join('credits_detail', function($join) {
            $join->on('credits_detail.serial_cdt', '=', 'schedule.serial_cdt');
            $join->where('status_cdt', '=', 'ACTIVE');
        });

        $query->leftJoin('subject_course as sbc', function($join) {
            $join->on('sbc.serial_sbc', '=', 'schedule.serial_sbc');
            $join->where('sbc.status_sbc', '=', 'ACTIVE');
        });
        $query->leftJoin('subject as sbj', function($join) {
            $join->on('sbj.serial_sbj', '=', 'sbc.serial_sbj');
            $join->where('sbj.status_sbj', '=', 'ACTIVE');
        });
        $query->leftJoin('cat_subject as csb', function($join) {
            $join->on('csb.serial_csb', '=', 'sbj.serial_csb');
            $join->where('csb.status_csb', '=', 'ACTIVE');
        });
        $query->leftJoin('cat_subject as csb2', function($join) {
            $join->on('csb2.serial_csb', '=', 'csb.cat_serial_csb');
            $join->where('csb2.status_csb', '=', 'ACTIVE');
        });
        $query->leftJoin('level_optional_group as lop', function($join) {
            $join->on('lop.serial_lop', '=', 'schedule.serial_lop');
            $join->where('lop.status_lop', '=', 'ACTIVE');
        });
        $query->leftJoin('special_subject_course as sps', function($join) {
            $join->on('sps.serial_sps', '=', 'schedule.serial_sps');
        });
        $query->leftJoin('cat_subject as csb3', function($join) {
            $join->on('csb3.serial_csb', '=', 'sps.serial_csb');
            $join->where('csb3.status_csb', '=', 'ACTIVE');
        });

        $query->where('schedule.status_scd', '=', 'ACTIVE');
        $query->where('schedule.serial_crs', '=', $serial_crs);

        $query->where('shd.date_start_shd', '<=', $date);
        $query->where('shd.date_end_shd', '>=', $date);
        $query->orderBy('day_scd');
        $query->orderBy('start_at_cdt');
        $query->distinct();
        $query->selectRaw("
          schedule.serial_lop,schedule.serial_sbc,schedule.serial_sps,schedule.day_scd,start_at_cdt, end_at_cdt, schedule.serial_emp as serial_emp_scd, sbc.serial_emp as serial_emp_sbc, sps.serial_emp as serial_emp_sps,
          name_lop,csb.name_csb as name_csb_csb,csb2.name_csb as name_csb_csb2,csb3.name_csb as name_csb_csb3
          ");

        return $this->doQueryGet($query);
    }


    public function getNextSchedulesBySchedule($serial_scd){
        $queryAux= $this->newQuery();
        $queryAux->join('credits_detail as cdt',function($join) {
            $join->on('cdt.serial_cdt','schedule.serial_cdt')
                ->where('cdt.status_cdt','ACTIVE');
        })->where('schedule.serial_scd',$serial_scd);
        $schedule= $this->doQueryFirst($queryAux);
        if($schedule){
            $query = $this->newQuery();
            $query->join('credits_detail as cdt',function($join) use($schedule){
                $join->on('cdt.serial_cdt','schedule.serial_cdt')
                    ->where('cdt.status_cdt','ACTIVE')
                    ->where('cdt.start_at_cdt','>=',$schedule->start_at_cdt);
            })
                ->where('serial_shd',$schedule->serial_shd)
                ->where('day_scd',$schedule->day_scd)
                ->where('status_scd','=','ACTIVE')
                ->orderBy('cdt.start_at_cdt');
            $scheduleList= $this->doQueryGet($query);
            if(count($scheduleList)>0){
                return $scheduleList;
            }else{
                return null;
            }
        }else{
            return null;
        }
    }

    public function getCreditDetailBySchedule($serial_scd){
        $queryAux= $this->newQuery();
        $queryAux->join('credits_detail as cdt',function($join) {
            $join->on('cdt.serial_cdt','schedule.serial_cdt')
                ->where('cdt.status_cdt','ACTIVE');
        })->where('schedule.serial_scd',$serial_scd)
        ->select('cdt.*');
        return $this->doQueryFirst($queryAux);
    }

    public function getCourseBySchedule($serial_scd){
        $queryAux= $this->newQuery();
        $queryAux->join('course as crs',function($join) {
            $join->on('crs.serial_crs','schedule.serial_crs')
                ->where('crs.status_crs','ACTIVE');
        })->where('schedule.serial_scd',$serial_scd)
        ->select('crs.*');
        return $this->doQueryFirst($queryAux);
    }


}
