<?php

namespace App\Repositories;

use App\Models\Section;
use App\Repositories\BaseRepository;

/**
 * Repostiorio para seccion
 * @since   2017-07-09
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2017-07-09<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del archivo<br/>
 *          #edit1
 */
class SectionRepository extends BaseRepository {

    protected $modelClass = Section::class;

    //Declaración del constructor
    public function __construct(Section $section) {
        $this->section = $section;
    }

    /**
     * Método que retorna section , especialidad, nivel por anio lectivo
     * @since   2017-07-15
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   void
     * @return  ListObject Section
     * 
     */
    public function getSectionSpecialtyLevelBySchoolYear($serial_scy) {
        $query = $this->newQuery();
        $query->join('level_section_year_specialty AS lsy', 'section.serial_set', '=', 'lsy.serial_set');
        $query->join('level AS lvl', 'lsy.serial_lvl', '=', 'lvl.serial_lvl');
        $query->join('specialty AS spt', 'lsy.serial_spt', '=', 'spt.serial_spt');

        $query->where('lsy.serial_scy', '=', $serial_scy);
        $query->where('lsy.status_lss', '=', 'ACTIVE');
        $query->where('lvl.status_lvl', '=', 'ACTIVE');
        $query->where('spt.status_spt', '=', 'ACTIVE');

        return $this->doQueryGet($query);
    }

     /**
     * Devuelve las secciones del año lectico pasado como parámetro
     *
     * @param   void
     * 
     * @return  object $sections secciones del año lectico correspondiente
     * @throws  InvalidArgumentException
     * @since   2017-02-15
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     */
    public function getSectionBySchoolYear($serial_scy) {
        $sections = $this->newQuery();
        $sections->join('level_section_year', 'level_section_year.serial_set', '=', 'section.serial_set');
        $sections->where('serial_scy', '=', $serial_scy);
        $sections->where('status_lsy', '=', 'ACTIVE');
        $sections->where('status_set', '=', 'ACTIVE');
        $sections->distinct();
        $sections->select('section.*');

        return $this->doQueryGet($sections);
    }

    /**
     * Devuelve las secciones por año lectivo
     *
     * @param   $serial_scy
     *
     * @return  object $sections secciones del año lectico correspondiente
     * @since   2019-01-25
     * @author  Henry Vivanco <hvvivanco@rutatec.com>
     *
     */
    public function getSectionsBySchoolYear($serial_scy) {
        $sections = Section::where('status_set', '=', 'ACTIVE')
            ->join('level_section_year', 'level_section_year.serial_set', '=', 'section.serial_set')
            ->where('serial_scy', '=', $serial_scy)
            ->where('status_set', '=', 'ACTIVE')
        ->distinct()
        ->pluck('section.name_set','section.serial_set');

        return $sections;
    }

}
