<?php

namespace App\Repositories;

use App\Models\StudentByCourse;
use App\Repositories\BaseRepository;

/**
 * Repositorio del usuario

 * @author Erick Vaca <epvaca@rutatec.com>
 * @since  2018-04-05
 */
class StudentByCourseRepository extends BaseRepository {

    protected $modelClass = StudentByCourse::class;

    protected $studentByCourse;


    public function __construct(StudentByCourse $studentByCourse) {
        $this->studentByCourse = $studentByCourse;
    }

    public function getSubjectsByStudent($serial_scy, $serial_prd, $serial_std) {

        $subjects = $this->newQuery();
        $subjects->join('course', 'course.serial_crs', '=', 'student_by_course.serial_crs');
        $subjects->join('subject_course', 'subject_course.serial_crs', '=', 'course.serial_crs');
        $subjects->join('subject', 'subject.serial_sbj', '=', 'subject_course.serial_sbj');
        $subjects->join('school_year', 'school_year.serial_scy', '=', 'subject.serial_scy');
        $subjects->join('cat_subject', 'cat_subject.serial_csb', '=', 'subject.serial_csb');
        $subjects->leftjoin('cat_subject as cate', function ($join) {
            $join->on('cate.serial_csb', '=', 'cat_subject.cat_serial_csb')
                    ->where('cate.status_csb', '=', 'ACTIVE');
        });
        $subjects->join('period_group_year', 'period_group_year.serial_scy', '=', 'school_year.serial_scy');
        $subjects->join('period', 'period.serial_pgy', '=', 'period_group_year.serial_pgy');
        $subjects->where('subject_course.has_grades', '=', 1);
        $subjects->where('subject_course.has_opt_subject_relation', '=', 0);
        $subjects->where('student_by_course.serial_std', '=', $serial_std);
        $subjects->where('period.serial_prd', '=', $serial_prd);
        $subjects->where('period_group_year.serial_scy', '=', $serial_scy);
        $subjects->where('status_crs', '=', 'ACTIVE');
        $subjects->where('status_sbc', '=', 'ACTIVE');
        $subjects->where('status_sbj', '=', 'ACTIVE');
        $subjects->where('cat_subject.status_csb', '=', 'ACTIVE');
        $subjects->where('status_pgy', '=', 'ACTIVE');
        $subjects->where('status_prd', '=', 'ACTIVE');
        $subjects->selectRaw('distinct cat_subject.name_csb as cat_name, subject_course.serial_sbc, "N/A" as serial_lop');

        return $this->doQueryGet($subjects);
    }

}
