<?php

namespace App\Repositories;

use App\Models\Student;
use App\Repositories\BaseRepository;
use App\Repositories\CourseRepository;
use App\Repositories\PeriodRepository;

/**
 * Repositorio del usuario

 * @author Erick Vaca <epvaca@rutatec.com>
 * @since  2018-04-05
 */
class StudentRepository extends BaseRepository {

    protected $modelClass = Student::class;
    protected $student;
    protected $course;
    protected $period;

    public function __construct(Student $student, CourseRepository $course, PeriodRepository $period) {
        $this->student = $student;
        $this->course = $course;
        $this->period = $period;
    }

    /**
     * Devuelve el usuario si es que existe
     *
     * @param   void
     * 
     * @return  object user
     * @throws  InvalidArgumentException
     * @since   2017-05-19
     * @author  Fernando Salas <rfsalas@rutatec.com>
     *
     * @edit    2017-05-19<br />
     *          Fernando Salas <rfsalas@rutatec.com><br />
     *          creación del método<br/>
     *          #edit1
     */
    public function getStudentInfo($serial_usr) {
        $student = $this->newQuery();
        $student->join('user', 'student.serial_usr', '=', 'user.serial_usr');
        $student->where('status_std', '=', 'ACTIVE');
        $student->where('user.serial_usr', '=', $serial_usr);
        $student->select('student.*');
        return $this->doQueryFirst($student);
    }

    /**
     * Método que devuelve estudiante  por serial_std
     * @since   2018-04-05  
     * @param   integer serial_usr serial_usr
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getStudentBySerialStd($serial_std) {
        $query = $this->newQuery();
        $query->where('serial_std', '=', $serial_std);
        return $this->doQueryFirst($query);
    }

    public function getSubjectsByStudentYear($serial_std, $serial_scy) {
        $periods = $this->period->getPeriodsByYearByStudent($serial_std, $serial_scy);
        $optatives = $this->newQuery();
        $optatives->join('student_year as sty', function($join)use($serial_scy) {
            $join->on('sty.serial_std', '=', 'student.serial_std')
                    ->where('sty.serial_scy', '=', $serial_scy)
                    ->where('sty.status_sty', '<>', 'RETIRED');
        });
        $optatives->join('student_by_course as stc', function($join) {
            $join->on('stc.serial_std', '=', 'student.serial_std')
                    ->where('status_stc', '=', 'ACTIVE');
        });
        $optatives->join('course as crs', function($join) {
            $join->on('stc.serial_crs', '=', 'crs.serial_crs')
                    ->where('status_crs', '=', 'ACTIVE');
        });
        $optatives->join('student_optional_subject as sos', function($join) {
            $join->on('student.serial_std', '=', 'sos.serial_std')
                    ->where('status_sos', '=', 'ACTIVE');
        });
        $optatives->join('optional_subject_level as osl', function($join) {
            $join->on('sos.serial_osb', '=', 'osl.serial_osb')
                    ->on('osl.serial_scy', '=', 'crs.serial_scy')
                    ->where('status_osl', '=', 'ACTIVE')
                    ->where('osl.has_grades', '=', 1);
        });
        $optatives->join('optional_subject as osb', function($join) {
            $join->on('sos.serial_osb', '=', 'osb.serial_osb')
                    ->where('status_osb', '=', 'ACTIVE');
        });
        $optatives->join('group_optional_subject as gos', function($join) {
            $join->on('osb.serial_osb', '=', 'gos.serial_osb')
                    ->on('sos.serial_lop', '=', 'gos.serial_lop')
                    ->where('status_gos', '=', 'ACTIVE');
        });
        $optatives->join('level_optional_group as lop', function($join) {
            $join->on('sos.serial_lop', '=', 'lop.serial_lop')
                    ->on('lop.serial_scy', '=', 'sty.serial_scy')
                    ->on('lop.serial_lvl', '=', 'sty.serial_lvl');
        });
        $optatives->join('cat_subject as csb', function($join) {
            $join->on('osb.serial_csb', '=', 'csb.serial_csb')
                    ->where('status_csb', '=', 'ACTIVE');
        });
        $optatives->leftjoin('subject_category as scat', function($join) {
            $join->on('osl.serial_scat', '=', 'scat.serial_scat')
                    ->where('status_scat', '=', 'ACTIVE');
        });
        foreach ($periods as $period) {
            $optatives->leftjoin('student_subject_period_average as ' . 'stgo' . $period->serial_prd, function($join)use($period) {
                $join->on('stgo' . $period->serial_prd . '.serial_lop', '=', 'sos.serial_lop')
                        ->on('stgo' . $period->serial_prd . '.serial_std', '=', 'student.serial_std')
                        ->where('stgo' . $period->serial_prd . '.serial_prd', '=', $period->serial_prd);
            });
            $optatives->where('student.serial_std', '=', $serial_std);
        }
        $optatives->leftjoin('student_subject_year_average AS sya', function($join) use($serial_scy) {
            $join->on('student.serial_std', '=', 'sya.serial_std')
                    ->on('gos.serial_lop', '=', 'sya.serial_lop');
        });
        $optatives->selectRaw('IF(csb.short_name_csb is null,IF(scat.name_scat is null,csb.name_csb,scat.name_scat),csb.short_name_csb) as "MATERIA"');
        foreach ($periods as $period) {
            $namePeriod = explode(" ", $period->name_prd);
            $periodAcronym = "";

            foreach ($namePeriod as $name) {
                $periodAcronym .= $name[0];
            }
            if (strpos($period->options_prd, 'SHOW_GRADES_IN_PARENTS_REPORT') !== false) {
                $optatives->selectRaw('IF(stgo' . $period->serial_prd . '.average_ssa is null,null,round(stgo' . $period->serial_prd . '.average_ssa,2)) as "' . $periodAcronym . '"');
            } else {
                $optatives->selectRaw(' "--" as "' . $periodAcronym . '"');
            }
        }
        if (strpos($period->options_pgy, 'SHOW_GRADES_IN_PARENTS_REPORT') !== false) {
            $optatives->selectRaw('IF(average_sya is null,null,round(average_sya,2)) as PR');
            $optatives->selectRaw('IF(final_grade_sya is null,null,round(final_grade_sya,2)) as PF');
        } else {
            $optatives->selectRaw('"--" as PR');
            $optatives->selectRaw('"--" as PF');
        }
        $optatives->selectRaw('lop.serial_lop as "SERIAL"');
        $optatives->selectRaw('osb.grade_type_sbc as "GRADETYPE"');
        $optatives->selectRaw('"OPTATIVE" as "TYPE"');
        $mandatories = $this->newQuery();
        $mandatories->join('student_year as sty', function($join)use($serial_std, $serial_scy) {
            $join->on('student.serial_std', '=', 'sty.serial_std')
                    ->where('sty.serial_scy', '=', $serial_scy)
                    ->where('sty.serial_std', '=', $serial_std)
                    ->where('sty.status_sty', '<>', 'RETIRED');
        });
        $mandatories->join('student_by_course as stc', function($join) {
            $join->on('student.serial_std', '=', 'stc.serial_std')
                    ->where('status_stc', '=', 'ACTIVE');
        });
        $mandatories->join('course as crs', function($join) {
            $join->on('sty.serial_scy', '=', 'crs.serial_scy')
                    ->on('stc.serial_crs', '=', 'crs.serial_crs')
                    ->where('status_crs', '=', 'ACTIVE');
        });
        $mandatories->join('subject_course as sbc', function($join) {
            $join->on('sbc.serial_crs', '=', 'crs.serial_crs')
                    ->where('status_sbc', '=', 'ACTIVE')
                    ->where('sbc.has_grades', '=', 1)
                    ->where('sbc.has_opt_subject_relation', '=', 0);
        });
        $mandatories->join('subject as sbj', function($join) {
            $join->on('sbj.serial_sbj', '=', 'sbc.serial_sbj')
                    ->where('status_sbj', '=', 'ACTIVE');
        });
        $mandatories->join('cat_subject as csb', function($join) {
            $join->on('csb.serial_csb', '=', 'sbj.serial_csb')
                    ->where('status_csb', '=', 'ACTIVE');
        });
        foreach ($periods as $period) {
            $mandatories->leftjoin('student_subject_period_average as ' . 'stgm' . $period->serial_prd, function($join)use($period, $serial_std) {
                $join
                        ->on('stgm' . $period->serial_prd . '.serial_sbc', '=', 'sbc.serial_sbc')
                        ->on('stgm' . $period->serial_prd . '.serial_std', '=', 'student.serial_std')
                        ->where('stgm' . $period->serial_prd . '.serial_prd', '=', $period->serial_prd);
            });
        }
        $mandatories->leftjoin('student_subject_year_average AS sya', function($join) use($serial_scy) {
            $join->on('student.serial_std', '=', 'sya.serial_std')
                    ->on('sbc.serial_sbc', '=', 'sya.serial_sbc');
        });
        $mandatories->selectRaw('IF(csb.short_name_csb is null,csb.name_csb,csb.short_name_csb) as "MATERIA"');
        foreach ($periods as $period) {
            $namePeriod = explode(" ", $period->name_prd);
            $periodAcronym = "";

            foreach ($namePeriod as $name) {
                $periodAcronym .= $name[0];
            }
            if (strpos($period->options_prd, 'SHOW_GRADES_IN_PARENTS_REPORT') !== false) {
                $mandatories->selectRaw('IF(stgm' . $period->serial_prd . '.average_ssa is null,null,round(stgm' . $period->serial_prd . '.average_ssa,2)) as "' . $periodAcronym . '"');
            } else {
                $mandatories->selectRaw(' "--" as "' . $periodAcronym . '"');
            }
        }

        if (strpos($period->options_pgy, 'SHOW_GRADES_IN_PARENTS_REPORT') !== false) {
            $mandatories->selectRaw('IF(average_sya is null,null,round(average_sya,2)) as PR');
            $mandatories->selectRaw('IF(final_grade_sya is null,null,round(final_grade_sya,2)) as PF');
        } else {
            $mandatories->selectRaw('"--" as PR');
            $mandatories->selectRaw('"--" as PF');
        }
        $mandatories->selectRaw('sbc.serial_sbc as "SERIAL"');
        $mandatories->selectRaw('sbc.grade_type_sbc as "GRADETYPE"');
        $mandatories->selectRaw('"MANDATORY" as "TYPE"');
        $subjects = $mandatories->union($optatives);
        return $this->doQueryGet($subjects);
    }

    public function getSubjectsByPeriod($serial_prd, $serial_std) {
        $period = \App\Models\Period::find($serial_prd);
        $periodGroupYear = $period->periodGroupYear;
        $optatives = $this->newQuery();
        $optatives->join('student_year as sty', function($join)use($periodGroupYear) {
            $join->on('sty.serial_std', '=', 'student.serial_std')
                    ->where('sty.serial_scy', '=', $periodGroupYear->serial_scy)
                    ->where('sty.status_sty', '<>', 'RETIRED');
        });
        $optatives->join('student_by_course as stc', function($join) {
            $join->on('stc.serial_std', '=', 'student.serial_std')
                    ->where('status_stc', '=', 'ACTIVE');
        });
        $optatives->join('course as crs', function($join) {
            $join->on('stc.serial_crs', '=', 'crs.serial_crs')
                    ->where('status_crs', '=', 'ACTIVE');
        });
        $optatives->join('student_optional_subject as sos', function($join)use($serial_prd) {
            $join->on('student.serial_std', '=', 'sos.serial_std')
                    ->where('status_sos', '=', 'ACTIVE')
                    ->where('sos.serial_prd', '=', $serial_prd);
        });
        $optatives->join('optional_subject_level as osl', function($join) {
            $join->on('sos.serial_osb', '=', 'osl.serial_osb')
                    ->on('osl.serial_scy', '=', 'crs.serial_scy')
                    ->where('status_osl', '=', 'ACTIVE')
                    ->where('osl.has_grades', '=', 1);
        });
        $optatives->join('optional_subject as osb', function($join) {
            $join->on('sos.serial_osb', '=', 'osb.serial_osb')
                    ->where('status_osb', '=', 'ACTIVE');
        });
        $optatives->leftjoin('optional_with_subject_course as owsc', function($leftjoin) {
            $leftjoin->on('owsc.serial_osb', '=', 'osb.serial_osb')
                    ->where('status_owsc', '=', 'ACTIVE');
        });
        $optatives->leftjoin('subject_course as sbc', function($leftjoin) {
            $leftjoin->on('sbc.serial_sbc', '=', 'owsc.serial_sbc')
                ->where('status_sbc', '=', 'ACTIVE')
                ->where('sbc.has_grades', '=', 1);
        });
        $optatives->leftjoin('subject_course_period as scp', function ($leftjoin) {
            $leftjoin->on('scp.serial_sbc', '=', 'sbc.serial_sbc')
                ->where('status_scp', '=', 'ACTIVE');
        });
        $optatives->join('group_optional_subject as gos', function($join) {
            $join->on('osb.serial_osb', '=', 'gos.serial_osb')
                    ->on('sos.serial_lop', '=', 'gos.serial_lop')
                    ->where('status_gos', '=', 'ACTIVE');
        });
        $optatives->join('cat_subject as csb', function($join) {
            $join->on('osb.serial_csb', '=', 'csb.serial_csb')
                    ->where('status_csb', '=', 'ACTIVE');
        });
        $optatives->leftjoin('subject_category as scat', function($join) {
            $join->on('osl.serial_scat', '=', 'scat.serial_scat')
                    ->where('status_scat', '=', 'ACTIVE');
        });

        foreach ($period->subperiods as $subperiod) {
            $optatives->leftjoin('student_subject_subperiod_average as ' . 'stgo' . $subperiod->serial_sbp, function($join)use($subperiod, $serial_std) {
                $join->on('stgo' . $subperiod->serial_sbp . '.serial_lop', '=', 'sos.serial_lop')
                        ->on('stgo' . $subperiod->serial_sbp . '.serial_std', '=', 'student.serial_std')
                        ->where('stgo' . $subperiod->serial_sbp . '.serial_sbp', '=', $subperiod->serial_sbp);
            });
        }
        $optatives->join('year_period_exam_setup as ype', function($join) {
            $join->on('crs.serial_scy', '=', 'ype.serial_scy')
                    ->on('sos.serial_prd', '=', 'ype.serial_prd')
                    ->on('crs.serial_lvl', '=', 'ype.serial_lvl')
                    ->where('ype.status_ype', '=', 'ACTIVE');
        });
        $optatives->join('subject_period_exam as spx', function($join) {
            $join->on('osb.serial_osb', '=', 'spx.serial_osb')
                    ->on('spx.serial_ype', '=', 'ype.serial_ype')
                    ->where('spx.status_spx', '=', 'ACTIVE');
        });
        $optatives->leftjoin('student_period_exam as stx', function($join) {
            $join->on('student.serial_std', '=', 'stx.serial_std')
                    ->on('spx.serial_spx', '=', 'stx.serial_spx')
                    ->where('stx.status_stx', '=', 'ACTIVE');
        });
        $optatives->leftjoin('student_subject_period_average as ssa', function($join) {
            $join->on('student.serial_std', '=', 'ssa.serial_std')
                    ->on('sos.serial_lop', '=', 'ssa.serial_lop')
                    ->on('ssa.serial_prd', '=', 'sos.serial_prd');
        });
        $optatives->where('student.serial_std', '=', $serial_std);
        $optatives->selectRaw('CONCAT(IF(csb.short_name_csb is null,IF(scat.name_scat is null,csb.name_csb,scat.name_scat),csb.short_name_csb)," ",IF(scp.weight_scp is not null AND scp.weight_scp<100,CONCAT("(",scp.weight_scp,"%)"),"")) as "MATERIA"');
        foreach ($period->subperiods as $subperiod) {
            $nameSubperiod = explode(" ", $subperiod->name_sbp);
            $subperiodAcronym = "";

            foreach ($nameSubperiod as $name) {
                $subperiodAcronym .= $name[0];
            }
            if (strpos($subperiod->options_sbp, 'SHOW_GRADES_IN_PARENTS_REPORT') !== false) {
                $optatives->selectRaw('IF(stgo' . $subperiod->serial_sbp . '.average_sss is null,null,round(stgo' . $subperiod->serial_sbp . '.average_sss,2)) as "' . $subperiodAcronym . '"');
            } else {
                $optatives->selectRaw(' "--" as "' . $subperiodAcronym . '"');
            }
        }
        if (strpos($period->options_prd, 'SHOW_GRADES_IN_PARENTS_REPORT') !== false) {
            $optatives->selectRaw('IF(stx.grade_stx is null,null,truncate(stx.grade_stx,2)) as EX');
            $optatives->selectRaw('IF(ssa.average_ssa is null,null,truncate(ssa.average_ssa,2)) as "PROM"');
        } else {
            $optatives->selectRaw('"--" as EX');
            $optatives->selectRaw('"--" as "PROM"');
        }
        $optatives->selectRaw('IF(stx.grade_stx is null,null,truncate(stx.grade_stx,2)) as EX');
        $optatives->selectRaw('IF(ssa.average_ssa is null,null,truncate(ssa.average_ssa,2)) as "PR"');
        $optatives->selectRaw('osb.serial_osb as "SERIAL"');
        $optatives->selectRaw('osb.grade_type_sbc as "GRADETYPE"');
        $optatives->selectRaw('"OPTATIVE" as "TYPE"');

        $mandatories = $this->newQuery();
        $mandatories->join('student_year as sty', function($join)use($serial_std, $periodGroupYear) {
            $join->on('student.serial_std', '=', 'sty.serial_std')
                    ->where('sty.serial_scy', '=', $periodGroupYear->serial_scy)
                    ->where('sty.serial_std', '=', $serial_std)
                    ->where('sty.status_sty', '<>', 'RETIRED');
        });
        $mandatories->join('student_by_course as stc', function($join) {
            $join->on('student.serial_std', '=', 'stc.serial_std')
                    ->where('status_stc', '=', 'ACTIVE');
        });
        $mandatories->join('course as crs', function($join) {
            $join->on('sty.serial_scy', '=', 'crs.serial_scy')
                    ->on('stc.serial_crs', '=', 'crs.serial_crs')
                    ->where('status_crs', '=', 'ACTIVE');
        });
        $mandatories->join('subject_course as sbc', function($join) {
            $join->on('sbc.serial_crs', '=', 'crs.serial_crs')
                    ->where('status_sbc', '=', 'ACTIVE')
                    ->where('sbc.has_grades', '=', 1)
                    ->where('sbc.has_opt_subject_relation', '=', 0);
        });

        $mandatories->join('subject_course_period as scp', function ($join) {
            $join->on('scp.serial_sbc', '=', 'sbc.serial_sbc')
                ->where('status_scp', '=', 'ACTIVE');
        });
        $mandatories->join('subject as sbj', function($join) {
            $join->on('sbj.serial_sbj', '=', 'sbc.serial_sbj')
                    ->where('status_sbj', '=', 'ACTIVE');
        });
        $mandatories->join('cat_subject as csb', function($join) {
            $join->on('csb.serial_csb', '=', 'sbj.serial_csb')
                    ->where('status_csb', '=', 'ACTIVE');
        });
        foreach ($period->subperiods as $subperiod) {
            $mandatories->leftjoin('student_subject_subperiod_average as ' . 'stgm' . $subperiod->serial_sbp, function($join)use($subperiod, $serial_std) {
                $join
                        ->on('stgm' . $subperiod->serial_sbp . '.serial_sbc', '=', 'sbc.serial_sbc')
                        ->on('stgm' . $subperiod->serial_sbp . '.serial_std', '=', 'student.serial_std')
                        ->where('stgm' . $subperiod->serial_sbp . '.serial_sbp', '=', $subperiod->serial_sbp);
            });
        }
        $mandatories->join('year_period_exam_setup as ype', function ($join)use($serial_prd) {
            $join->on('crs.serial_scy', '=', 'ype.serial_scy')
                    ->on('crs.serial_lvl', '=', 'ype.serial_lvl')
                    ->where('ype.serial_prd', '=', $serial_prd)
                    ->where('ype.status_ype', '=', 'ACTIVE');
        });
        $mandatories->join('subject_period_exam as spx', function($join) {
            $join->on('sbc.serial_sbc', '=', 'spx.serial_sbc')
                    ->on('spx.serial_ype', '=', 'ype.serial_ype')
                    ->where('spx.status_spx', '=', 'ACTIVE');
        });
        $mandatories->leftjoin('student_period_exam as stx', function($join) {
            $join->on('student.serial_std', '=', 'stx.serial_std')
                    ->on('spx.serial_spx', '=', 'stx.serial_spx')
                    ->where('stx.status_stx', '=', 'ACTIVE');
        });
        $mandatories->leftjoin('student_subject_period_average as ssa', function($join) {
            $join->on('student.serial_std', '=', 'ssa.serial_std')
                    ->on('sbc.serial_sbc', '=', 'ssa.serial_sbc')
                    ->on('ssa.serial_prd', '=', 'ype.serial_prd');
        });
        $mandatories->selectRaw('CONCAT(IF(csb.short_name_csb is null,csb.name_csb,csb.short_name_csb)," ",IF(scp.weight_scp<100,CONCAT("(",scp.weight_scp,"%)"),"")) as "MATERIA"');
        foreach ($period->subperiods as $subperiod) {
            $nameSubperiod = explode(" ", $subperiod->name_sbp);
            $subperiodAcronym = "";

            foreach ($nameSubperiod as $name) {
                $subperiodAcronym .= $name[0];
            }
            if (strpos($subperiod->options_sbp, 'SHOW_GRADES_IN_PARENTS_REPORT') !== false) {
                $mandatories->selectRaw('IF(stgm' . $subperiod->serial_sbp . '.average_sss is null,null,round(stgm' . $subperiod->serial_sbp . '.average_sss,2)) as "' . $subperiodAcronym . '"');
            } else {
                $mandatories->selectRaw('"--" as "' . $subperiodAcronym . '"');
            }
        }
        if (strpos($period->options_prd, 'SHOW_GRADES_IN_PARENTS_REPORT') !== false) {
            $mandatories->selectRaw('IF(stx.grade_stx is null,null,truncate(stx.grade_stx,2)) as EX');
            $mandatories->selectRaw('IF(ssa.average_ssa is null,null,truncate(ssa.average_ssa,2)) as "PROM"');
        } else {
            $mandatories->selectRaw('"--" as EX');
            $mandatories->selectRaw('"--" as "PROM"');
        }
        $mandatories->selectRaw('IF(stx.grade_stx is null,null,truncate(stx.grade_stx,2)) as EX');
        $mandatories->selectRaw('IF(ssa.average_ssa is null,null,truncate(ssa.average_ssa,2)) as "PR"');
        $mandatories->selectRaw('sbc.serial_sbc as "SERIAL"');
        $mandatories->selectRaw('sbc.grade_type_sbc as "GRADETYPE"');
        $mandatories->selectRaw('"MANDATORY" as "TYPE"');
        $subjects = $mandatories->union($optatives);
        return $this->doQueryGet($subjects);
    }

    /**
     * Método que devuelve representantes por estudiante
     * @since   2018-07-24  
     * @param   integer serial_std serial_std
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getGuardiansBySerialStd($serial_std) {
        $query = $this->newQuery();
        $query->join('student_guardian as sg', 'sg.serial_std', '=', 'student.serial_std');
        $query->join('guardian as g', 'g.serial_grd', '=', 'sg.serial_grd');
        $query->where('student.serial_std', '=', $serial_std);
        $query->where('student.status_std', '=', 'ACTIVE');
        $query->where('sg.status_stg', '=', 'ACTIVE');
        $query->where('g.status_grd', '=', 'ACTIVE');

        $query->selectRaw("
          g.serial_grd,g.first_name_grd,g.last_name_grd,g.email_grd,g.mobile_grd
          ");

        return $this->doQueryGet($query);
    }

}
