<?php

namespace App\Repositories;

use App\Models\SubperiodDiscipline;
use App\Repositories\BaseRepository;

/**
 * Repostiorio para disciplina por subperiodo
 * @since   2018-03-28
 * @author  Erick Vaca <epvaca@rutatec.com>
 *
 * @edit    2018-03-28<br />
 *          Erick Vaca <epvaca@rutatec.com><br /><br/>
 *          Creación del archivo<br/>
 *          #edit1
 */
class SubperiodDisciplineRepository extends BaseRepository {

    protected $modelClass = SubperiodDiscipline::class;

    //Declaración del constructor
    public function __construct(SubperiodDiscipline $subperiodDiscipline) {
        $this->subperiodDiscipline = $subperiodDiscipline;
    }
  
     /**
     * Método que devuelve la disciplina de un estudiante por subperiodo
     * @since   2018-03-28
     * @author  Erick Vaca <epvaca@rutatec.com> 
     * @param   array data
     * @return  array
     *
     */
    public function getStudentSubperiodDiscipline($data) {

        $query = $this->newQuery();

        $query->where('serial_sbp', $data['serial_sbp']);
        $query->where('serial_std', $data['serial_std']);
        $query->where('serial_spt', $data['serial_spt']);
        $query->where('serial_lvl', $data['serial_lvl']);
        $query->where('serial_scy', $data['serial_scy']);
        $query->where('serial_set', $data['serial_set']); 
                       
        return $this->doQueryFirst($query);
    }


    /**
     * Método que guarda subperiod discipline
     * @since   2018-03-28
     * @author  Erick Vaca <epvaca@rutatec.com> 
     *
     * @param   array $data
     * @return  
     * 
     */
    public function saveSubperiodDiscipline($data) {

        //se realiza la búsqueda para determinar si existe ya registrada la categoria padre
        $subDis = $this->getStudentSubperiodDiscipline($data);

         //se actualiza si se encontró ya una categoria padre registrada
        if ($subDis == null) { 
            //Crea nuevo registro
            $subDis = new SubperiodDiscipline;

            $subDis->serial_sbp = $data['serial_sbp'];        
            $subDis->serial_std = $data['serial_std'];                   
            $subDis->serial_spt = $data['serial_spt'];
            $subDis->serial_lvl = $data['serial_lvl'];
            $subDis->serial_scy = $data['serial_scy'];
            $subDis->serial_set = $data['serial_set'];
            $subDis->supervisor_sbp_grade_dsc = $data['supervisor_sbp_grade_dsc'];
            $subDis->subjects_sbp_grade_dsc = $data['subjects_sbp_grade_dsc'];
            $subDis->course_director_sbp_grade_dsc = $data['course_director_sbp_grade_dsc'];
            $subDis->general_sbp_grade_dsc = $data['general_sbp_grade_dsc'];
            $subDis->adjustment_sbp_dsc  = $data['adjustment_sbp_dsc'];
            $subDis->save();        
        }else{
            $subDis->supervisor_sbp_grade_dsc = $data['supervisor_sbp_grade_dsc'];
            $subDis->subjects_sbp_grade_dsc = $data['subjects_sbp_grade_dsc'];
            $subDis->course_director_sbp_grade_dsc = $data['course_director_sbp_grade_dsc'];
            $subDis->general_sbp_grade_dsc = $data['general_sbp_grade_dsc'];
            $subDis->adjustment_sbp_dsc  = $data['adjustment_sbp_dsc'];             
            $subDis->save();
        }
        return $subDis;        
    }

}
