<?php

namespace App\Repositories;

use App\Models\User;
use App\Repositories\BaseRepository;
use App\Repositories\StudentYearRepository;
use App\Repositories\SchoolYearRepository;

/**
 * Repositorio del usuario
 * @author Fernando Salas <rfsalas@rutatec.com>
 * @since  2017-05-19
 */
class UserRepository extends BaseRepository {

    protected $modelClass = User::class;
    protected $user;
    protected $studentYear;
    protected $schoolYear;
    protected $image_path = 'img/users/';

    //Declaración del constructor
    public function __construct(User $user, StudentYearRepository $studentYear, SchoolYearRepository $schoolYear) {
        $this->user = $user;
        $this->studentYear = $studentYear;
        $this->schoolYear = $schoolYear;
    }

    public function getUser($credentials) {
        $usr = $this->newQuery();
        $usr->where('username_usr', '=', $credentials['user']);
        $usr->where('password_usr', '=', md5($credentials['password']));
        $usr->where('status_usr', 'ACTIVE');
        $user = $this->doQueryFirst($usr);
        if ($user != null) {
            $schoolYear = $this->schoolYear->getCurrentSchoolYear();
            $studentsInCurrentSchoolYear = array();
            if ($user->serial_std != null) {
                $studentYear = $this->studentYear->getStudentYear($user->serial_std, $schoolYear->serial_scy);
                if ($studentYear != null) {
                    return $user;
                }
                return false;
            } elseif ($user->serial_grd != null) {
                $guardian = \App\Models\Guardian::find($user->serial_grd);
                $students = $guardian->students;
                foreach ($students as $student) {
                    $studentInYear = $this->studentYear->getStudentYear($student->serial_std, $schoolYear->serial_scy);
                    if ($studentInYear != null) {
                        array_push($studentsInCurrentSchoolYear, $studentInYear);
                    }
                }
                if (sizeof($studentsInCurrentSchoolYear) > 0) {
                    return $user;
                } elseif ($user->serial_emp != null){
                    return $user;
                } else {
                    return false;
                }
            } else {
                return $user;
            }
        }
        return false;
    }

    public function getImageUser($serial_usr) {
        $schoolSetup = \App\Models\SchoolSetup::first();
        $image = $this->newQuery();
        $image->where('status_usr', '=', 'ACTIVE');
        $image->where('serial_usr', '=', $serial_usr);
        $img = $this->doQueryFirst($image);
        $photo_usr = env('SERVER_LEARNBOX') . $this->image_path;
        if ($img->photo_usr != null) {
            return $photo_usr . $img->photo_usr;
        }
        return $photo_usr . 'noavatar.png';
    }



    /**
     * Método que devuelve usuario  por serial_usr
     * @since   2018-04-04     
     * @param   integer serial_usr serial_usr
     * @author   Erick Vaca <epvaca@rutatec.com><br /><br/>
     * @return  json resultado
     *
     */
    public function getUserBySerialusr($serial_usr) {
        $query = $this->newQuery();
        $query->where('serial_usr', '=', $serial_usr);                
        return $this->doQueryFirst($query);
    }

    /**
     * Método para obtener los docentes asignados a una sección
     * por año lectivo y serial de usuario, si el usuario no tiene
     * asignado una sección, se obtiene los datos del usuario y TODAS
     * las secciones del año lectivo
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-28
     * @return $employeeSection
     */
    public function getEmployeeSection($data) {
        $employeeSection = User::where('user.serial_usr','=',$data['serial_usr'])
            ->where('status_usr','=','ACTIVE')
            ->leftJoin('employee as emp','emp.serial_usr','=','user.serial_usr')
            ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','emp.serial_emp')
            ->leftJoin('school_year as scy','scy.serial_scy','=','emp_set.serial_scy')
            ->leftJoin('section as set','set.serial_set','=','emp_set.serial_set')
            ->leftJoin('course_employee as cem','cem.serial_emp','=','emp.serial_emp')
            ->where('emp_set.serial_scy','=',$data['serial_scy'])
            ->where('emp_set.status','=','ACTIVE')
            ->select('set.serial_set','set.name_set')
            ->distinct()
            ->pluck('set.name_set','set.serial_set');
        if (count($employeeSection)) {
            $employeeSection->prepend('TODOS', 'all');
            return $employeeSection;
        } else {
            $employeeSection = User::where('user.serial_usr','=',$data['serial_usr'])
                ->where('status_usr','=','ACTIVE')
                ->leftJoin('employee as emp','emp.serial_usr','=','user.serial_usr')
                ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','emp.serial_emp')
                ->join('level_section_year as pgy','pgy.serial_scy','=','pgy.serial_scy')
                ->where('pgy.serial_scy','=',$data['serial_scy'])
                ->join('school_year as scy','scy.serial_scy','=','pgy.serial_scy')
                ->join('section as set','set.serial_set','=','pgy.serial_set')
                ->leftJoin('course_employee as cem','cem.serial_emp','=','emp.serial_emp')
                ->select('set.serial_set','set.name_set')
                ->distinct()
                ->pluck('set.name_set','set.serial_set');
            $employeeSection->prepend('TODOS', 'all');
            return $employeeSection;
        }
    }

    /**
     * Método para obtener los docentes asignados a un nivel
     * por sección, año lectivo y usuario
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-29
     * @return $employeeLevel
     */

    public function getEmployeeLevel($data) {
        $employeeLevel = User::where('user.serial_usr','=',$data['serial_usr'])
            ->where('status_usr','=','ACTIVE')
            ->leftJoin('employee as emp','emp.serial_usr','=','user.serial_usr')
            ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','emp.serial_emp')
            ->leftJoin('school_year as scy','scy.serial_scy','=','emp_set.serial_scy')
            ->leftJoin('section as set','set.serial_set','=','emp_set.serial_set')
            ->leftJoin('level as lvl','lvl.serial_lvl','=','emp_set.serial_lvl')
            ->leftJoin('course_employee as cem','cem.serial_emp','=','emp.serial_emp')
            ->where('emp_set.serial_scy','=',$data['serial_scy'])
            ->where('emp_set.serial_scy','=',$data['serial_set'])
            ->where('emp_set.status','=','ACTIVE')
            ->select('lvl.serial_lvl','lvl.name_lvl')
            ->distinct()
            ->pluck('lvl.name_lvl','lvl.serial_lvl');
        if (count($employeeLevel)) {
            $employeeLevel->prepend('TODOS', 'all');
            return $employeeLevel;
        } else {
            $employeeLevel = User::where('user.serial_usr','=',$data['serial_usr'])
                ->where('status_usr','=','ACTIVE')
                ->leftJoin('employee as emp','emp.serial_usr','=','user.serial_usr')
                ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','emp.serial_emp')
                ->join('level_section_year as pgy','pgy.serial_scy','=','pgy.serial_scy')
                ->where('pgy.serial_scy','=',$data['serial_scy'])
                ->where('pgy.serial_set','=',$data['serial_set'])
                ->join('school_year as scy','scy.serial_scy','=','pgy.serial_scy')
                ->join('section as set','set.serial_set','=','pgy.serial_set')
                ->join('level as lvl','lvl.serial_lvl','=','pgy.serial_lvl')
                ->leftJoin('course_employee as cem','cem.serial_emp','=','emp.serial_emp')
                ->select('lvl.serial_lvl','lvl.name_lvl')
                ->distinct()
                ->pluck('lvl.name_lvl','lvl.serial_lvl');
            $employeeLevel->prepend('TODOS', 'all');
            return $employeeLevel;
        }
    }

    /**
     * Método para obtener todos los aplicantes de todas
     * las secciones por año lectivo
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-30
     * @return $employeeSection
     */

    public function getAllSections($data) {
        $employeeSection = User::where('user.serial_usr','=',$data['serial_usr'])
            ->where('status_usr','=','ACTIVE')
            ->leftJoin('employee as emp','emp.serial_usr','=','user.serial_usr')
            ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','emp.serial_emp')
            ->leftJoin('school_year as scy','scy.serial_scy','=','emp_set.serial_scy')
            ->leftJoin('section as set','set.serial_set','=','emp_set.serial_set')
            ->leftJoin('course_employee as cem','cem.serial_emp','=','emp.serial_emp')
            ->where('emp_set.serial_scy','=',$data['serial_scy'])
            ->where('emp_set.status','=','ACTIVE')
            ->select('set.serial_set')
            ->distinct()
            ->pluck('set.serial_set');
        if (count($employeeSection)) {
            return $employeeSection;
        } else {
            $employeeSection = User::where('user.serial_usr','=',$data['serial_usr'])
                ->where('status_usr','=','ACTIVE')
                ->leftJoin('employee as emp','emp.serial_usr','=','user.serial_usr')
                ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','emp.serial_emp')
                ->join('level_section_year as pgy','pgy.serial_scy','=','pgy.serial_scy')
                ->where('pgy.serial_scy','=',$data['serial_scy'])
                ->join('school_year as scy','scy.serial_scy','=','pgy.serial_scy')
                ->join('section as set','set.serial_set','=','pgy.serial_set')
                ->leftJoin('course_employee as cem','cem.serial_emp','=','emp.serial_emp')
                ->select('set.serial_set')
                ->distinct()
                ->pluck('set.serial_set');
            return $employeeSection;
        }
    }

    /**
     * Método para obtener todos los aplicantes de todos
     * los niveles por seccion y año lectivo
     * @author Henry Vivanco <hvvivanco@rutatec.com>
     * @since  2019-01-30
     * @return $employeeSection
     */

    public function getAllLevels($data) {
        $employeeSection = User::where('user.serial_usr','=',$data['serial_usr'])
            ->where('status_usr','=','ACTIVE')
            ->leftJoin('employee as emp','emp.serial_usr','=','user.serial_usr')
            ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','emp.serial_emp')
            ->leftJoin('school_year as scy','scy.serial_scy','=','emp_set.serial_scy')
            ->leftJoin('section as set','set.serial_set','=','emp_set.serial_set')
            ->leftJoin('level as lvl','lvl.serial_lvl','=','emp_set.serial_lvl')
            ->leftJoin('course_employee as cem','cem.serial_emp','=','emp.serial_emp')
            ->where('emp_set.serial_scy','=',$data['serial_scy'])
            ->where('emp_set.serial_set','=',$data['serial_set'])
            ->where('emp_set.status','=','ACTIVE')
            ->select('lvl.serial_lvl')
            ->distinct()
            ->pluck('lvl.serial_lvl');
        if (count($employeeSection)) {
            return $employeeSection;
        } else {
            $employeeSection = User::where('user.serial_usr','=',$data['serial_usr'])
                ->where('status_usr','=','ACTIVE')
                ->leftJoin('employee as emp','emp.serial_usr','=','user.serial_usr')
                ->leftJoin('employee_section as emp_set','emp_set.serial_emp','=','emp.serial_emp')
                ->join('level_section_year as pgy','pgy.serial_scy','=','pgy.serial_scy')
                ->where('pgy.serial_scy','=',$data['serial_scy'])
                ->where('pgy.serial_set','=',$data['serial_set'])
                ->join('school_year as scy','scy.serial_scy','=','pgy.serial_scy')
                ->join('section as set','set.serial_set','=','pgy.serial_set')
                ->join('level as lvl','lvl.serial_lvl','=','pgy.serial_lvl')
                ->leftJoin('course_employee as cem','cem.serial_emp','=','emp.serial_emp')
                ->select('lvl.serial_lvl')
                ->distinct()
                ->pluck('lvl.serial_lvl');
            return $employeeSection;
        }
    }


}
